/*
 * Interface wrapper code.
 *
 * Generated by SIP 6.16.1
 */

#include "sipAPIpictureflow.h"

#line 11 "/build/calibre/src/calibre-9.13.0/build/pyqt/pictureflow/pictureflow.sip"
#include <pictureflow.h>
#line 12 "/build/calibre/src/calibre-9.13.0/build/pyqt/pictureflow/build/pictureflow/sippictureflowFlowImages.cpp"

#line 24 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qstring.sip"
#include <qstring.h>
#line 16 "/build/calibre/src/calibre-9.13.0/build/pyqt/pictureflow/build/pictureflow/sippictureflowFlowImages.cpp"
#line 33 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qimage.sip"
#include <qimage.h>
#line 19 "/build/calibre/src/calibre-9.13.0/build/pyqt/pictureflow/build/pictureflow/sippictureflowFlowImages.cpp"
#line 26 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qobjectdefs.sip"
#include <qobjectdefs.h>
#line 154 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qobjectdefs.sip"
#include <qobjectdefs.h>
#line 24 "/build/calibre/src/calibre-9.13.0/build/pyqt/pictureflow/build/pictureflow/sippictureflowFlowImages.cpp"
#line 26 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qmetaobject.sip"
#include <qmetaobject.h>
#line 27 "/build/calibre/src/calibre-9.13.0/build/pyqt/pictureflow/build/pictureflow/sippictureflowFlowImages.cpp"
#line 26 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qcoreevent.sip"
#include <qcoreevent.h>
#line 30 "/build/calibre/src/calibre-9.13.0/build/pyqt/pictureflow/build/pictureflow/sippictureflowFlowImages.cpp"
#line 277 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qcoreevent.sip"
#include <qcoreevent.h>
#line 33 "/build/calibre/src/calibre-9.13.0/build/pyqt/pictureflow/build/pictureflow/sippictureflowFlowImages.cpp"
#line 254 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qcoreevent.sip"
#include <qcoreevent.h>
#line 36 "/build/calibre/src/calibre-9.13.0/build/pyqt/pictureflow/build/pictureflow/sippictureflowFlowImages.cpp"
#line 28 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qobject.sip"
#include <qobject.h>
#line 39 "/build/calibre/src/calibre-9.13.0/build/pyqt/pictureflow/build/pictureflow/sippictureflowFlowImages.cpp"
#line 26 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qvariant.sip"
#include <qvariant.h>
#line 42 "/build/calibre/src/calibre-9.13.0/build/pyqt/pictureflow/build/pictureflow/sippictureflowFlowImages.cpp"
#line 28 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qpycore_qlist.sip"
#include <qlist.h>
#line 45 "/build/calibre/src/calibre-9.13.0/build/pyqt/pictureflow/build/pictureflow/sippictureflowFlowImages.cpp"
#line 30 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qbytearray.sip"
#include <qbytearray.h>
#line 48 "/build/calibre/src/calibre-9.13.0/build/pyqt/pictureflow/build/pictureflow/sippictureflowFlowImages.cpp"
#line 135 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qpycore_qlist.sip"
#include <qlist.h>
#line 51 "/build/calibre/src/calibre-9.13.0/build/pyqt/pictureflow/build/pictureflow/sippictureflowFlowImages.cpp"
#line 26 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qnamespace.sip"
#include <qnamespace.h>
#line 154 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qtextstream.sip"
#include <qtextstream.h>
#line 56 "/build/calibre/src/calibre-9.13.0/build/pyqt/pictureflow/build/pictureflow/sippictureflowFlowImages.cpp"
#line 26 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qthread.sip"
#include <qthread.h>
#line 59 "/build/calibre/src/calibre-9.13.0/build/pyqt/pictureflow/build/pictureflow/sippictureflowFlowImages.cpp"
#line 26 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qanystringview.sip"
#include <qanystringview.h>
#line 62 "/build/calibre/src/calibre-9.13.0/build/pyqt/pictureflow/build/pictureflow/sippictureflowFlowImages.cpp"
#line 25 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qflags.sip"
#include <qglobal.h>
#line 65 "/build/calibre/src/calibre-9.13.0/build/pyqt/pictureflow/build/pictureflow/sippictureflowFlowImages.cpp"
#line 26 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qregularexpression.sip"
#include <qregularexpression.h>
#line 68 "/build/calibre/src/calibre-9.13.0/build/pyqt/pictureflow/build/pictureflow/sippictureflowFlowImages.cpp"
#line 26 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qobjectdefs.sip"
#include <qobjectdefs.h>
#line 71 "/build/calibre/src/calibre-9.13.0/build/pyqt/pictureflow/build/pictureflow/sippictureflowFlowImages.cpp"


class sipFlowImages : public ::FlowImages
{
public:
    sipFlowImages();
    virtual ~sipFlowImages();

    int qt_metacall(QMetaObject::Call, int, void **) SIP_OVERRIDE;
    void *qt_metacast(const char *) SIP_OVERRIDE;
    const QMetaObject *metaObject() const SIP_OVERRIDE;

    /*
     * There is a protected method for every virtual method visible from
     * this class.
     */
protected:
    ::QString subtitle(int) SIP_OVERRIDE;
    ::QString caption(int) SIP_OVERRIDE;
    ::QImage image(int) SIP_OVERRIDE;
    int count() SIP_OVERRIDE;
    bool event(::QEvent*) SIP_OVERRIDE;
    bool eventFilter(::QObject*, ::QEvent*) SIP_OVERRIDE;
    void timerEvent(::QTimerEvent*) SIP_OVERRIDE;
    void childEvent(::QChildEvent*) SIP_OVERRIDE;
    void customEvent(::QEvent*) SIP_OVERRIDE;
    void connectNotify(const ::QMetaMethod&) SIP_OVERRIDE;
    void disconnectNotify(const ::QMetaMethod&) SIP_OVERRIDE;

public:
    sipSimpleWrapper *sipPySelf;

private:
    sipFlowImages(const sipFlowImages &);
    sipFlowImages &operator = (const sipFlowImages &);

    char sipPyMethods[11];
};

sipFlowImages::sipFlowImages(): ::FlowImages(), sipPySelf(SIP_NULLPTR)
{
    memset(sipPyMethods, 0, sizeof (sipPyMethods));
}

sipFlowImages::~sipFlowImages()
{
    sipInstanceDestroyedEx(&sipPySelf);
}

const QMetaObject *sipFlowImages::metaObject() const
{
    if (sipGetInterpreter())
        return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : sip_pictureflow_qt_metaobject(sipPySelf, sipType_FlowImages);

    return ::FlowImages::metaObject();
}

int sipFlowImages::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ::FlowImages::qt_metacall(_c, _id, _a);

    if (_id >= 0)
    {
        SIP_BLOCK_THREADS
        _id = sip_pictureflow_qt_metacall(sipPySelf, sipType_FlowImages, _c, _id, _a);
        SIP_UNBLOCK_THREADS
    }

    return _id;
}

void *sipFlowImages::qt_metacast(const char *_clname)
{
    void *sipCpp;

    return (sip_pictureflow_qt_metacast(sipPySelf, sipType_FlowImages, _clname, &sipCpp) ? sipCpp : ::FlowImages::qt_metacast(_clname));
}

::QString sipFlowImages::subtitle(int a0)
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, &sipPyMethods[0], &sipPySelf, SIP_NULLPTR, sipName_subtitle);

    if (!sipMeth)
        return ::FlowImages::subtitle(a0);

    extern ::QString sipVH_pictureflow_8(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *, int);

    return sipVH_pictureflow_8(sipGILState, sipImportedVirtErrorHandlers_pictureflow_QtCore[0].iveh_handler, sipPySelf, sipMeth, a0);
}

::QString sipFlowImages::caption(int a0)
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, &sipPyMethods[1], &sipPySelf, SIP_NULLPTR, sipName_caption);

    if (!sipMeth)
        return ::FlowImages::caption(a0);

    extern ::QString sipVH_pictureflow_8(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *, int);

    return sipVH_pictureflow_8(sipGILState, sipImportedVirtErrorHandlers_pictureflow_QtCore[0].iveh_handler, sipPySelf, sipMeth, a0);
}

::QImage sipFlowImages::image(int a0)
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, &sipPyMethods[2], &sipPySelf, SIP_NULLPTR, sipName_image);

    if (!sipMeth)
        return ::FlowImages::image(a0);

    extern ::QImage sipVH_pictureflow_7(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *, int);

    return sipVH_pictureflow_7(sipGILState, sipImportedVirtErrorHandlers_pictureflow_QtCore[0].iveh_handler, sipPySelf, sipMeth, a0);
}

int sipFlowImages::count()
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, &sipPyMethods[3], &sipPySelf, SIP_NULLPTR, sipName_count);

    if (!sipMeth)
        return ::FlowImages::count();

    extern int sipVH_pictureflow_6(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *);

    return sipVH_pictureflow_6(sipGILState, sipImportedVirtErrorHandlers_pictureflow_QtCore[0].iveh_handler, sipPySelf, sipMeth);
}

bool sipFlowImages::event(::QEvent*a0)
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, &sipPyMethods[4], &sipPySelf, SIP_NULLPTR, sipName_event);

    if (!sipMeth)
        return ::FlowImages::event(a0);

    extern bool sipVH_pictureflow_5(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *, ::QEvent*);

    return sipVH_pictureflow_5(sipGILState, sipImportedVirtErrorHandlers_pictureflow_QtCore[0].iveh_handler, sipPySelf, sipMeth, a0);
}

bool sipFlowImages::eventFilter(::QObject*a0, ::QEvent*a1)
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, &sipPyMethods[5], &sipPySelf, SIP_NULLPTR, sipName_eventFilter);

    if (!sipMeth)
        return ::FlowImages::eventFilter(a0, a1);

    extern bool sipVH_pictureflow_4(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *, ::QObject*, ::QEvent*);

    return sipVH_pictureflow_4(sipGILState, sipImportedVirtErrorHandlers_pictureflow_QtCore[0].iveh_handler, sipPySelf, sipMeth, a0, a1);
}

void sipFlowImages::timerEvent(::QTimerEvent*a0)
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, &sipPyMethods[6], &sipPySelf, SIP_NULLPTR, sipName_timerEvent);

    if (!sipMeth)
    {
        ::FlowImages::timerEvent(a0);
        return;
    }

    extern void sipVH_pictureflow_3(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *, ::QTimerEvent*);

    sipVH_pictureflow_3(sipGILState, sipImportedVirtErrorHandlers_pictureflow_QtCore[0].iveh_handler, sipPySelf, sipMeth, a0);
}

void sipFlowImages::childEvent(::QChildEvent*a0)
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, &sipPyMethods[7], &sipPySelf, SIP_NULLPTR, sipName_childEvent);

    if (!sipMeth)
    {
        ::FlowImages::childEvent(a0);
        return;
    }

    extern void sipVH_pictureflow_2(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *, ::QChildEvent*);

    sipVH_pictureflow_2(sipGILState, sipImportedVirtErrorHandlers_pictureflow_QtCore[0].iveh_handler, sipPySelf, sipMeth, a0);
}

void sipFlowImages::customEvent(::QEvent*a0)
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, &sipPyMethods[8], &sipPySelf, SIP_NULLPTR, sipName_customEvent);

    if (!sipMeth)
    {
        ::FlowImages::customEvent(a0);
        return;
    }

    extern void sipVH_pictureflow_1(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *, ::QEvent*);

    sipVH_pictureflow_1(sipGILState, sipImportedVirtErrorHandlers_pictureflow_QtCore[0].iveh_handler, sipPySelf, sipMeth, a0);
}

void sipFlowImages::connectNotify(const ::QMetaMethod& a0)
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, &sipPyMethods[9], &sipPySelf, SIP_NULLPTR, sipName_connectNotify);

    if (!sipMeth)
    {
        ::FlowImages::connectNotify(a0);
        return;
    }

    extern void sipVH_pictureflow_0(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *, const ::QMetaMethod&);

    sipVH_pictureflow_0(sipGILState, sipImportedVirtErrorHandlers_pictureflow_QtCore[0].iveh_handler, sipPySelf, sipMeth, a0);
}

void sipFlowImages::disconnectNotify(const ::QMetaMethod& a0)
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, &sipPyMethods[10], &sipPySelf, SIP_NULLPTR, sipName_disconnectNotify);

    if (!sipMeth)
    {
        ::FlowImages::disconnectNotify(a0);
        return;
    }

    extern void sipVH_pictureflow_0(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *, const ::QMetaMethod&);

    sipVH_pictureflow_0(sipGILState, sipImportedVirtErrorHandlers_pictureflow_QtCore[0].iveh_handler, sipPySelf, sipMeth, a0);
}


PyDoc_STRVAR(doc_FlowImages_count, "count(self) -> int");

extern "C" {static PyObject *meth_FlowImages_count(PyObject *, PyObject *);}
static PyObject *meth_FlowImages_count(PyObject *sipSelf, PyObject *sipArgs)
{
    PyObject *sipParseErr = SIP_NULLPTR;
    bool sipSelfWasArg = (!PyObject_TypeCheck(sipSelf, sipTypeAsPyTypeObject(sipType_FlowImages)) || sipIsDerivedClass((sipSimpleWrapper *)sipSelf));

    {
        ::FlowImages *sipCpp;

        if (sipParseArgs(&sipParseErr, sipArgs, "B", &sipSelf, sipType_FlowImages, &sipCpp))
        {
            int sipRes;

            sipRes = (sipSelfWasArg ? sipCpp->::FlowImages::count() : sipCpp->count());

            return PyLong_FromLong(sipRes);
        }
    }

    sipNoMethod(sipParseErr, sipName_FlowImages, sipName_count, doc_FlowImages_count);

    return SIP_NULLPTR;
}


PyDoc_STRVAR(doc_FlowImages_image, "image(self, index: int) -> QImage");

extern "C" {static PyObject *meth_FlowImages_image(PyObject *, PyObject *);}
static PyObject *meth_FlowImages_image(PyObject *sipSelf, PyObject *sipArgs)
{
    PyObject *sipParseErr = SIP_NULLPTR;
    bool sipSelfWasArg = (!PyObject_TypeCheck(sipSelf, sipTypeAsPyTypeObject(sipType_FlowImages)) || sipIsDerivedClass((sipSimpleWrapper *)sipSelf));

    {
        int a0;
        ::FlowImages *sipCpp;

        if (sipParseArgs(&sipParseErr, sipArgs, "Bi", &sipSelf, sipType_FlowImages, &sipCpp, &a0))
        {
            ::QImage*sipRes;

            sipRes = new ::QImage((sipSelfWasArg ? sipCpp->::FlowImages::image(a0) : sipCpp->image(a0)));

            return sipConvertFromNewType(sipRes, sipType_QImage, SIP_NULLPTR);
        }
    }

    sipNoMethod(sipParseErr, sipName_FlowImages, sipName_image, doc_FlowImages_image);

    return SIP_NULLPTR;
}


PyDoc_STRVAR(doc_FlowImages_caption, "caption(self, index: int) -> str");

extern "C" {static PyObject *meth_FlowImages_caption(PyObject *, PyObject *);}
static PyObject *meth_FlowImages_caption(PyObject *sipSelf, PyObject *sipArgs)
{
    PyObject *sipParseErr = SIP_NULLPTR;
    bool sipSelfWasArg = (!PyObject_TypeCheck(sipSelf, sipTypeAsPyTypeObject(sipType_FlowImages)) || sipIsDerivedClass((sipSimpleWrapper *)sipSelf));

    {
        int a0;
        ::FlowImages *sipCpp;

        if (sipParseArgs(&sipParseErr, sipArgs, "Bi", &sipSelf, sipType_FlowImages, &sipCpp, &a0))
        {
            ::QString*sipRes;

            sipRes = new ::QString((sipSelfWasArg ? sipCpp->::FlowImages::caption(a0) : sipCpp->caption(a0)));

            return sipConvertFromNewType(sipRes, sipType_QString, SIP_NULLPTR);
        }
    }

    sipNoMethod(sipParseErr, sipName_FlowImages, sipName_caption, doc_FlowImages_caption);

    return SIP_NULLPTR;
}


PyDoc_STRVAR(doc_FlowImages_subtitle, "subtitle(self, index: int) -> str");

extern "C" {static PyObject *meth_FlowImages_subtitle(PyObject *, PyObject *);}
static PyObject *meth_FlowImages_subtitle(PyObject *sipSelf, PyObject *sipArgs)
{
    PyObject *sipParseErr = SIP_NULLPTR;
    bool sipSelfWasArg = (!PyObject_TypeCheck(sipSelf, sipTypeAsPyTypeObject(sipType_FlowImages)) || sipIsDerivedClass((sipSimpleWrapper *)sipSelf));

    {
        int a0;
        ::FlowImages *sipCpp;

        if (sipParseArgs(&sipParseErr, sipArgs, "Bi", &sipSelf, sipType_FlowImages, &sipCpp, &a0))
        {
            ::QString*sipRes;

            sipRes = new ::QString((sipSelfWasArg ? sipCpp->::FlowImages::subtitle(a0) : sipCpp->subtitle(a0)));

            return sipConvertFromNewType(sipRes, sipType_QString, SIP_NULLPTR);
        }
    }

    sipNoMethod(sipParseErr, sipName_FlowImages, sipName_subtitle, doc_FlowImages_subtitle);

    return SIP_NULLPTR;
}


/* Cast a pointer to a type somewhere in its inheritance hierarchy. */
extern "C" {static void *cast_FlowImages(void *, const sipTypeDef *);}
static void *cast_FlowImages(void *sipCppV, const sipTypeDef *targetType)
{
    ::FlowImages *sipCpp = reinterpret_cast< ::FlowImages *>(sipCppV);

    if (targetType == sipType_FlowImages)
        return sipCppV;

    if (targetType == sipType_QObject)
        return static_cast< ::QObject *>(sipCpp);

    return SIP_NULLPTR;
}


/* Call the instance's destructor. */
extern "C" {static void release_FlowImages(void *, int);}
static void release_FlowImages(void *sipCppV, int)
{
    ::FlowImages *sipCpp = reinterpret_cast< ::FlowImages *>(sipCppV);

    if (QThread::currentThread() == sipCpp->thread())
        delete sipCpp;
    else
        sipCpp->deleteLater();
}


extern "C" {static void *array_FlowImages(Py_ssize_t);}
static void *array_FlowImages(Py_ssize_t sipNrElem)
{
    return new ::FlowImages[sipNrElem];
}


extern "C" {static void array_delete_FlowImages(void *);}
static void array_delete_FlowImages(void *sipCpp)
{
    delete[] reinterpret_cast< ::FlowImages *>(sipCpp);
}


extern "C" {static void dealloc_FlowImages(sipSimpleWrapper *);}
static void dealloc_FlowImages(sipSimpleWrapper *sipSelf)
{
    if (sipIsDerivedClass(sipSelf))
        reinterpret_cast<sipFlowImages *>(sipGetAddress(sipSelf))->sipPySelf = SIP_NULLPTR;

    if (sipIsOwnedByPython(sipSelf))
    {
        release_FlowImages(sipGetAddress(sipSelf), sipIsDerivedClass(sipSelf));
    }
}


extern "C" {static void *init_type_FlowImages(sipSimpleWrapper *, PyObject *, PyObject *, PyObject **, PyObject **, PyObject **);}
static void *init_type_FlowImages(sipSimpleWrapper *sipSelf, PyObject *sipArgs, PyObject *sipKwds, PyObject **sipUnused, PyObject **, PyObject **sipParseErr)
{
    sipFlowImages *sipCpp = SIP_NULLPTR;

    {
        if (sipParseKwdArgs(sipParseErr, sipArgs, sipKwds, SIP_NULLPTR, sipUnused, ""))
        {
            sipCpp = new sipFlowImages();

            sipCpp->sipPySelf = sipSelf;

            return sipCpp;
        }
    }

    return SIP_NULLPTR;
}


/* Define this type's super-types. */
static sipEncodedTypeDef supers_FlowImages[] = {{4, 0, 1}};


static PyMethodDef methods_FlowImages[] = {
    {sipName_caption, meth_FlowImages_caption, METH_VARARGS, doc_FlowImages_caption},
    {sipName_count, meth_FlowImages_count, METH_VARARGS, doc_FlowImages_count},
    {sipName_image, meth_FlowImages_image, METH_VARARGS, doc_FlowImages_image},
    {sipName_subtitle, meth_FlowImages_subtitle, METH_VARARGS, doc_FlowImages_subtitle}
};

PyDoc_STRVAR(doc_FlowImages, "\1FlowImages()");


/* Define this type's signals. */
static const pyqt6QtSignal signals_FlowImages[] = {
    {"dataChanged()", "\1dataChanged(self)", SIP_NULLPTR, SIP_NULLPTR},
    {SIP_NULLPTR, SIP_NULLPTR, SIP_NULLPTR, SIP_NULLPTR}
};


static pyqt6ClassPluginDef plugin_FlowImages = {
    &::FlowImages::staticMetaObject,
    signals_FlowImages,
    SIP_NULLPTR
};


sipClassTypeDef sipTypeDef_pictureflow_FlowImages = {
    {
        SIP_NULLPTR,
        SIP_TYPE_SCC|SIP_TYPE_SUPER_INIT|SIP_TYPE_CLASS,
        sipNameNr_FlowImages,
        SIP_NULLPTR,
        &plugin_FlowImages,
    },
    {
        sipNameNr_FlowImages,
        {0, 0, 1},
        4, methods_FlowImages,
        0, SIP_NULLPTR,
        {SIP_NULLPTR, SIP_NULLPTR, SIP_NULLPTR, SIP_NULLPTR, SIP_NULLPTR, SIP_NULLPTR, SIP_NULLPTR, SIP_NULLPTR, SIP_NULLPTR, SIP_NULLPTR},
    },
    doc_FlowImages,
    -1,
    -1,
    supers_FlowImages,
    SIP_NULLPTR,
    init_type_FlowImages,
    SIP_NULLPTR,
    SIP_NULLPTR,
    SIP_NULLPTR,
    SIP_NULLPTR,
    dealloc_FlowImages,
    SIP_NULLPTR,
    array_FlowImages,
    SIP_NULLPTR,
    release_FlowImages,
    cast_FlowImages,
    SIP_NULLPTR,
    SIP_NULLPTR,
    SIP_NULLPTR,
    SIP_NULLPTR,
    SIP_NULLPTR,
    SIP_NULLPTR,
    array_delete_FlowImages,
    sizeof (::FlowImages),
};
