/*
 * Internal module API header file.
 *
 * Generated by SIP 6.16.1
 */

#ifndef _imageopsAPI_H
#define _imageopsAPI_H

#include "sip.h"

#include <QMetaType>
#include <QThread>

/* These are the qualifiers that are enabled. */
#define SIP_FEATURE_PyQt_Wayland
#define SIP_FEATURE_PyQt_XCB
#define SIP_FEATURE_PyQt_Permissions
#define SIP_FEATURE_PyQt_DTLS
#define SIP_FEATURE_PyQt_WebChannel
#define SIP_FEATURE_PyQt_Process
#define SIP_FEATURE_PyQt_OpenGL
#define SIP_FEATURE_PyQt_RawFont
#define SIP_FEATURE_PyQt_PrintPreviewDialog
#define SIP_FEATURE_PyQt_PrintPreviewWidget
#define SIP_FEATURE_PyQt_Printer
#define SIP_FEATURE_PyQt_PrintDialog
#define SIP_FEATURE_PyQt_qreal_double
#define SIP_FEATURE_PyQt_SSL
#define SIP_FEATURE_PyQt_SessionManager
#define SIP_FEATURE_PyQt_Accessibility
#define SIP_PLATFORM_Linux
#define SIP_TIMELINE_Qt_6_11_0


/*
 * Convenient names to refer to various strings defined in this module.
 * Only the class names are part of the public API.
 */
#define sipNameNr_load_from_data_without_gil 0
#define sipName_load_from_data_without_gil &sipStrings_imageops[0]
#define sipNameNr_has_transparent_pixels 27
#define sipName_has_transparent_pixels &sipStrings_imageops[27]
#define sipNameNr_gaussian_sharpen 50
#define sipName_gaussian_sharpen &sipStrings_imageops[50]
#define sipNameNr_dominant_color 67
#define sipName_dominant_color &sipStrings_imageops[67]
#define sipNameNr_ordered_dither 82
#define sipName_ordered_dither &sipStrings_imageops[82]
#define sipNameNr_remove_borders 97
#define sipName_remove_borders &sipStrings_imageops[97]
#define sipNameNr_gaussian_blur 112
#define sipName_gaussian_blur &sipStrings_imageops[112]
#define sipNameNr_texture_image 126
#define sipName_texture_image &sipStrings_imageops[126]
#define sipNameNr_set_opacity 140
#define sipName_set_opacity &sipStrings_imageops[140]
#define sipNameNr_despeckle 152
#define sipName_despeckle &sipStrings_imageops[152]
#define sipNameNr_grayscale 162
#define sipName_grayscale &sipStrings_imageops[162]
#define sipNameNr_normalize 172
#define sipName_normalize &sipStrings_imageops[172]
#define sipNameNr_oil_paint 182
#define sipName_oil_paint &sipStrings_imageops[182]
#define sipNameNr_imageops 192
#define sipName_imageops &sipStrings_imageops[192]
#define sipNameNr_quantize 201
#define sipName_quantize &sipStrings_imageops[201]
#define sipNameNr_overlay 210
#define sipName_overlay &sipStrings_imageops[210]

#define sipMalloc                   sipAPI_imageops->api_malloc
#define sipFree                     sipAPI_imageops->api_free
#define sipBuildResult              sipAPI_imageops->api_build_result
#define sipCallMethod               sipAPI_imageops->api_call_method
#define sipCallProcedureMethod      sipAPI_imageops->api_call_procedure_method
#define sipCallErrorHandler         sipAPI_imageops->api_call_error_handler
#define sipParseResultEx            sipAPI_imageops->api_parse_result_ex
#define sipParseResult              sipAPI_imageops->api_parse_result
#define sipParseArgs                sipAPI_imageops->api_parse_args
#define sipParseKwdArgs             sipAPI_imageops->api_parse_kwd_args
#define sipParsePair                sipAPI_imageops->api_parse_pair
#define sipInstanceDestroyed        sipAPI_imageops->api_instance_destroyed
#define sipInstanceDestroyedEx      sipAPI_imageops->api_instance_destroyed_ex
#define sipConvertFromSequenceIndex sipAPI_imageops->api_convert_from_sequence_index
#define sipConvertFromSliceObject   sipAPI_imageops->api_convert_from_slice_object
#define sipConvertFromVoidPtr       sipAPI_imageops->api_convert_from_void_ptr
#define sipConvertToVoidPtr         sipAPI_imageops->api_convert_to_void_ptr
#define sipAddException             sipAPI_imageops->api_add_exception
#define sipNoFunction               sipAPI_imageops->api_no_function
#define sipNoMethod                 sipAPI_imageops->api_no_method
#define sipAbstractMethod           sipAPI_imageops->api_abstract_method
#define sipBadClass                 sipAPI_imageops->api_bad_class
#define sipBadCatcherResult         sipAPI_imageops->api_bad_catcher_result
#define sipBadCallableArg           sipAPI_imageops->api_bad_callable_arg
#define sipBadOperatorArg           sipAPI_imageops->api_bad_operator_arg
#define sipTrace                    sipAPI_imageops->api_trace
#define sipTransferBack             sipAPI_imageops->api_transfer_back
#define sipTransferTo               sipAPI_imageops->api_transfer_to
#define sipSimpleWrapper_Type       sipAPI_imageops->api_simplewrapper_type
#define sipWrapper_Type             sipAPI_imageops->api_wrapper_type
#define sipWrapperType_Type         sipAPI_imageops->api_wrappertype_type
#define sipVoidPtr_Type             sipAPI_imageops->api_voidptr_type
#define sipGetPyObject              sipAPI_imageops->api_get_pyobject
#define sipGetAddress               sipAPI_imageops->api_get_address
#define sipGetMixinAddress          sipAPI_imageops->api_get_mixin_address
#define sipGetCppPtr                sipAPI_imageops->api_get_cpp_ptr
#define sipGetComplexCppPtr         sipAPI_imageops->api_get_complex_cpp_ptr
#define sipCallHook                 sipAPI_imageops->api_call_hook
#define sipEndThread                sipAPI_imageops->api_end_thread
#define sipRaiseUnknownException    sipAPI_imageops->api_raise_unknown_exception
#define sipRaiseTypeException       sipAPI_imageops->api_raise_type_exception
#define sipBadLengthForSlice        sipAPI_imageops->api_bad_length_for_slice
#define sipAddTypeInstance          sipAPI_imageops->api_add_type_instance
#define sipPySlotExtend             sipAPI_imageops->api_pyslot_extend
#define sipAddDelayedDtor           sipAPI_imageops->api_add_delayed_dtor
#define sipCanConvertToType         sipAPI_imageops->api_can_convert_to_type
#define sipConvertToType            sipAPI_imageops->api_convert_to_type
#define sipForceConvertToType       sipAPI_imageops->api_force_convert_to_type
#define sipConvertToEnum            sipAPI_imageops->api_convert_to_enum
#define sipConvertToBool            sipAPI_imageops->api_convert_to_bool
#define sipReleaseType              sipAPI_imageops->api_release_type
#define sipConvertFromType          sipAPI_imageops->api_convert_from_type
#define sipConvertFromNewType       sipAPI_imageops->api_convert_from_new_type
#define sipConvertFromNewPyType     sipAPI_imageops->api_convert_from_new_pytype
#define sipConvertFromEnum          sipAPI_imageops->api_convert_from_enum
#define sipGetState                 sipAPI_imageops->api_get_state
#define sipExportSymbol             sipAPI_imageops->api_export_symbol
#define sipImportSymbol             sipAPI_imageops->api_import_symbol
#define sipFindType                 sipAPI_imageops->api_find_type
#define sipBytes_AsChar             sipAPI_imageops->api_bytes_as_char
#define sipBytes_AsString           sipAPI_imageops->api_bytes_as_string
#define sipString_AsASCIIChar       sipAPI_imageops->api_string_as_ascii_char
#define sipString_AsASCIIString     sipAPI_imageops->api_string_as_ascii_string
#define sipString_AsLatin1Char      sipAPI_imageops->api_string_as_latin1_char
#define sipString_AsLatin1String    sipAPI_imageops->api_string_as_latin1_string
#define sipString_AsUTF8Char        sipAPI_imageops->api_string_as_utf8_char
#define sipString_AsUTF8String      sipAPI_imageops->api_string_as_utf8_string
#define sipUnicode_AsWChar          sipAPI_imageops->api_unicode_as_wchar
#define sipUnicode_AsWString        sipAPI_imageops->api_unicode_as_wstring
#define sipConvertFromConstVoidPtr  sipAPI_imageops->api_convert_from_const_void_ptr
#define sipConvertFromVoidPtrAndSize    sipAPI_imageops->api_convert_from_void_ptr_and_size
#define sipConvertFromConstVoidPtrAndSize   sipAPI_imageops->api_convert_from_const_void_ptr_and_size
#define sipWrappedTypeName(wt)      ((wt)->wt_td->td_cname)
#define sipGetReference             sipAPI_imageops->api_get_reference
#define sipKeepReference            sipAPI_imageops->api_keep_reference
#define sipRegisterProxyResolver    sipAPI_imageops->api_register_proxy_resolver
#define sipRegisterPyType           sipAPI_imageops->api_register_py_type
#define sipTypeFromPyTypeObject     sipAPI_imageops->api_type_from_py_type_object
#define sipTypeScope                sipAPI_imageops->api_type_scope
#define sipResolveTypedef           sipAPI_imageops->api_resolve_typedef
#define sipRegisterAttributeGetter  sipAPI_imageops->api_register_attribute_getter
#define sipEnableAutoconversion     sipAPI_imageops->api_enable_autoconversion
#define sipInitMixin                sipAPI_imageops->api_init_mixin
#define sipExportModule             sipAPI_imageops->api_export_module
#define sipInitModule               sipAPI_imageops->api_init_module
#define sipGetInterpreter           sipAPI_imageops->api_get_interpreter
#define sipSetTypeUserData          sipAPI_imageops->api_set_type_user_data
#define sipGetTypeUserData          sipAPI_imageops->api_get_type_user_data
#define sipPyTypeDict               sipAPI_imageops->api_py_type_dict
#define sipPyTypeName               sipAPI_imageops->api_py_type_name
#define sipGetCFunction             sipAPI_imageops->api_get_c_function
#define sipGetMethod                sipAPI_imageops->api_get_method
#define sipFromMethod               sipAPI_imageops->api_from_method
#define sipGetDate                  sipAPI_imageops->api_get_date
#define sipFromDate                 sipAPI_imageops->api_from_date
#define sipGetDateTime              sipAPI_imageops->api_get_datetime
#define sipFromDateTime             sipAPI_imageops->api_from_datetime
#define sipGetTime                  sipAPI_imageops->api_get_time
#define sipFromTime                 sipAPI_imageops->api_from_time
#define sipIsUserType               sipAPI_imageops->api_is_user_type
#define sipCheckPluginForType       sipAPI_imageops->api_check_plugin_for_type
#define sipUnicodeNew               sipAPI_imageops->api_unicode_new
#define sipUnicodeWrite             sipAPI_imageops->api_unicode_write
#define sipUnicodeData              sipAPI_imageops->api_unicode_data
#define sipGetBufferInfo            sipAPI_imageops->api_get_buffer_info
#define sipReleaseBufferInfo        sipAPI_imageops->api_release_buffer_info
#define sipIsOwnedByPython          sipAPI_imageops->api_is_owned_by_python
#define sipIsDerivedClass           sipAPI_imageops->api_is_derived_class
#define sipGetUserObject            sipAPI_imageops->api_get_user_object
#define sipSetUserObject            sipAPI_imageops->api_set_user_object
#define sipRegisterEventHandler     sipAPI_imageops->api_register_event_handler
#define sipConvertToArray           sipAPI_imageops->api_convert_to_array
#define sipConvertToTypedArray      sipAPI_imageops->api_convert_to_typed_array
#define sipEnableGC                 sipAPI_imageops->api_enable_gc
#define sipPrintObject              sipAPI_imageops->api_print_object
#define sipLong_AsChar              sipAPI_imageops->api_long_as_char
#define sipLong_AsSignedChar        sipAPI_imageops->api_long_as_signed_char
#define sipLong_AsUnsignedChar      sipAPI_imageops->api_long_as_unsigned_char
#define sipLong_AsShort             sipAPI_imageops->api_long_as_short
#define sipLong_AsUnsignedShort     sipAPI_imageops->api_long_as_unsigned_short
#define sipLong_AsInt               sipAPI_imageops->api_long_as_int
#define sipLong_AsUnsignedInt       sipAPI_imageops->api_long_as_unsigned_int
#define sipLong_AsLong              sipAPI_imageops->api_long_as_long
#define sipLong_AsUnsignedLong      sipAPI_imageops->api_long_as_unsigned_long
#define sipLong_AsLongLong          sipAPI_imageops->api_long_as_long_long
#define sipLong_AsUnsignedLongLong  sipAPI_imageops->api_long_as_unsigned_long_long
#define sipLong_AsSizeT             sipAPI_imageops->api_long_as_size_t
#define sipVisitWrappers            sipAPI_imageops->api_visit_wrappers
#define sipRegisterExitNotifier     sipAPI_imageops->api_register_exit_notifier
#define sipDeprecated               sipAPI_imageops->api_deprecated
#define sipPyTypeDictRef            sipAPI_imageops->api_py_type_dict_ref
#define sipNextExceptionHandler     sipAPI_imageops->api_next_exception_handler
#define sipIsEnumFlag               sipAPI_imageops->api_is_enum_flag
#define sipConvertToTypeUS          sipAPI_imageops->api_convert_to_type_us
#define sipForceConvertToTypeUS     sipAPI_imageops->api_force_convert_to_type_us
#define sipReleaseTypeUS            sipAPI_imageops->api_release_type_us
#define sipIsPyMethod               sipAPI_imageops->api_is_py_method_12_8

/* The strings used by this module. */
extern const char sipStrings_imageops[];

/* The SIP API, this module's API and the APIs of any imported modules. */
extern const sipAPIDef *sipAPI_imageops;
extern sipExportedModuleDef sipModuleAPI_imageops;

#define sipType_QList_1900 sipImportedTypes_imageops_QtCore[0].it_td
extern sipImportedTypeDef sipImportedTypes_imageops_QtCore[];

#define sipType_QColor sipImportedTypes_imageops_QtGui[0].it_td

#define sipType_QImage sipImportedTypes_imageops_QtGui[1].it_td
extern sipImportedTypeDef sipImportedTypes_imageops_QtGui[];

typedef const QMetaObject *(*sip_qt_metaobject_func)(sipSimpleWrapper *, sipTypeDef *);
extern sip_qt_metaobject_func sip_imageops_qt_metaobject;

typedef int (*sip_qt_metacall_func)(sipSimpleWrapper *, sipTypeDef *, QMetaObject::Call, int, void **);
extern sip_qt_metacall_func sip_imageops_qt_metacall;

typedef bool (*sip_qt_metacast_func)(sipSimpleWrapper *, const sipTypeDef *, const char *, void **);
extern sip_qt_metacast_func sip_imageops_qt_metacast;

#endif
