/*
 * Module code.
 *
 * Generated by SIP 6.16.1
 */

#include "sipAPIprogress_indicator.h"

#line 28 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qobject.sip"
#include <qobject.h>
#line 12 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorcmodule.cpp"
#line 26 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtWidgets/qsizepolicy.sip"
#include <qsizepolicy.h>
#line 15 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorcmodule.cpp"
#line 26 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qnamespace.sip"
#include <qnamespace.h>
#line 154 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qtextstream.sip"
#include <qtextstream.h>
#line 20 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorcmodule.cpp"
#line 26 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtWidgets/qstyleoption.sip"
#include <qstyleoption.h>
#line 23 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorcmodule.cpp"
#line 28 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtWidgets/qwidget.sip"
#include <qwidget.h>
#line 26 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorcmodule.cpp"
#line 26 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qpixmap.sip"
#include <qpixmap.h>
#line 29 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorcmodule.cpp"
#line 26 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qicon.sip"
#include <qicon.h>
#line 32 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorcmodule.cpp"
#line 26 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtWidgets/qstyle.sip"
#include <qstyle.h>
#line 35 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorcmodule.cpp"
#line 923 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtWidgets/qstyleoption.sip"
#include <qstyleoption.h>
#line 38 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorcmodule.cpp"
#line 30 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qsize.sip"
#include <qsize.h>
#line 41 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorcmodule.cpp"
#line 30 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qrect.sip"
#include <qrect.h>
#line 44 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorcmodule.cpp"
#line 749 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtWidgets/qstyleoption.sip"
#include <qstyleoption.h>
#line 47 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorcmodule.cpp"
#line 30 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qpoint.sip"
#include <qpoint.h>
#line 50 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorcmodule.cpp"
#line 26 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qpainter.sip"
#include <qpainter.h>
#line 53 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorcmodule.cpp"
#line 26 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qpalette.sip"
#include <qpalette.h>
#line 56 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorcmodule.cpp"
#line 24 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qstring.sip"
#include <qstring.h>
#line 59 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorcmodule.cpp"
#line 26 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qfontmetrics.sip"
#include <qfontmetrics.h>
#line 62 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorcmodule.cpp"
#line 28 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtWidgets/qapplication.sip"
#include <qapplication.h>
#line 65 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorcmodule.cpp"
#line 26 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qcoreevent.sip"
#include <qcoreevent.h>
#line 68 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorcmodule.cpp"
#line 254 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qcoreevent.sip"
#include <qcoreevent.h>
#line 71 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorcmodule.cpp"
#line 277 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qcoreevent.sip"
#include <qcoreevent.h>
#line 74 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorcmodule.cpp"
#line 26 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qmetaobject.sip"
#include <qmetaobject.h>
#line 77 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorcmodule.cpp"
#line 30 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qbytearray.sip"
#include <qbytearray.h>
#line 80 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorcmodule.cpp"
#line 116 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qglobal.sip"
#include <QtGlobal>
#line 83 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorcmodule.cpp"
#line 26 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qvariant.sip"
#include <qvariant.h>
#line 86 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorcmodule.cpp"
#line 380 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qevent.sip"
#include <qevent.h>
#line 89 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorcmodule.cpp"
#line 339 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qevent.sip"
#include <qevent.h>
#line 92 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorcmodule.cpp"
#line 325 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qevent.sip"
#include <qevent.h>
#line 95 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorcmodule.cpp"
#line 444 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qevent.sip"
#include <qevent.h>
#line 98 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorcmodule.cpp"
#line 497 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qevent.sip"
#include <qevent.h>
#line 101 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorcmodule.cpp"
#line 466 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qevent.sip"
#include <qevent.h>
#line 104 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorcmodule.cpp"
#line 483 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qevent.sip"
#include <qevent.h>
#line 107 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorcmodule.cpp"
#line 555 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qevent.sip"
#include <qevent.h>
#line 110 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorcmodule.cpp"
#line 807 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qevent.sip"
#include <qevent.h>
#line 113 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorcmodule.cpp"
#line 353 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qevent.sip"
#include <qevent.h>
#line 116 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorcmodule.cpp"
#line 297 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qevent.sip"
#include <qevent.h>
#line 119 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorcmodule.cpp"
#line 283 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qevent.sip"
#include <qevent.h>
#line 122 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorcmodule.cpp"
#line 269 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qevent.sip"
#include <qevent.h>
#line 125 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorcmodule.cpp"
#line 254 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qevent.sip"
#include <qevent.h>
#line 128 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorcmodule.cpp"
#line 742 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qevent.sip"
#include <qevent.h>
#line 131 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorcmodule.cpp"
#line 239 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qevent.sip"
#include <qevent.h>
#line 134 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorcmodule.cpp"
#line 213 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qevent.sip"
#include <qevent.h>
#line 137 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorcmodule.cpp"
#line 788 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qevent.sip"
#include <qevent.h>
#line 140 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorcmodule.cpp"
#line 754 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qevent.sip"
#include <qevent.h>
#line 143 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorcmodule.cpp"
#line 50 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qpaintengine.sip"
#include <qpaintengine.h>
#line 146 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorcmodule.cpp"
#line 26 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qpaintdevice.sip"
#include <qpaintdevice.h>
#line 149 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorcmodule.cpp"
#line 26 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qcolor.sip"
#include <qcolor.h>
#line 152 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorcmodule.cpp"
#line 28 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qaction.sip"
#include <qaction.h>
#line 155 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorcmodule.cpp"
#line 26 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtWidgets/qmenu.sip"
#include <qmenu.h>
#line 158 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorcmodule.cpp"
#line 33 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qimage.sip"
#include <qimage.h>
#line 161 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorcmodule.cpp"
#line 27 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/QProgressIndicator.sip"
#include <QProgressIndicator.h>
#line 164 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorcmodule.cpp"
#line 50 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/QProgressIndicator.sip"
#include <QProgressIndicator.h>
#line 167 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorcmodule.cpp"

/* Define the strings used by this module. */
const char sipStrings_progress_indicator[] = {
    'g', 'e', 't', '_', 'i', 'm', 'a', 'g', 'e', '_', 'a', 'l', 'l', 'o', 'c', 'a', 't', 'i', 'o', 'n', '_', 'l', 'i', 'm', 'i', 't', 0,
    's', 'e', 't', '_', 'i', 'm', 'a', 'g', 'e', '_', 'a', 'l', 'l', 'o', 'c', 'a', 't', 'i', 'o', 'n', '_', 'l', 'i', 'm', 'i', 't', 0,
    's', 'e', 't', '_', 'n', 'o', '_', 'a', 'c', 't', 'i', 'v', 'a', 't', 'e', '_', 'o', 'n', '_', 'c', 'l', 'i', 'c', 'k', 0,
    'h', 'i', 't', 'T', 'e', 's', 't', 'C', 'o', 'm', 'p', 'l', 'e', 'x', 'C', 'o', 'n', 't', 'r', 'o', 'l', 0,
    'i', 'n', 's', 't', 'a', 'l', 'l', '_', 'q', 't', '_', 't', 'r', 'a', 'n', 's', 'l', 'a', 't', 'o', 'r', 0,
    'm', 'o', 'u', 's', 'e', 'D', 'o', 'u', 'b', 'l', 'e', 'C', 'l', 'i', 'c', 'k', 'E', 'v', 'e', 'n', 't', 0,
    'r', 'u', 'n', 'n', 'i', 'n', 'g', '_', 's', 't', 'a', 't', 'e', '_', 'c', 'h', 'a', 'n', 'g', 'e', 'd', 0,
    'g', 'e', 't', '_', 'o', 'v', 'e', 'r', 'a', 'l', 'l', '_', 'r', 'o', 't', 'a', 't', 'i', 'o', 'n', 0,
    's', 'e', 't', '_', 'o', 'v', 'e', 'r', 'a', 'l', 'l', '_', 'r', 'o', 't', 'a', 't', 'i', 'o', 'n', 0,
    'g', 'e', 'n', 'e', 'r', 'a', 't', 'e', 'd', 'I', 'c', 'o', 'n', 'P', 'i', 'x', 'm', 'a', 'p', 0,
    'Q', 'P', 'r', 'o', 'g', 'r', 'e', 's', 's', 'I', 'n', 'd', 'i', 'c', 'a', 't', 'o', 'r', 0,
    'd', 'r', 'a', 'w', 'C', 'o', 'm', 'p', 'l', 'e', 'x', 'C', 'o', 'n', 't', 'r', 'o', 'l', 0,
    'd', 'r', 'a', 'w', '_', 's', 'n', 'a', 'k', 'e', '_', 's', 'p', 'i', 'n', 'n', 'e', 'r', 0,
    'f', 'o', 'c', 'u', 's', 'N', 'e', 'x', 't', 'P', 'r', 'e', 'v', 'C', 'h', 'i', 'l', 'd', 0,
    'i', 'm', 'a', 'g', 'e', '_', 'f', 'r', 'o', 'm', '_', 'h', 'b', 'i', 't', 'm', 'a', 'p', 0,
    'p', 'r', 'o', 'g', 'r', 'e', 's', 's', '_', 'i', 'n', 'd', 'i', 'c', 'a', 't', 'o', 'r', 0,
    's', 'e', 't', '_', 'm', 'e', 'n', 'u', '_', 'o', 'n', '_', 'a', 'c', 't', 'i', 'o', 'n', 0,
    'h', 'a', 's', 'H', 'e', 'i', 'g', 'h', 't', 'F', 'o', 'r', 'W', 'i', 'd', 't', 'h', 0,
    'm', 'o', 'u', 's', 'e', 'R', 'e', 'l', 'e', 'a', 's', 'e', 'E', 'v', 'e', 'n', 't', 0,
    'c', 'o', 'n', 't', 'e', 'x', 't', 'M', 'e', 'n', 'u', 'E', 'v', 'e', 'n', 't', 0,
    'd', 'i', 's', 'c', 'o', 'n', 'n', 'e', 'c', 't', 'N', 'o', 't', 'i', 'f', 'y', 0,
    'g', 'e', 't', '_', 'a', 'r', 'c', '_', 'r', 'o', 't', 'a', 't', 'i', 'o', 'n', 0,
    'i', 'm', 'a', 'g', 'e', '_', 'f', 'r', 'o', 'm', '_', 'h', 'i', 'c', 'o', 'n', 0,
    'i', 'n', 'p', 'u', 't', 'M', 'e', 't', 'h', 'o', 'd', 'E', 'v', 'e', 'n', 't', 0,
    'i', 'n', 'p', 'u', 't', 'M', 'e', 't', 'h', 'o', 'd', 'Q', 'u', 'e', 'r', 'y', 0,
    's', 'e', 't', '_', 'a', 'r', 'c', '_', 'r', 'o', 't', 'a', 't', 'i', 'o', 'n', 0,
    's', 'i', 'z', 'e', 'F', 'r', 'o', 'm', 'C', 'o', 'n', 't', 'e', 'n', 't', 's', 0,
    'i', 'c', 'o', 'n', '_', 'f', 'r', 'o', 'm', '_', 'p', 'a', 't', 'h', 's', 0,
    'k', 'e', 'y', 'R', 'e', 'l', 'e', 'a', 's', 'e', 'E', 'v', 'e', 'n', 't', 0,
    'm', 'e', 'n', 'u', '_', 'f', 'o', 'r', '_', 'a', 'c', 't', 'i', 'o', 'n', 0,
    'm', 'i', 'n', 'i', 'm', 'u', 'm', 'S', 'i', 'z', 'e', 'H', 'i', 'n', 't', 0,
    'm', 'o', 'u', 's', 'e', 'P', 'r', 'e', 's', 's', 'E', 'v', 'e', 'n', 't', 0,
    's', 't', 'a', 'n', 'd', 'a', 'r', 'd', 'P', 'a', 'l', 'e', 't', 't', 'e', 0,
    'c', 'o', 'n', 't', 'r', 'a', 's', 't', '_', 'r', 'a', 't', 'i', 'o', 0,
    'd', 'r', 'a', 'g', 'E', 'n', 't', 'e', 'r', 'E', 'v', 'e', 'n', 't', 0,
    'd', 'r', 'a', 'g', 'L', 'e', 'a', 'v', 'e', 'E', 'v', 'e', 'n', 't', 0,
    'd', 'r', 'a', 'w', 'I', 't', 'e', 'm', 'P', 'i', 'x', 'm', 'a', 'p', 0,
    'g', 'e', 't', '_', 'a', 'r', 'c', '_', 'l', 'e', 'n', 'g', 't', 'h', 0,
    'h', 'e', 'i', 'g', 'h', 't', 'F', 'o', 'r', 'W', 'i', 'd', 't', 'h', 0,
    'i', 'c', 'o', 'n', '_', 'f', 'r', 'o', 'm', '_', 'n', 'a', 'm', 'e', 0,
    'i', 't', 'e', 'm', 'P', 'i', 'x', 'm', 'a', 'p', 'R', 'e', 'c', 't', 0,
    'm', 'o', 'u', 's', 'e', 'M', 'o', 'v', 'e', 'E', 'v', 'e', 'n', 't', 0,
    's', 'e', 't', 'D', 'i', 's', 'p', 'l', 'a', 'y', 'S', 'i', 'z', 'e', 0,
    's', 'e', 't', '_', 'a', 'r', 'c', '_', 'l', 'e', 'n', 'g', 't', 'h', 0,
    's', 'e', 't', '_', 'i', 'c', 'o', 'n', '_', 't', 'h', 'e', 'm', 'e', 0,
    's', 't', 'a', 'n', 'd', 'a', 'r', 'd', 'P', 'i', 'x', 'm', 'a', 'p', 0,
    's', 't', 'a', 'r', 't', 'A', 'n', 'i', 'm', 'a', 't', 'i', 'o', 'n', 0,
    's', 'u', 'b', 'C', 'o', 'n', 't', 'r', 'o', 'l', 'R', 'e', 'c', 't', 0,
    's', 'u', 'b', 'E', 'l', 'e', 'm', 'e', 'n', 't', 'R', 'e', 'c', 't', 0,
    'd', 'r', 'a', 'g', 'M', 'o', 'v', 'e', 'E', 'v', 'e', 'n', 't', 0,
    'd', 'r', 'a', 'w', 'P', 'r', 'i', 'm', 'i', 't', 'i', 'v', 'e', 0,
    'f', 'o', 'c', 'u', 's', 'O', 'u', 't', 'E', 'v', 'e', 'n', 't', 0,
    'k', 'e', 'y', 'P', 'r', 'e', 's', 's', 'E', 'v', 'e', 'n', 't', 0,
    'l', 'a', 'y', 'o', 'u', 't', 'S', 'p', 'a', 'c', 'i', 'n', 'g', 0,
    's', 'h', 'a', 'r', 'e', 'd', 'P', 'a', 'i', 'n', 't', 'e', 'r', 0,
    's', 't', 'o', 'p', 'A', 'n', 'i', 'm', 'a', 't', 'i', 'o', 'n', 0,
    'C', 'a', 'l', 'i', 'b', 'r', 'e', 'S', 't', 'y', 'l', 'e', 0,
    'S', 'p', 'i', 'n', 'A', 'n', 'i', 'm', 'a', 't', 'o', 'r', 0,
    'd', 'r', 'a', 'w', 'I', 't', 'e', 'm', 'T', 'e', 'x', 't', 0,
    'f', 'o', 'c', 'u', 's', 'I', 'n', 'E', 'v', 'e', 'n', 't', 0,
    'i', 't', 'e', 'm', 'T', 'e', 'x', 't', 'R', 'e', 'c', 't', 0,
    's', 't', 'a', 'n', 'd', 'a', 'r', 'd', 'I', 'c', 'o', 'n', 0,
    'a', 'c', 't', 'i', 'o', 'n', 'E', 'v', 'e', 'n', 't', 0,
    'c', 'h', 'a', 'n', 'g', 'e', 'E', 'v', 'e', 'n', 't', 0,
    'c', 'u', 's', 't', 'o', 'm', 'E', 'v', 'e', 'n', 't', 0,
    'd', 'i', 's', 'p', 'l', 'a', 'y', 'S', 'i', 'z', 'e', 0,
    'd', 'r', 'a', 'w', 'C', 'o', 'n', 't', 'r', 'o', 'l', 0,
    'e', 'v', 'e', 'n', 't', 'F', 'i', 'l', 't', 'e', 'r', 0,
    'i', 'n', 'i', 't', 'P', 'a', 'i', 'n', 't', 'e', 'r', 0,
    'n', 'a', 't', 'i', 'v', 'e', 'E', 'v', 'e', 'n', 't', 0,
    'p', 'a', 'i', 'n', 't', 'E', 'n', 'g', 'i', 'n', 'e', 0,
    'p', 'i', 'x', 'e', 'l', 'M', 'e', 't', 'r', 'i', 'c', 0,
    'r', 'e', 's', 'i', 'z', 'e', 'E', 'v', 'e', 'n', 't', 0,
    's', 'e', 't', 'S', 'i', 'z', 'e', 'H', 'i', 'n', 't', 0,
    't', 'a', 'b', 'l', 'e', 't', 'E', 'v', 'e', 'n', 't', 0,
    'u', 't', 'f', '1', '6', '_', 's', 'l', 'i', 'c', 'e', 0,
    'c', 'h', 'i', 'l', 'd', 'E', 'v', 'e', 'n', 't', 0,
    'c', 'l', 'o', 's', 'e', 'E', 'v', 'e', 'n', 't', 0,
    'e', 'n', 't', 'e', 'r', 'E', 'v', 'e', 'n', 't', 0,
    'i', 's', 'A', 'n', 'i', 'm', 'a', 't', 'e', 'd', 0,
    'i', 's', '_', 'r', 'u', 'n', 'n', 'i', 'n', 'g', 0,
    'l', 'e', 'a', 'v', 'e', 'E', 'v', 'e', 'n', 't', 0,
    'p', 'a', 'i', 'n', 't', 'E', 'v', 'e', 'n', 't', 0,
    's', 'e', 't', 'V', 'i', 's', 'i', 'b', 'l', 'e', 0,
    't', 'i', 'm', 'e', 'r', 'E', 'v', 'e', 'n', 't', 0,
    'w', 'h', 'e', 'e', 'l', 'E', 'v', 'e', 'n', 't', 0,
    'd', 'r', 'o', 'p', 'E', 'v', 'e', 'n', 't', 0,
    'h', 'i', 'd', 'e', 'E', 'v', 'e', 'n', 't', 0,
    'm', 'o', 'v', 'e', 'E', 'v', 'e', 'n', 't', 0,
    's', 'h', 'o', 'w', 'E', 'v', 'e', 'n', 't', 0,
    's', 't', 'y', 'l', 'e', 'H', 'i', 'n', 't', 0,
    's', 'i', 'z', 'e', 'H', 'i', 'n', 't', 0,
    'u', 'n', 'p', 'o', 'l', 'i', 's', 'h', 0,
    'd', 'e', 'v', 'T', 'y', 'p', 'e', 0,
    'u', 'p', 'd', 'a', 't', 'e', 'd', 0,
    'm', 'e', 't', 'r', 'i', 'c', 0,
    'e', 'v', 'e', 'n', 't', 0,
    's', 't', 'a', 'r', 't', 0,
    'd', 'r', 'a', 'w', 0,
    's', 't', 'o', 'p', 0,
};

int sipVH_progress_indicator_57(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod, ::QPaintDevice::PaintDeviceMetric a0)
{
    int sipRes = 0;
    PyObject *sipResObj = sipCallMethod(SIP_NULLPTR, sipMethod, "F", a0, sipType_QPaintDevice_PaintDeviceMetric);

    sipParseResultEx(sipGILState, sipErrorHandler, sipPySelf, sipMethod, sipResObj, "i", &sipRes);

    return sipRes;
}

::QPaintEngine* sipVH_progress_indicator_56(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod)
{
    ::QPaintEngine* sipRes = 0;
    PyObject *sipResObj = sipCallMethod(SIP_NULLPTR, sipMethod, "");

    sipParseResultEx(sipGILState, sipErrorHandler, sipPySelf, sipMethod, sipResObj, "H0", sipType_QPaintEngine, &sipRes);

    return sipRes;
}

int sipVH_progress_indicator_55(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod)
{
    int sipRes = 0;
    PyObject *sipResObj = sipCallMethod(SIP_NULLPTR, sipMethod, "");

    sipParseResultEx(sipGILState, sipErrorHandler, sipPySelf, sipMethod, sipResObj, "i", &sipRes);

    return sipRes;
}

void sipVH_progress_indicator_54(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod, bool a0)
{
    sipCallProcedureMethod(sipGILState, sipErrorHandler, sipPySelf, sipMethod, "b", a0);
}

::QSize sipVH_progress_indicator_53(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod)
{
    ::QSize sipRes;
    PyObject *sipResObj = sipCallMethod(SIP_NULLPTR, sipMethod, "");

    sipParseResultEx(sipGILState, sipErrorHandler, sipPySelf, sipMethod, sipResObj, "H5", sipType_QSize, &sipRes);

    return sipRes;
}

int sipVH_progress_indicator_52(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod, int a0)
{
    int sipRes = 0;
    PyObject *sipResObj = sipCallMethod(SIP_NULLPTR, sipMethod, "i", a0);

    sipParseResultEx(sipGILState, sipErrorHandler, sipPySelf, sipMethod, sipResObj, "i", &sipRes);

    return sipRes;
}

void sipVH_progress_indicator_51(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod, ::QMouseEvent*a0)
{
    sipCallProcedureMethod(sipGILState, sipErrorHandler, sipPySelf, sipMethod, "D", a0, sipType_QMouseEvent, SIP_NULLPTR);
}

void sipVH_progress_indicator_50(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod, ::QWheelEvent*a0)
{
    sipCallProcedureMethod(sipGILState, sipErrorHandler, sipPySelf, sipMethod, "D", a0, sipType_QWheelEvent, SIP_NULLPTR);
}

void sipVH_progress_indicator_49(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod, ::QKeyEvent*a0)
{
    sipCallProcedureMethod(sipGILState, sipErrorHandler, sipPySelf, sipMethod, "D", a0, sipType_QKeyEvent, SIP_NULLPTR);
}

void sipVH_progress_indicator_48(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod, ::QFocusEvent*a0)
{
    sipCallProcedureMethod(sipGILState, sipErrorHandler, sipPySelf, sipMethod, "D", a0, sipType_QFocusEvent, SIP_NULLPTR);
}

void sipVH_progress_indicator_47(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod, ::QEnterEvent*a0)
{
    sipCallProcedureMethod(sipGILState, sipErrorHandler, sipPySelf, sipMethod, "D", a0, sipType_QEnterEvent, SIP_NULLPTR);
}

void sipVH_progress_indicator_46(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod, ::QPaintEvent*a0)
{
    sipCallProcedureMethod(sipGILState, sipErrorHandler, sipPySelf, sipMethod, "D", a0, sipType_QPaintEvent, SIP_NULLPTR);
}

void sipVH_progress_indicator_45(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod, ::QMoveEvent*a0)
{
    sipCallProcedureMethod(sipGILState, sipErrorHandler, sipPySelf, sipMethod, "D", a0, sipType_QMoveEvent, SIP_NULLPTR);
}

void sipVH_progress_indicator_44(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod, ::QResizeEvent*a0)
{
    sipCallProcedureMethod(sipGILState, sipErrorHandler, sipPySelf, sipMethod, "D", a0, sipType_QResizeEvent, SIP_NULLPTR);
}

void sipVH_progress_indicator_43(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod, ::QCloseEvent*a0)
{
    sipCallProcedureMethod(sipGILState, sipErrorHandler, sipPySelf, sipMethod, "D", a0, sipType_QCloseEvent, SIP_NULLPTR);
}

void sipVH_progress_indicator_42(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod, ::QContextMenuEvent*a0)
{
    sipCallProcedureMethod(sipGILState, sipErrorHandler, sipPySelf, sipMethod, "D", a0, sipType_QContextMenuEvent, SIP_NULLPTR);
}

void sipVH_progress_indicator_41(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod, ::QTabletEvent*a0)
{
    sipCallProcedureMethod(sipGILState, sipErrorHandler, sipPySelf, sipMethod, "D", a0, sipType_QTabletEvent, SIP_NULLPTR);
}

void sipVH_progress_indicator_40(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod, ::QActionEvent*a0)
{
    sipCallProcedureMethod(sipGILState, sipErrorHandler, sipPySelf, sipMethod, "D", a0, sipType_QActionEvent, SIP_NULLPTR);
}

void sipVH_progress_indicator_39(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod, ::QDragEnterEvent*a0)
{
    sipCallProcedureMethod(sipGILState, sipErrorHandler, sipPySelf, sipMethod, "D", a0, sipType_QDragEnterEvent, SIP_NULLPTR);
}

void sipVH_progress_indicator_38(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod, ::QDragMoveEvent*a0)
{
    sipCallProcedureMethod(sipGILState, sipErrorHandler, sipPySelf, sipMethod, "D", a0, sipType_QDragMoveEvent, SIP_NULLPTR);
}

void sipVH_progress_indicator_37(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod, ::QDragLeaveEvent*a0)
{
    sipCallProcedureMethod(sipGILState, sipErrorHandler, sipPySelf, sipMethod, "D", a0, sipType_QDragLeaveEvent, SIP_NULLPTR);
}

void sipVH_progress_indicator_36(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod, ::QDropEvent*a0)
{
    sipCallProcedureMethod(sipGILState, sipErrorHandler, sipPySelf, sipMethod, "D", a0, sipType_QDropEvent, SIP_NULLPTR);
}

void sipVH_progress_indicator_35(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod, ::QShowEvent*a0)
{
    sipCallProcedureMethod(sipGILState, sipErrorHandler, sipPySelf, sipMethod, "D", a0, sipType_QShowEvent, SIP_NULLPTR);
}

void sipVH_progress_indicator_34(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod, ::QHideEvent*a0)
{
    sipCallProcedureMethod(sipGILState, sipErrorHandler, sipPySelf, sipMethod, "D", a0, sipType_QHideEvent, SIP_NULLPTR);
}

void sipVH_progress_indicator_33(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod, ::QInputMethodEvent*a0)
{
    sipCallProcedureMethod(sipGILState, sipErrorHandler, sipPySelf, sipMethod, "D", a0, sipType_QInputMethodEvent, SIP_NULLPTR);
}

::QVariant sipVH_progress_indicator_32(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod, ::Qt::InputMethodQuery a0)
{
    ::QVariant sipRes;
    PyObject *sipResObj = sipCallMethod(SIP_NULLPTR, sipMethod, "F", a0, sipType_Qt_InputMethodQuery);

    sipParseResultEx(sipGILState, sipErrorHandler, sipPySelf, sipMethod, sipResObj, "H5", sipType_QVariant, &sipRes);

    return sipRes;
}

bool sipVH_progress_indicator_31(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod, bool a0)
{
    bool sipRes = 0;
    PyObject *sipResObj = sipCallMethod(SIP_NULLPTR, sipMethod, "b", a0);

    sipParseResultEx(sipGILState, sipErrorHandler, sipPySelf, sipMethod, sipResObj, "b", &sipRes);

    return sipRes;
}

bool sipVH_progress_indicator_30(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod)
{
    bool sipRes = 0;
    PyObject *sipResObj = sipCallMethod(SIP_NULLPTR, sipMethod, "");

    sipParseResultEx(sipGILState, sipErrorHandler, sipPySelf, sipMethod, sipResObj, "b", &sipRes);

    return sipRes;
}

bool sipVH_progress_indicator_29(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod, const ::QByteArray& a0, void*a1, ::qintptr*a2)
{
    bool sipRes = 0;
    PyObject *sipResObj = sipCallMethod(SIP_NULLPTR, sipMethod, "NV", new ::QByteArray(a0), sipType_QByteArray, SIP_NULLPTR, a1);

    sipParseResultEx(sipGILState, sipErrorHandler, sipPySelf, sipMethod, sipResObj, "(bH4)", &sipRes, sipType_qintptr, a2);

    return sipRes;
}

::QPainter* sipVH_progress_indicator_28(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod)
{
    ::QPainter* sipRes = 0;
    PyObject *sipResObj = sipCallMethod(SIP_NULLPTR, sipMethod, "");

    sipParseResultEx(sipGILState, sipErrorHandler, sipPySelf, sipMethod, sipResObj, "H0", sipType_QPainter, &sipRes);

    return sipRes;
}

void sipVH_progress_indicator_27(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod, ::QPainter*a0)
{
    sipCallProcedureMethod(sipGILState, sipErrorHandler, sipPySelf, sipMethod, "D", a0, sipType_QPainter, SIP_NULLPTR);
}

void sipVH_progress_indicator_26(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod, const ::QMetaMethod& a0)
{
    sipCallProcedureMethod(sipGILState, sipErrorHandler, sipPySelf, sipMethod, "N", new ::QMetaMethod(a0), sipType_QMetaMethod, SIP_NULLPTR);
}

void sipVH_progress_indicator_25(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod, ::QEvent*a0)
{
    sipCallProcedureMethod(sipGILState, sipErrorHandler, sipPySelf, sipMethod, "D", a0, sipType_QEvent, SIP_NULLPTR);
}

void sipVH_progress_indicator_24(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod, ::QChildEvent*a0)
{
    sipCallProcedureMethod(sipGILState, sipErrorHandler, sipPySelf, sipMethod, "D", a0, sipType_QChildEvent, SIP_NULLPTR);
}

void sipVH_progress_indicator_23(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod, ::QTimerEvent*a0)
{
    sipCallProcedureMethod(sipGILState, sipErrorHandler, sipPySelf, sipMethod, "D", a0, sipType_QTimerEvent, SIP_NULLPTR);
}

bool sipVH_progress_indicator_22(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod, ::QObject*a0, ::QEvent*a1)
{
    bool sipRes = 0;
    PyObject *sipResObj = sipCallMethod(SIP_NULLPTR, sipMethod, "DD", a0, sipType_QObject, SIP_NULLPTR, a1, sipType_QEvent, SIP_NULLPTR);

    sipParseResultEx(sipGILState, sipErrorHandler, sipPySelf, sipMethod, sipResObj, "b", &sipRes);

    return sipRes;
}

bool sipVH_progress_indicator_21(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod, ::QEvent*a0)
{
    bool sipRes = 0;
    PyObject *sipResObj = sipCallMethod(SIP_NULLPTR, sipMethod, "D", a0, sipType_QEvent, SIP_NULLPTR);

    sipParseResultEx(sipGILState, sipErrorHandler, sipPySelf, sipMethod, sipResObj, "b", &sipRes);

    return sipRes;
}

void sipVH_progress_indicator_20(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod, ::QWidget*a0)
{
    sipCallProcedureMethod(sipGILState, sipErrorHandler, sipPySelf, sipMethod, "D", a0, sipType_QWidget, SIP_NULLPTR);
}

void sipVH_progress_indicator_19(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod, ::QApplication*a0)
{
    sipCallProcedureMethod(sipGILState, sipErrorHandler, sipPySelf, sipMethod, "D", a0, sipType_QApplication, SIP_NULLPTR);
}

void sipVH_progress_indicator_18(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod, ::QPalette& a0)
{
    PyObject *sipResObj = sipCallMethod(SIP_NULLPTR, sipMethod, "D", &a0, sipType_QPalette, SIP_NULLPTR);

    sipParseResultEx(sipGILState, sipErrorHandler, sipPySelf, sipMethod, sipResObj, "H5", sipType_QPalette, &a0);
}

::QRect sipVH_progress_indicator_17(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod, const ::QFontMetrics& a0, const ::QRect& a1, int a2, bool a3, const ::QString& a4)
{
    ::QRect sipRes;
    PyObject *sipResObj = sipCallMethod(SIP_NULLPTR, sipMethod, "NNibN", new ::QFontMetrics(a0), sipType_QFontMetrics, SIP_NULLPTR, new ::QRect(a1), sipType_QRect, SIP_NULLPTR, a2, a3, new ::QString(a4), sipType_QString, SIP_NULLPTR);

    sipParseResultEx(sipGILState, sipErrorHandler, sipPySelf, sipMethod, sipResObj, "H5", sipType_QRect, &sipRes);

    return sipRes;
}

::QRect sipVH_progress_indicator_16(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod, const ::QRect& a0, int a1, const ::QPixmap& a2)
{
    ::QRect sipRes;
    PyObject *sipResObj = sipCallMethod(SIP_NULLPTR, sipMethod, "NiN", new ::QRect(a0), sipType_QRect, SIP_NULLPTR, a1, new ::QPixmap(a2), sipType_QPixmap, SIP_NULLPTR);

    sipParseResultEx(sipGILState, sipErrorHandler, sipPySelf, sipMethod, sipResObj, "H5", sipType_QRect, &sipRes);

    return sipRes;
}

void sipVH_progress_indicator_15(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod, ::QPainter*a0, const ::QRect& a1, int a2, const ::QPalette& a3, bool a4, const ::QString& a5, ::QPalette::ColorRole a6)
{
    sipCallProcedureMethod(sipGILState, sipErrorHandler, sipPySelf, sipMethod, "DNiNbNF", a0, sipType_QPainter, SIP_NULLPTR, new ::QRect(a1), sipType_QRect, SIP_NULLPTR, a2, new ::QPalette(a3), sipType_QPalette, SIP_NULLPTR, a4, new ::QString(a5), sipType_QString, SIP_NULLPTR, a6, sipType_QPalette_ColorRole);
}

void sipVH_progress_indicator_14(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod, ::QPainter*a0, const ::QRect& a1, int a2, const ::QPixmap& a3)
{
    sipCallProcedureMethod(sipGILState, sipErrorHandler, sipPySelf, sipMethod, "DNiN", a0, sipType_QPainter, SIP_NULLPTR, new ::QRect(a1), sipType_QRect, SIP_NULLPTR, a2, new ::QPixmap(a3), sipType_QPixmap, SIP_NULLPTR);
}

::QPalette sipVH_progress_indicator_13(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod)
{
    ::QPalette sipRes;
    PyObject *sipResObj = sipCallMethod(SIP_NULLPTR, sipMethod, "");

    sipParseResultEx(sipGILState, sipErrorHandler, sipPySelf, sipMethod, sipResObj, "H5", sipType_QPalette, &sipRes);

    return sipRes;
}

void sipVH_progress_indicator_12(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod, ::QStyle::PrimitiveElement a0, const ::QStyleOption*a1, ::QPainter*a2, const ::QWidget*a3)
{
    sipCallProcedureMethod(sipGILState, sipErrorHandler, sipPySelf, sipMethod, "FDDD", a0, sipType_QStyle_PrimitiveElement, const_cast< ::QStyleOption *>(a1), sipType_QStyleOption, SIP_NULLPTR, a2, sipType_QPainter, SIP_NULLPTR, const_cast< ::QWidget *>(a3), sipType_QWidget, SIP_NULLPTR);
}

void sipVH_progress_indicator_11(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod, ::QStyle::ControlElement a0, const ::QStyleOption*a1, ::QPainter*a2, const ::QWidget*a3)
{
    sipCallProcedureMethod(sipGILState, sipErrorHandler, sipPySelf, sipMethod, "FDDD", a0, sipType_QStyle_ControlElement, const_cast< ::QStyleOption *>(a1), sipType_QStyleOption, SIP_NULLPTR, a2, sipType_QPainter, SIP_NULLPTR, const_cast< ::QWidget *>(a3), sipType_QWidget, SIP_NULLPTR);
}

::QRect sipVH_progress_indicator_10(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod, ::QStyle::SubElement a0, const ::QStyleOption*a1, const ::QWidget*a2)
{
    ::QRect sipRes;
    PyObject *sipResObj = sipCallMethod(SIP_NULLPTR, sipMethod, "FDD", a0, sipType_QStyle_SubElement, const_cast< ::QStyleOption *>(a1), sipType_QStyleOption, SIP_NULLPTR, const_cast< ::QWidget *>(a2), sipType_QWidget, SIP_NULLPTR);

    sipParseResultEx(sipGILState, sipErrorHandler, sipPySelf, sipMethod, sipResObj, "H5", sipType_QRect, &sipRes);

    return sipRes;
}

void sipVH_progress_indicator_9(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod, ::QStyle::ComplexControl a0, const ::QStyleOptionComplex*a1, ::QPainter*a2, const ::QWidget*a3)
{
    sipCallProcedureMethod(sipGILState, sipErrorHandler, sipPySelf, sipMethod, "FDDD", a0, sipType_QStyle_ComplexControl, const_cast< ::QStyleOptionComplex *>(a1), sipType_QStyleOptionComplex, SIP_NULLPTR, a2, sipType_QPainter, SIP_NULLPTR, const_cast< ::QWidget *>(a3), sipType_QWidget, SIP_NULLPTR);
}

::QStyle::SubControl sipVH_progress_indicator_8(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod, ::QStyle::ComplexControl a0, const ::QStyleOptionComplex*a1, const ::QPoint& a2, const ::QWidget*a3)
{
    ::QStyle::SubControl sipRes = ::QStyle::SC_None;
    PyObject *sipResObj = sipCallMethod(SIP_NULLPTR, sipMethod, "FDND", a0, sipType_QStyle_ComplexControl, const_cast< ::QStyleOptionComplex *>(a1), sipType_QStyleOptionComplex, SIP_NULLPTR, new ::QPoint(a2), sipType_QPoint, SIP_NULLPTR, const_cast< ::QWidget *>(a3), sipType_QWidget, SIP_NULLPTR);

    sipParseResultEx(sipGILState, sipErrorHandler, sipPySelf, sipMethod, sipResObj, "F", sipType_QStyle_SubControl, &sipRes);

    return sipRes;
}

::QRect sipVH_progress_indicator_7(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod, ::QStyle::ComplexControl a0, const ::QStyleOptionComplex*a1, ::QStyle::SubControl a2, const ::QWidget*a3)
{
    ::QRect sipRes;
    PyObject *sipResObj = sipCallMethod(SIP_NULLPTR, sipMethod, "FDFD", a0, sipType_QStyle_ComplexControl, const_cast< ::QStyleOptionComplex *>(a1), sipType_QStyleOptionComplex, SIP_NULLPTR, a2, sipType_QStyle_SubControl, const_cast< ::QWidget *>(a3), sipType_QWidget, SIP_NULLPTR);

    sipParseResultEx(sipGILState, sipErrorHandler, sipPySelf, sipMethod, sipResObj, "H5", sipType_QRect, &sipRes);

    return sipRes;
}

int sipVH_progress_indicator_6(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod, ::QStyle::PixelMetric a0, const ::QStyleOption*a1, const ::QWidget*a2)
{
    int sipRes = 0;
    PyObject *sipResObj = sipCallMethod(SIP_NULLPTR, sipMethod, "FDD", a0, sipType_QStyle_PixelMetric, const_cast< ::QStyleOption *>(a1), sipType_QStyleOption, SIP_NULLPTR, const_cast< ::QWidget *>(a2), sipType_QWidget, SIP_NULLPTR);

    sipParseResultEx(sipGILState, sipErrorHandler, sipPySelf, sipMethod, sipResObj, "i", &sipRes);

    return sipRes;
}

::QSize sipVH_progress_indicator_5(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod, ::QStyle::ContentsType a0, const ::QStyleOption*a1, const ::QSize& a2, const ::QWidget*a3)
{
    ::QSize sipRes;
    PyObject *sipResObj = sipCallMethod(SIP_NULLPTR, sipMethod, "FDND", a0, sipType_QStyle_ContentsType, const_cast< ::QStyleOption *>(a1), sipType_QStyleOption, SIP_NULLPTR, new ::QSize(a2), sipType_QSize, SIP_NULLPTR, const_cast< ::QWidget *>(a3), sipType_QWidget, SIP_NULLPTR);

    sipParseResultEx(sipGILState, sipErrorHandler, sipPySelf, sipMethod, sipResObj, "H5", sipType_QSize, &sipRes);

    return sipRes;
}

int sipVH_progress_indicator_4(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod, ::QStyle::StyleHint a0, const ::QStyleOption*a1, const ::QWidget*a2, ::QStyleHintReturn*a3)
{
    int sipRes = 0;
    PyObject *sipResObj = sipCallMethod(SIP_NULLPTR, sipMethod, "FDDD", a0, sipType_QStyle_StyleHint, const_cast< ::QStyleOption *>(a1), sipType_QStyleOption, SIP_NULLPTR, const_cast< ::QWidget *>(a2), sipType_QWidget, SIP_NULLPTR, a3, sipType_QStyleHintReturn, SIP_NULLPTR);

    sipParseResultEx(sipGILState, sipErrorHandler, sipPySelf, sipMethod, sipResObj, "i", &sipRes);

    return sipRes;
}

::QPixmap sipVH_progress_indicator_3(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod, ::QStyle::StandardPixmap a0, const ::QStyleOption*a1, const ::QWidget*a2)
{
    ::QPixmap sipRes;
    PyObject *sipResObj = sipCallMethod(SIP_NULLPTR, sipMethod, "FDD", a0, sipType_QStyle_StandardPixmap, const_cast< ::QStyleOption *>(a1), sipType_QStyleOption, SIP_NULLPTR, const_cast< ::QWidget *>(a2), sipType_QWidget, SIP_NULLPTR);

    sipParseResultEx(sipGILState, sipErrorHandler, sipPySelf, sipMethod, sipResObj, "H5", sipType_QPixmap, &sipRes);

    return sipRes;
}

::QIcon sipVH_progress_indicator_2(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod, ::QStyle::StandardPixmap a0, const ::QStyleOption*a1, const ::QWidget*a2)
{
    ::QIcon sipRes;
    PyObject *sipResObj = sipCallMethod(SIP_NULLPTR, sipMethod, "FDD", a0, sipType_QStyle_StandardPixmap, const_cast< ::QStyleOption *>(a1), sipType_QStyleOption, SIP_NULLPTR, const_cast< ::QWidget *>(a2), sipType_QWidget, SIP_NULLPTR);

    sipParseResultEx(sipGILState, sipErrorHandler, sipPySelf, sipMethod, sipResObj, "H5", sipType_QIcon, &sipRes);

    return sipRes;
}

::QPixmap sipVH_progress_indicator_1(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod, ::QIcon::Mode a0, const ::QPixmap& a1, const ::QStyleOption*a2)
{
    ::QPixmap sipRes;
    PyObject *sipResObj = sipCallMethod(SIP_NULLPTR, sipMethod, "FND", a0, sipType_QIcon_Mode, new ::QPixmap(a1), sipType_QPixmap, SIP_NULLPTR, const_cast< ::QStyleOption *>(a2), sipType_QStyleOption, SIP_NULLPTR);

    sipParseResultEx(sipGILState, sipErrorHandler, sipPySelf, sipMethod, sipResObj, "H5", sipType_QPixmap, &sipRes);

    return sipRes;
}

int sipVH_progress_indicator_0(sip_gilstate_t sipGILState, sipVirtErrorHandlerFunc sipErrorHandler, sipSimpleWrapper *sipPySelf, PyObject *sipMethod, ::QSizePolicy::ControlType a0, ::QSizePolicy::ControlType a1, ::Qt::Orientation a2, const ::QStyleOption*a3, const ::QWidget*a4)
{
    int sipRes = 0;
    PyObject *sipResObj = sipCallMethod(SIP_NULLPTR, sipMethod, "FFFDD", a0, sipType_QSizePolicy_ControlType, a1, sipType_QSizePolicy_ControlType, a2, sipType_Qt_Orientation, const_cast< ::QStyleOption *>(a3), sipType_QStyleOption, SIP_NULLPTR, const_cast< ::QWidget *>(a4), sipType_QWidget, SIP_NULLPTR);

    sipParseResultEx(sipGILState, sipErrorHandler, sipPySelf, sipMethod, sipResObj, "i", &sipRes);

    return sipRes;
}


PyDoc_STRVAR(doc_install_qt_translator, "install_qt_translator(v: Any)");

extern "C" {static PyObject *func_install_qt_translator(PyObject *, PyObject *);}
static PyObject *func_install_qt_translator(PyObject *, PyObject *sipArgs)
{
    PyObject *sipParseErr = SIP_NULLPTR;

    {
        PyObject * a0;

        if (sipParseArgs(&sipParseErr, sipArgs, "P0", &a0))
        {
#line 101 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/QProgressIndicator.sip"
    if (PyLong_Check(a0)) install_translator(PyLong_AsVoidPtr(a0));
    else install_translator(NULL);
#line 706 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorcmodule.cpp"

            Py_INCREF(Py_None);
            return Py_None;
        }
    }

    /* Raise an exception if the arguments couldn't be parsed. */
    sipNoFunction(sipParseErr, sipName_install_qt_translator, doc_install_qt_translator);

    return SIP_NULLPTR;
}


PyDoc_STRVAR(doc_icon_from_paths, "icon_from_paths(any_path: str|None, light_path: str|None, dark_path: str|None) -> QIcon");

extern "C" {static PyObject *func_icon_from_paths(PyObject *, PyObject *);}
static PyObject *func_icon_from_paths(PyObject *, PyObject *sipArgs)
{
    PyObject *sipParseErr = SIP_NULLPTR;

    {
        ::QString* a0;
        int a0State = 0;
        ::QString* a1;
        int a1State = 0;
        ::QString* a2;
        int a2State = 0;

        if (sipParseArgs(&sipParseErr, sipArgs, "J1J1J1", sipType_QString, &a0, &a0State, sipType_QString, &a1, &a1State, sipType_QString, &a2, &a2State))
        {
            ::QIcon*sipRes;

            sipRes = new ::QIcon(icon_from_paths(*a0, *a1, *a2));
            sipReleaseType(a0, sipType_QString, a0State);
            sipReleaseType(a1, sipType_QString, a1State);
            sipReleaseType(a2, sipType_QString, a2State);

            return sipConvertFromNewType(sipRes, sipType_QIcon, SIP_NULLPTR);
        }
    }

    /* Raise an exception if the arguments couldn't be parsed. */
    sipNoFunction(sipParseErr, sipName_icon_from_paths, doc_icon_from_paths);

    return SIP_NULLPTR;
}


PyDoc_STRVAR(doc_icon_from_name, "icon_from_name(name: str|None, fallback_data: QByteArray|bytes|bytearray|memoryview) -> QIcon");

extern "C" {static PyObject *func_icon_from_name(PyObject *, PyObject *);}
static PyObject *func_icon_from_name(PyObject *, PyObject *sipArgs)
{
    PyObject *sipParseErr = SIP_NULLPTR;

    {
        ::QString* a0;
        int a0State = 0;
        const ::QByteArray* a1;
        int a1State = 0;

        if (sipParseArgs(&sipParseErr, sipArgs, "J1J1", sipType_QString, &a0, &a0State, sipType_QByteArray, &a1, &a1State))
        {
            ::QIcon*sipRes;

            sipRes = new ::QIcon(icon_from_name(*a0, *a1));
            sipReleaseType(a0, sipType_QString, a0State);
            sipReleaseType(const_cast< ::QByteArray *>(a1), sipType_QByteArray, a1State);

            return sipConvertFromNewType(sipRes, sipType_QIcon, SIP_NULLPTR);
        }
    }

    /* Raise an exception if the arguments couldn't be parsed. */
    sipNoFunction(sipParseErr, sipName_icon_from_name, doc_icon_from_name);

    return SIP_NULLPTR;
}


PyDoc_STRVAR(doc_set_icon_theme, "set_icon_theme(is_dark: bool, has_dark_user_theme: bool, has_light_user_theme: bool, has_any_user_theme: bool)");

extern "C" {static PyObject *func_set_icon_theme(PyObject *, PyObject *);}
static PyObject *func_set_icon_theme(PyObject *, PyObject *sipArgs)
{
    PyObject *sipParseErr = SIP_NULLPTR;

    {
        bool a0;
        bool a1;
        bool a2;
        bool a3;

        if (sipParseArgs(&sipParseErr, sipArgs, "bbbb", &a0, &a1, &a2, &a3))
        {
            set_icon_theme(a0, a1, a2, a3);

            Py_INCREF(Py_None);
            return Py_None;
        }
    }

    /* Raise an exception if the arguments couldn't be parsed. */
    sipNoFunction(sipParseErr, sipName_set_icon_theme, doc_set_icon_theme);

    return SIP_NULLPTR;
}


PyDoc_STRVAR(doc_utf16_slice, "utf16_slice(src: str|None, pos: int, n: int = -1) -> str");

extern "C" {static PyObject *func_utf16_slice(PyObject *, PyObject *);}
static PyObject *func_utf16_slice(PyObject *, PyObject *sipArgs)
{
    PyObject *sipParseErr = SIP_NULLPTR;

    {
        const ::QString* a0;
        int a0State = 0;
        ::qsizetype a1;
        ::qsizetype a2 = -1;

        if (sipParseArgs(&sipParseErr, sipArgs, "J1n|n", sipType_QString, &a0, &a0State, &a1, &a2))
        {
            ::QString*sipRes;

            sipRes = new ::QString(utf16_slice(*a0, a1, a2));
            sipReleaseType(const_cast< ::QString *>(a0), sipType_QString, a0State);

            return sipConvertFromNewType(sipRes, sipType_QString, SIP_NULLPTR);
        }
    }

    /* Raise an exception if the arguments couldn't be parsed. */
    sipNoFunction(sipParseErr, sipName_utf16_slice, doc_utf16_slice);

    return SIP_NULLPTR;
}


PyDoc_STRVAR(doc_contrast_ratio, "contrast_ratio(c1: QColor|Qt.GlobalColor|int, c2: QColor|Qt.GlobalColor|int) -> float");

extern "C" {static PyObject *func_contrast_ratio(PyObject *, PyObject *);}
static PyObject *func_contrast_ratio(PyObject *, PyObject *sipArgs)
{
    PyObject *sipParseErr = SIP_NULLPTR;

    {
        const ::QColor* a0;
        int a0State = 0;
        const ::QColor* a1;
        int a1State = 0;

        if (sipParseArgs(&sipParseErr, sipArgs, "J1J1", sipType_QColor, &a0, &a0State, sipType_QColor, &a1, &a1State))
        {
            double sipRes;

            sipRes = contrast_ratio(*a0, *a1);
            sipReleaseType(const_cast< ::QColor *>(a0), sipType_QColor, a0State);
            sipReleaseType(const_cast< ::QColor *>(a1), sipType_QColor, a1State);

            return PyFloat_FromDouble(sipRes);
        }
    }

    /* Raise an exception if the arguments couldn't be parsed. */
    sipNoFunction(sipParseErr, sipName_contrast_ratio, doc_contrast_ratio);

    return SIP_NULLPTR;
}


PyDoc_STRVAR(doc_image_from_hbitmap, "image_from_hbitmap(a0: PyQt6.sip.voidptr|None) -> QImage");

extern "C" {static PyObject *func_image_from_hbitmap(PyObject *, PyObject *);}
static PyObject *func_image_from_hbitmap(PyObject *, PyObject *sipArgs)
{
    PyObject *sipParseErr = SIP_NULLPTR;

    {
        void* a0;

        if (sipParseArgs(&sipParseErr, sipArgs, "v", &a0))
        {
            ::QImage*sipRes;

            sipRes = new ::QImage(image_from_hbitmap(a0));

            return sipConvertFromNewType(sipRes, sipType_QImage, SIP_NULLPTR);
        }
    }

    /* Raise an exception if the arguments couldn't be parsed. */
    sipNoFunction(sipParseErr, sipName_image_from_hbitmap, doc_image_from_hbitmap);

    return SIP_NULLPTR;
}


PyDoc_STRVAR(doc_image_from_hicon, "image_from_hicon(a0: PyQt6.sip.voidptr|None) -> QImage");

extern "C" {static PyObject *func_image_from_hicon(PyObject *, PyObject *);}
static PyObject *func_image_from_hicon(PyObject *, PyObject *sipArgs)
{
    PyObject *sipParseErr = SIP_NULLPTR;

    {
        void* a0;

        if (sipParseArgs(&sipParseErr, sipArgs, "v", &a0))
        {
            ::QImage*sipRes;

            sipRes = new ::QImage(image_from_hicon(a0));

            return sipConvertFromNewType(sipRes, sipType_QImage, SIP_NULLPTR);
        }
    }

    /* Raise an exception if the arguments couldn't be parsed. */
    sipNoFunction(sipParseErr, sipName_image_from_hicon, doc_image_from_hicon);

    return SIP_NULLPTR;
}


PyDoc_STRVAR(doc_get_image_allocation_limit, "get_image_allocation_limit() -> int");

extern "C" {static PyObject *func_get_image_allocation_limit(PyObject *, PyObject *);}
static PyObject *func_get_image_allocation_limit(PyObject *, PyObject *sipArgs)
{
    PyObject *sipParseErr = SIP_NULLPTR;

    {
        if (sipParseArgs(&sipParseErr, sipArgs, ""))
        {
            int sipRes;

            sipRes = get_image_allocation_limit();

            return PyLong_FromLong(sipRes);
        }
    }

    /* Raise an exception if the arguments couldn't be parsed. */
    sipNoFunction(sipParseErr, sipName_get_image_allocation_limit, doc_get_image_allocation_limit);

    return SIP_NULLPTR;
}


PyDoc_STRVAR(doc_set_image_allocation_limit, "set_image_allocation_limit(megabytes: int)");

extern "C" {static PyObject *func_set_image_allocation_limit(PyObject *, PyObject *);}
static PyObject *func_set_image_allocation_limit(PyObject *, PyObject *sipArgs)
{
    PyObject *sipParseErr = SIP_NULLPTR;

    {
        int a0;

        if (sipParseArgs(&sipParseErr, sipArgs, "i", &a0))
        {
            set_image_allocation_limit(a0);

            Py_INCREF(Py_None);
            return Py_None;
        }
    }

    /* Raise an exception if the arguments couldn't be parsed. */
    sipNoFunction(sipParseErr, sipName_set_image_allocation_limit, doc_set_image_allocation_limit);

    return SIP_NULLPTR;
}


PyDoc_STRVAR(doc_menu_for_action, "menu_for_action(ac: QAction|None) -> QMenu|None");

extern "C" {static PyObject *func_menu_for_action(PyObject *, PyObject *);}
static PyObject *func_menu_for_action(PyObject *, PyObject *sipArgs)
{
    PyObject *sipParseErr = SIP_NULLPTR;

    {
        const ::QAction* a0;

        if (sipParseArgs(&sipParseErr, sipArgs, "J8", sipType_QAction, &a0))
        {
            ::QMenu*sipRes;

            sipRes = menu_for_action(a0);

            return sipConvertFromType(sipRes, sipType_QMenu, SIP_NULLPTR);
        }
    }

    /* Raise an exception if the arguments couldn't be parsed. */
    sipNoFunction(sipParseErr, sipName_menu_for_action, doc_menu_for_action);

    return SIP_NULLPTR;
}


PyDoc_STRVAR(doc_set_menu_on_action, "set_menu_on_action(ac: QAction|None, menu: QMenu|None)");

extern "C" {static PyObject *func_set_menu_on_action(PyObject *, PyObject *);}
static PyObject *func_set_menu_on_action(PyObject *, PyObject *sipArgs)
{
    PyObject *sipParseErr = SIP_NULLPTR;

    {
        ::QAction* a0;
        ::QMenu* a1;

        if (sipParseArgs(&sipParseErr, sipArgs, "J8J8", sipType_QAction, &a0, sipType_QMenu, &a1))
        {
            set_menu_on_action(a0, a1);

            Py_INCREF(Py_None);
            return Py_None;
        }
    }

    /* Raise an exception if the arguments couldn't be parsed. */
    sipNoFunction(sipParseErr, sipName_set_menu_on_action, doc_set_menu_on_action);

    return SIP_NULLPTR;
}


PyDoc_STRVAR(doc_draw_snake_spinner, "draw_snake_spinner(painter: QPainter, rect: QRect, angle: int, light: QColor|Qt.GlobalColor|int, dark: QColor|Qt.GlobalColor|int)");

extern "C" {static PyObject *func_draw_snake_spinner(PyObject *, PyObject *);}
static PyObject *func_draw_snake_spinner(PyObject *, PyObject *sipArgs)
{
    PyObject *sipParseErr = SIP_NULLPTR;

    {
        ::QPainter* a0;
        ::QRect* a1;
        int a2;
        const ::QColor* a3;
        int a3State = 0;
        const ::QColor* a4;
        int a4State = 0;

        if (sipParseArgs(&sipParseErr, sipArgs, "J9J9iJ1J1", sipType_QPainter, &a0, sipType_QRect, &a1, &a2, sipType_QColor, &a3, &a3State, sipType_QColor, &a4, &a4State))
        {
            Py_BEGIN_ALLOW_THREADS
            draw_snake_spinner(*a0, *a1, a2, *a3, *a4);
            Py_END_ALLOW_THREADS
            sipReleaseType(const_cast< ::QColor *>(a3), sipType_QColor, a3State);
            sipReleaseType(const_cast< ::QColor *>(a4), sipType_QColor, a4State);

            Py_INCREF(Py_None);
            return Py_None;
        }
    }

    /* Raise an exception if the arguments couldn't be parsed. */
    sipNoFunction(sipParseErr, sipName_draw_snake_spinner, doc_draw_snake_spinner);

    return SIP_NULLPTR;
}


PyDoc_STRVAR(doc_set_no_activate_on_click, "set_no_activate_on_click(widget: QWidget|None)");

extern "C" {static PyObject *func_set_no_activate_on_click(PyObject *, PyObject *);}
static PyObject *func_set_no_activate_on_click(PyObject *, PyObject *sipArgs)
{
    PyObject *sipParseErr = SIP_NULLPTR;

    {
        ::QWidget* a0;

        if (sipParseArgs(&sipParseErr, sipArgs, "J8", sipType_QWidget, &a0))
        {
            set_no_activate_on_click(a0);

            Py_INCREF(Py_None);
            return Py_None;
        }
    }

    /* Raise an exception if the arguments couldn't be parsed. */
    sipNoFunction(sipParseErr, sipName_set_no_activate_on_click, doc_set_no_activate_on_click);

    return SIP_NULLPTR;
}


/*
 * This defines each type in this module.
 */
sipTypeDef *sipExportedTypes_progress_indicator[] = {
    &sipTypeDef_progress_indicator_CalibreStyle.ctd_base,
    &sipTypeDef_progress_indicator_QProgressIndicator.ctd_base,
    &sipTypeDef_progress_indicator_SpinAnimator.ctd_base,
};


/* This defines the types that this module needs to import from QtCore. */
sipImportedTypeDef sipImportedTypes_progress_indicator_QtCore[] = {
    {"QByteArray"},
    {"QChildEvent"},
    {"QEvent"},
    {"QMetaMethod"},
    {"QObject"},
    {"QPoint"},
    {"QRect"},
    {"QSize"},
    {"QString"},
    {"QTimerEvent"},
    {"QVariant"},
    {"Qt::InputMethodQuery"},
    {"Qt::Orientation"},
    {"qintptr"},
    {SIP_NULLPTR}
};


/*
 * This defines the virtual error handlers that this module needs to import
 * from QtCore.
 */
sipImportedVirtErrorHandlerDef sipImportedVirtErrorHandlers_progress_indicator_QtCore[] = {
    {"PyQt6"},
    {SIP_NULLPTR}
};


/* This defines the types that this module needs to import from QtGui. */
sipImportedTypeDef sipImportedTypes_progress_indicator_QtGui[] = {
    {"QAction"},
    {"QActionEvent"},
    {"QCloseEvent"},
    {"QColor"},
    {"QContextMenuEvent"},
    {"QDragEnterEvent"},
    {"QDragLeaveEvent"},
    {"QDragMoveEvent"},
    {"QDropEvent"},
    {"QEnterEvent"},
    {"QFocusEvent"},
    {"QFontMetrics"},
    {"QHideEvent"},
    {"QIcon"},
    {"QIcon::Mode"},
    {"QImage"},
    {"QInputMethodEvent"},
    {"QKeyEvent"},
    {"QMouseEvent"},
    {"QMoveEvent"},
    {"QPaintDevice"},
    {"QPaintDevice::PaintDeviceMetric"},
    {"QPaintEngine"},
    {"QPaintEvent"},
    {"QPainter"},
    {"QPalette"},
    {"QPalette::ColorRole"},
    {"QPixmap"},
    {"QResizeEvent"},
    {"QShowEvent"},
    {"QTabletEvent"},
    {"QWheelEvent"},
    {SIP_NULLPTR}
};


/* This defines the types that this module needs to import from QtWidgets. */
sipImportedTypeDef sipImportedTypes_progress_indicator_QtWidgets[] = {
    {"QApplication"},
    {"QCommonStyle"},
    {"QMenu"},
    {"QProxyStyle"},
    {"QSizePolicy::ControlType"},
    {"QStyle"},
    {"QStyle::ComplexControl"},
    {"QStyle::ContentsType"},
    {"QStyle::ControlElement"},
    {"QStyle::PixelMetric"},
    {"QStyle::PrimitiveElement"},
    {"QStyle::StandardPixmap"},
    {"QStyle::StyleHint"},
    {"QStyle::SubControl"},
    {"QStyle::SubElement"},
    {"QStyleHintReturn"},
    {"QStyleOption"},
    {"QStyleOptionComplex"},
    {"QWidget"},
    {SIP_NULLPTR}
};


/* This defines the modules that this module needs to import. */
static sipImportedModuleDef importsTable[] = {
    {"PyQt6.QtCore", sipImportedTypes_progress_indicator_QtCore, sipImportedVirtErrorHandlers_progress_indicator_QtCore, SIP_NULLPTR},
    {"PyQt6.QtGui", sipImportedTypes_progress_indicator_QtGui, SIP_NULLPTR, SIP_NULLPTR},
    {"PyQt6.QtWidgets", sipImportedTypes_progress_indicator_QtWidgets, SIP_NULLPTR, SIP_NULLPTR},
    {SIP_NULLPTR, SIP_NULLPTR, SIP_NULLPTR, SIP_NULLPTR}
};


/* This defines this module. */
sipExportedModuleDef sipModuleAPI_progress_indicator = {
    SIP_NULLPTR,
    8,
    sipNameNr_progress_indicator,
    0,
    sipStrings_progress_indicator,
    importsTable,
    3,
    sipExportedTypes_progress_indicator,
    SIP_NULLPTR,
    0,
    SIP_NULLPTR,
    SIP_NULLPTR,
    SIP_NULLPTR,
    {SIP_NULLPTR, SIP_NULLPTR, SIP_NULLPTR, SIP_NULLPTR, SIP_NULLPTR, SIP_NULLPTR, SIP_NULLPTR, SIP_NULLPTR, SIP_NULLPTR, SIP_NULLPTR},
    SIP_NULLPTR,
    SIP_NULLPTR,
    SIP_NULLPTR,
    SIP_NULLPTR,
    SIP_NULLPTR,
    SIP_NULLPTR,
    SIP_NULLPTR,
};


/* The SIP API and the APIs of any imported modules. */
const sipAPIDef *sipAPI_progress_indicator;

sip_qt_metaobject_func sip_progress_indicator_qt_metaobject;
sip_qt_metacall_func sip_progress_indicator_qt_metacall;
sip_qt_metacast_func sip_progress_indicator_qt_metacast;


/* The Python module initialisation function. */
#if defined(SIP_STATIC_MODULE)
extern "C" PyObject *PyInit_progress_indicator()
#else
PyMODINIT_FUNC PyInit_progress_indicator()
#endif
{
    static PyMethodDef sip_methods[] = {
        {sipName_install_qt_translator, func_install_qt_translator, METH_VARARGS, doc_install_qt_translator},
        {sipName_icon_from_paths, func_icon_from_paths, METH_VARARGS, doc_icon_from_paths},
        {sipName_icon_from_name, func_icon_from_name, METH_VARARGS, doc_icon_from_name},
        {sipName_set_icon_theme, func_set_icon_theme, METH_VARARGS, doc_set_icon_theme},
        {sipName_utf16_slice, func_utf16_slice, METH_VARARGS, doc_utf16_slice},
        {sipName_contrast_ratio, func_contrast_ratio, METH_VARARGS, doc_contrast_ratio},
        {sipName_image_from_hbitmap, func_image_from_hbitmap, METH_VARARGS, doc_image_from_hbitmap},
        {sipName_image_from_hicon, func_image_from_hicon, METH_VARARGS, doc_image_from_hicon},
        {sipName_get_image_allocation_limit, func_get_image_allocation_limit, METH_VARARGS, doc_get_image_allocation_limit},
        {sipName_set_image_allocation_limit, func_set_image_allocation_limit, METH_VARARGS, doc_set_image_allocation_limit},
        {sipName_menu_for_action, func_menu_for_action, METH_VARARGS, doc_menu_for_action},
        {sipName_set_menu_on_action, func_set_menu_on_action, METH_VARARGS, doc_set_menu_on_action},
        {sipName_draw_snake_spinner, func_draw_snake_spinner, METH_VARARGS, doc_draw_snake_spinner},
        {sipName_set_no_activate_on_click, func_set_no_activate_on_click, METH_VARARGS, doc_set_no_activate_on_click},
        {SIP_NULLPTR, SIP_NULLPTR, 0, SIP_NULLPTR}
    };
    static PyModuleDef sip_module_def = {
        PyModuleDef_HEAD_INIT,
        "progress_indicator",
        SIP_NULLPTR,
        -1,
        sip_methods,
        SIP_NULLPTR,
        SIP_NULLPTR,
        SIP_NULLPTR,
        SIP_NULLPTR
    };

    PyObject *sipModule, *sipModuleDict;
    PyObject *sip_sipmod, *sip_capiobj;

    /* Initialise the module and get it's dictionary. */
    if ((sipModule = PyModule_Create(&sip_module_def)) == SIP_NULLPTR)
        return SIP_NULLPTR;

    sipModuleDict = PyModule_GetDict(sipModule);

    /* Get the SIP module's API. */
    if ((sip_sipmod = PyImport_ImportModule("PyQt6.sip")) == SIP_NULLPTR)
    {
        Py_DECREF(sipModule);
        return SIP_NULLPTR;
    }

    sip_capiobj = PyDict_GetItemString(PyModule_GetDict(sip_sipmod), "_C_API");
    Py_DECREF(sip_sipmod);

    if (sip_capiobj == SIP_NULLPTR || !PyCapsule_CheckExact(sip_capiobj))
    {
        PyErr_SetString(PyExc_AttributeError, "PyQt6.sip._C_API is missing or has the wrong type");
        Py_DECREF(sipModule);
        return SIP_NULLPTR;
    }

    sipAPI_progress_indicator = reinterpret_cast<const sipAPIDef *>(PyCapsule_GetPointer(sip_capiobj, "PyQt6.sip._C_API"));

    if (sipAPI_progress_indicator == SIP_NULLPTR)
    {
        Py_DECREF(sipModule);
        return SIP_NULLPTR;
    }

    /* Export the module and publish it's API. */
    if (sipExportModule(&sipModuleAPI_progress_indicator, 13, 8, 0) < 0)
    {
        Py_DECREF(sipModule);
        return SIP_NULLPTR;
    }


    sip_progress_indicator_qt_metaobject = (sip_qt_metaobject_func)sipImportSymbol("qtcore_qt_metaobject");
    sip_progress_indicator_qt_metacall = (sip_qt_metacall_func)sipImportSymbol("qtcore_qt_metacall");
    sip_progress_indicator_qt_metacast = (sip_qt_metacast_func)sipImportSymbol("qtcore_qt_metacast");

    if (!sip_progress_indicator_qt_metacast)
        Py_FatalError("Unable to import qtcore_qt_metacast");

    /* Initialise the module now all its dependencies have been set up. */
    if (sipInitModule(&sipModuleAPI_progress_indicator, sipModuleDict) < 0)
    {
        Py_DECREF(sipModule);
        return SIP_NULLPTR;
    }

    return sipModule;
}
