/*
 * Internal module API header file.
 *
 * Generated by SIP 6.16.1
 */

#ifndef _progress_indicatorAPI_H
#define _progress_indicatorAPI_H

#include "sip.h"

#include <QMetaType>
#include <QThread>

/* These are the qualifiers that are enabled. */
#define SIP_FEATURE_PyQt_Wayland
#define SIP_FEATURE_PyQt_XCB
#define SIP_FEATURE_PyQt_Permissions
#define SIP_FEATURE_PyQt_DTLS
#define SIP_FEATURE_PyQt_WebChannel
#define SIP_FEATURE_PyQt_Process
#define SIP_FEATURE_PyQt_OpenGL
#define SIP_FEATURE_PyQt_RawFont
#define SIP_FEATURE_PyQt_PrintPreviewDialog
#define SIP_FEATURE_PyQt_PrintPreviewWidget
#define SIP_FEATURE_PyQt_Printer
#define SIP_FEATURE_PyQt_PrintDialog
#define SIP_FEATURE_PyQt_qreal_double
#define SIP_FEATURE_PyQt_SSL
#define SIP_FEATURE_PyQt_SessionManager
#define SIP_FEATURE_PyQt_Accessibility
#define SIP_PLATFORM_Linux
#define SIP_TIMELINE_Qt_6_11_0


/*
 * Convenient names to refer to various strings defined in this module.
 * Only the class names are part of the public API.
 */
#define sipNameNr_get_image_allocation_limit 0
#define sipName_get_image_allocation_limit &sipStrings_progress_indicator[0]
#define sipNameNr_set_image_allocation_limit 27
#define sipName_set_image_allocation_limit &sipStrings_progress_indicator[27]
#define sipNameNr_set_no_activate_on_click 54
#define sipName_set_no_activate_on_click &sipStrings_progress_indicator[54]
#define sipNameNr_hitTestComplexControl 79
#define sipName_hitTestComplexControl &sipStrings_progress_indicator[79]
#define sipNameNr_install_qt_translator 101
#define sipName_install_qt_translator &sipStrings_progress_indicator[101]
#define sipNameNr_mouseDoubleClickEvent 123
#define sipName_mouseDoubleClickEvent &sipStrings_progress_indicator[123]
#define sipNameNr_running_state_changed 145
#define sipName_running_state_changed &sipStrings_progress_indicator[145]
#define sipNameNr_get_overall_rotation 167
#define sipName_get_overall_rotation &sipStrings_progress_indicator[167]
#define sipNameNr_set_overall_rotation 188
#define sipName_set_overall_rotation &sipStrings_progress_indicator[188]
#define sipNameNr_generatedIconPixmap 209
#define sipName_generatedIconPixmap &sipStrings_progress_indicator[209]
#define sipNameNr_QProgressIndicator 229
#define sipName_QProgressIndicator &sipStrings_progress_indicator[229]
#define sipNameNr_drawComplexControl 248
#define sipName_drawComplexControl &sipStrings_progress_indicator[248]
#define sipNameNr_draw_snake_spinner 267
#define sipName_draw_snake_spinner &sipStrings_progress_indicator[267]
#define sipNameNr_focusNextPrevChild 286
#define sipName_focusNextPrevChild &sipStrings_progress_indicator[286]
#define sipNameNr_image_from_hbitmap 305
#define sipName_image_from_hbitmap &sipStrings_progress_indicator[305]
#define sipNameNr_progress_indicator 324
#define sipName_progress_indicator &sipStrings_progress_indicator[324]
#define sipNameNr_set_menu_on_action 343
#define sipName_set_menu_on_action &sipStrings_progress_indicator[343]
#define sipNameNr_hasHeightForWidth 362
#define sipName_hasHeightForWidth &sipStrings_progress_indicator[362]
#define sipNameNr_mouseReleaseEvent 380
#define sipName_mouseReleaseEvent &sipStrings_progress_indicator[380]
#define sipNameNr_contextMenuEvent 398
#define sipName_contextMenuEvent &sipStrings_progress_indicator[398]
#define sipNameNr_disconnectNotify 415
#define sipName_disconnectNotify &sipStrings_progress_indicator[415]
#define sipNameNr_get_arc_rotation 432
#define sipName_get_arc_rotation &sipStrings_progress_indicator[432]
#define sipNameNr_image_from_hicon 449
#define sipName_image_from_hicon &sipStrings_progress_indicator[449]
#define sipNameNr_inputMethodEvent 466
#define sipName_inputMethodEvent &sipStrings_progress_indicator[466]
#define sipNameNr_inputMethodQuery 483
#define sipName_inputMethodQuery &sipStrings_progress_indicator[483]
#define sipNameNr_set_arc_rotation 500
#define sipName_set_arc_rotation &sipStrings_progress_indicator[500]
#define sipNameNr_sizeFromContents 517
#define sipName_sizeFromContents &sipStrings_progress_indicator[517]
#define sipNameNr_icon_from_paths 534
#define sipName_icon_from_paths &sipStrings_progress_indicator[534]
#define sipNameNr_keyReleaseEvent 550
#define sipName_keyReleaseEvent &sipStrings_progress_indicator[550]
#define sipNameNr_menu_for_action 566
#define sipName_menu_for_action &sipStrings_progress_indicator[566]
#define sipNameNr_minimumSizeHint 582
#define sipName_minimumSizeHint &sipStrings_progress_indicator[582]
#define sipNameNr_mousePressEvent 598
#define sipName_mousePressEvent &sipStrings_progress_indicator[598]
#define sipNameNr_standardPalette 614
#define sipName_standardPalette &sipStrings_progress_indicator[614]
#define sipNameNr_contrast_ratio 630
#define sipName_contrast_ratio &sipStrings_progress_indicator[630]
#define sipNameNr_dragEnterEvent 645
#define sipName_dragEnterEvent &sipStrings_progress_indicator[645]
#define sipNameNr_dragLeaveEvent 660
#define sipName_dragLeaveEvent &sipStrings_progress_indicator[660]
#define sipNameNr_drawItemPixmap 675
#define sipName_drawItemPixmap &sipStrings_progress_indicator[675]
#define sipNameNr_get_arc_length 690
#define sipName_get_arc_length &sipStrings_progress_indicator[690]
#define sipNameNr_heightForWidth 705
#define sipName_heightForWidth &sipStrings_progress_indicator[705]
#define sipNameNr_icon_from_name 720
#define sipName_icon_from_name &sipStrings_progress_indicator[720]
#define sipNameNr_itemPixmapRect 735
#define sipName_itemPixmapRect &sipStrings_progress_indicator[735]
#define sipNameNr_mouseMoveEvent 750
#define sipName_mouseMoveEvent &sipStrings_progress_indicator[750]
#define sipNameNr_setDisplaySize 765
#define sipName_setDisplaySize &sipStrings_progress_indicator[765]
#define sipNameNr_set_arc_length 780
#define sipName_set_arc_length &sipStrings_progress_indicator[780]
#define sipNameNr_set_icon_theme 795
#define sipName_set_icon_theme &sipStrings_progress_indicator[795]
#define sipNameNr_standardPixmap 810
#define sipName_standardPixmap &sipStrings_progress_indicator[810]
#define sipNameNr_startAnimation 825
#define sipName_startAnimation &sipStrings_progress_indicator[825]
#define sipNameNr_subControlRect 840
#define sipName_subControlRect &sipStrings_progress_indicator[840]
#define sipNameNr_subElementRect 855
#define sipName_subElementRect &sipStrings_progress_indicator[855]
#define sipNameNr_connectNotify 418
#define sipName_connectNotify &sipStrings_progress_indicator[418]
#define sipNameNr_dragMoveEvent 870
#define sipName_dragMoveEvent &sipStrings_progress_indicator[870]
#define sipNameNr_drawPrimitive 884
#define sipName_drawPrimitive &sipStrings_progress_indicator[884]
#define sipNameNr_focusOutEvent 898
#define sipName_focusOutEvent &sipStrings_progress_indicator[898]
#define sipNameNr_keyPressEvent 912
#define sipName_keyPressEvent &sipStrings_progress_indicator[912]
#define sipNameNr_layoutSpacing 926
#define sipName_layoutSpacing &sipStrings_progress_indicator[926]
#define sipNameNr_sharedPainter 940
#define sipName_sharedPainter &sipStrings_progress_indicator[940]
#define sipNameNr_stopAnimation 954
#define sipName_stopAnimation &sipStrings_progress_indicator[954]
#define sipNameNr_CalibreStyle 968
#define sipName_CalibreStyle &sipStrings_progress_indicator[968]
#define sipNameNr_SpinAnimator 981
#define sipName_SpinAnimator &sipStrings_progress_indicator[981]
#define sipNameNr_drawItemText 994
#define sipName_drawItemText &sipStrings_progress_indicator[994]
#define sipNameNr_focusInEvent 1007
#define sipName_focusInEvent &sipStrings_progress_indicator[1007]
#define sipNameNr_itemTextRect 1020
#define sipName_itemTextRect &sipStrings_progress_indicator[1020]
#define sipNameNr_standardIcon 1033
#define sipName_standardIcon &sipStrings_progress_indicator[1033]
#define sipNameNr_actionEvent 1046
#define sipName_actionEvent &sipStrings_progress_indicator[1046]
#define sipNameNr_changeEvent 1058
#define sipName_changeEvent &sipStrings_progress_indicator[1058]
#define sipNameNr_customEvent 1070
#define sipName_customEvent &sipStrings_progress_indicator[1070]
#define sipNameNr_displaySize 1082
#define sipName_displaySize &sipStrings_progress_indicator[1082]
#define sipNameNr_drawControl 1094
#define sipName_drawControl &sipStrings_progress_indicator[1094]
#define sipNameNr_eventFilter 1106
#define sipName_eventFilter &sipStrings_progress_indicator[1106]
#define sipNameNr_initPainter 1118
#define sipName_initPainter &sipStrings_progress_indicator[1118]
#define sipNameNr_nativeEvent 1130
#define sipName_nativeEvent &sipStrings_progress_indicator[1130]
#define sipNameNr_paintEngine 1142
#define sipName_paintEngine &sipStrings_progress_indicator[1142]
#define sipNameNr_pixelMetric 1154
#define sipName_pixelMetric &sipStrings_progress_indicator[1154]
#define sipNameNr_resizeEvent 1166
#define sipName_resizeEvent &sipStrings_progress_indicator[1166]
#define sipNameNr_setSizeHint 1178
#define sipName_setSizeHint &sipStrings_progress_indicator[1178]
#define sipNameNr_tabletEvent 1190
#define sipName_tabletEvent &sipStrings_progress_indicator[1190]
#define sipNameNr_utf16_slice 1202
#define sipName_utf16_slice &sipStrings_progress_indicator[1202]
#define sipNameNr_childEvent 1214
#define sipName_childEvent &sipStrings_progress_indicator[1214]
#define sipNameNr_closeEvent 1225
#define sipName_closeEvent &sipStrings_progress_indicator[1225]
#define sipNameNr_enterEvent 1236
#define sipName_enterEvent &sipStrings_progress_indicator[1236]
#define sipNameNr_isAnimated 1247
#define sipName_isAnimated &sipStrings_progress_indicator[1247]
#define sipNameNr_is_running 1258
#define sipName_is_running &sipStrings_progress_indicator[1258]
#define sipNameNr_leaveEvent 1269
#define sipName_leaveEvent &sipStrings_progress_indicator[1269]
#define sipNameNr_paintEvent 1280
#define sipName_paintEvent &sipStrings_progress_indicator[1280]
#define sipNameNr_setVisible 1291
#define sipName_setVisible &sipStrings_progress_indicator[1291]
#define sipNameNr_timerEvent 1302
#define sipName_timerEvent &sipStrings_progress_indicator[1302]
#define sipNameNr_wheelEvent 1313
#define sipName_wheelEvent &sipStrings_progress_indicator[1313]
#define sipNameNr_dropEvent 1324
#define sipName_dropEvent &sipStrings_progress_indicator[1324]
#define sipNameNr_hideEvent 1334
#define sipName_hideEvent &sipStrings_progress_indicator[1334]
#define sipNameNr_moveEvent 1344
#define sipName_moveEvent &sipStrings_progress_indicator[1344]
#define sipNameNr_showEvent 1354
#define sipName_showEvent &sipStrings_progress_indicator[1354]
#define sipNameNr_styleHint 1364
#define sipName_styleHint &sipStrings_progress_indicator[1364]
#define sipNameNr_sizeHint 1374
#define sipName_sizeHint &sipStrings_progress_indicator[1374]
#define sipNameNr_unpolish 1383
#define sipName_unpolish &sipStrings_progress_indicator[1383]
#define sipNameNr_devType 1392
#define sipName_devType &sipStrings_progress_indicator[1392]
#define sipNameNr_updated 1400
#define sipName_updated &sipStrings_progress_indicator[1400]
#define sipNameNr_metric 1408
#define sipName_metric &sipStrings_progress_indicator[1408]
#define sipNameNr_polish 1385
#define sipName_polish &sipStrings_progress_indicator[1385]
#define sipNameNr_event 1415
#define sipName_event &sipStrings_progress_indicator[1415]
#define sipNameNr_start 1421
#define sipName_start &sipStrings_progress_indicator[1421]
#define sipNameNr_draw 1427
#define sipName_draw &sipStrings_progress_indicator[1427]
#define sipNameNr_stop 1432
#define sipName_stop &sipStrings_progress_indicator[1432]

#define sipMalloc                   sipAPI_progress_indicator->api_malloc
#define sipFree                     sipAPI_progress_indicator->api_free
#define sipBuildResult              sipAPI_progress_indicator->api_build_result
#define sipCallMethod               sipAPI_progress_indicator->api_call_method
#define sipCallProcedureMethod      sipAPI_progress_indicator->api_call_procedure_method
#define sipCallErrorHandler         sipAPI_progress_indicator->api_call_error_handler
#define sipParseResultEx            sipAPI_progress_indicator->api_parse_result_ex
#define sipParseResult              sipAPI_progress_indicator->api_parse_result
#define sipParseArgs                sipAPI_progress_indicator->api_parse_args
#define sipParseKwdArgs             sipAPI_progress_indicator->api_parse_kwd_args
#define sipParsePair                sipAPI_progress_indicator->api_parse_pair
#define sipInstanceDestroyed        sipAPI_progress_indicator->api_instance_destroyed
#define sipInstanceDestroyedEx      sipAPI_progress_indicator->api_instance_destroyed_ex
#define sipConvertFromSequenceIndex sipAPI_progress_indicator->api_convert_from_sequence_index
#define sipConvertFromSliceObject   sipAPI_progress_indicator->api_convert_from_slice_object
#define sipConvertFromVoidPtr       sipAPI_progress_indicator->api_convert_from_void_ptr
#define sipConvertToVoidPtr         sipAPI_progress_indicator->api_convert_to_void_ptr
#define sipAddException             sipAPI_progress_indicator->api_add_exception
#define sipNoFunction               sipAPI_progress_indicator->api_no_function
#define sipNoMethod                 sipAPI_progress_indicator->api_no_method
#define sipAbstractMethod           sipAPI_progress_indicator->api_abstract_method
#define sipBadClass                 sipAPI_progress_indicator->api_bad_class
#define sipBadCatcherResult         sipAPI_progress_indicator->api_bad_catcher_result
#define sipBadCallableArg           sipAPI_progress_indicator->api_bad_callable_arg
#define sipBadOperatorArg           sipAPI_progress_indicator->api_bad_operator_arg
#define sipTrace                    sipAPI_progress_indicator->api_trace
#define sipTransferBack             sipAPI_progress_indicator->api_transfer_back
#define sipTransferTo               sipAPI_progress_indicator->api_transfer_to
#define sipSimpleWrapper_Type       sipAPI_progress_indicator->api_simplewrapper_type
#define sipWrapper_Type             sipAPI_progress_indicator->api_wrapper_type
#define sipWrapperType_Type         sipAPI_progress_indicator->api_wrappertype_type
#define sipVoidPtr_Type             sipAPI_progress_indicator->api_voidptr_type
#define sipGetPyObject              sipAPI_progress_indicator->api_get_pyobject
#define sipGetAddress               sipAPI_progress_indicator->api_get_address
#define sipGetMixinAddress          sipAPI_progress_indicator->api_get_mixin_address
#define sipGetCppPtr                sipAPI_progress_indicator->api_get_cpp_ptr
#define sipGetComplexCppPtr         sipAPI_progress_indicator->api_get_complex_cpp_ptr
#define sipCallHook                 sipAPI_progress_indicator->api_call_hook
#define sipEndThread                sipAPI_progress_indicator->api_end_thread
#define sipRaiseUnknownException    sipAPI_progress_indicator->api_raise_unknown_exception
#define sipRaiseTypeException       sipAPI_progress_indicator->api_raise_type_exception
#define sipBadLengthForSlice        sipAPI_progress_indicator->api_bad_length_for_slice
#define sipAddTypeInstance          sipAPI_progress_indicator->api_add_type_instance
#define sipPySlotExtend             sipAPI_progress_indicator->api_pyslot_extend
#define sipAddDelayedDtor           sipAPI_progress_indicator->api_add_delayed_dtor
#define sipCanConvertToType         sipAPI_progress_indicator->api_can_convert_to_type
#define sipConvertToType            sipAPI_progress_indicator->api_convert_to_type
#define sipForceConvertToType       sipAPI_progress_indicator->api_force_convert_to_type
#define sipConvertToEnum            sipAPI_progress_indicator->api_convert_to_enum
#define sipConvertToBool            sipAPI_progress_indicator->api_convert_to_bool
#define sipReleaseType              sipAPI_progress_indicator->api_release_type
#define sipConvertFromType          sipAPI_progress_indicator->api_convert_from_type
#define sipConvertFromNewType       sipAPI_progress_indicator->api_convert_from_new_type
#define sipConvertFromNewPyType     sipAPI_progress_indicator->api_convert_from_new_pytype
#define sipConvertFromEnum          sipAPI_progress_indicator->api_convert_from_enum
#define sipGetState                 sipAPI_progress_indicator->api_get_state
#define sipExportSymbol             sipAPI_progress_indicator->api_export_symbol
#define sipImportSymbol             sipAPI_progress_indicator->api_import_symbol
#define sipFindType                 sipAPI_progress_indicator->api_find_type
#define sipBytes_AsChar             sipAPI_progress_indicator->api_bytes_as_char
#define sipBytes_AsString           sipAPI_progress_indicator->api_bytes_as_string
#define sipString_AsASCIIChar       sipAPI_progress_indicator->api_string_as_ascii_char
#define sipString_AsASCIIString     sipAPI_progress_indicator->api_string_as_ascii_string
#define sipString_AsLatin1Char      sipAPI_progress_indicator->api_string_as_latin1_char
#define sipString_AsLatin1String    sipAPI_progress_indicator->api_string_as_latin1_string
#define sipString_AsUTF8Char        sipAPI_progress_indicator->api_string_as_utf8_char
#define sipString_AsUTF8String      sipAPI_progress_indicator->api_string_as_utf8_string
#define sipUnicode_AsWChar          sipAPI_progress_indicator->api_unicode_as_wchar
#define sipUnicode_AsWString        sipAPI_progress_indicator->api_unicode_as_wstring
#define sipConvertFromConstVoidPtr  sipAPI_progress_indicator->api_convert_from_const_void_ptr
#define sipConvertFromVoidPtrAndSize    sipAPI_progress_indicator->api_convert_from_void_ptr_and_size
#define sipConvertFromConstVoidPtrAndSize   sipAPI_progress_indicator->api_convert_from_const_void_ptr_and_size
#define sipWrappedTypeName(wt)      ((wt)->wt_td->td_cname)
#define sipGetReference             sipAPI_progress_indicator->api_get_reference
#define sipKeepReference            sipAPI_progress_indicator->api_keep_reference
#define sipRegisterProxyResolver    sipAPI_progress_indicator->api_register_proxy_resolver
#define sipRegisterPyType           sipAPI_progress_indicator->api_register_py_type
#define sipTypeFromPyTypeObject     sipAPI_progress_indicator->api_type_from_py_type_object
#define sipTypeScope                sipAPI_progress_indicator->api_type_scope
#define sipResolveTypedef           sipAPI_progress_indicator->api_resolve_typedef
#define sipRegisterAttributeGetter  sipAPI_progress_indicator->api_register_attribute_getter
#define sipEnableAutoconversion     sipAPI_progress_indicator->api_enable_autoconversion
#define sipInitMixin                sipAPI_progress_indicator->api_init_mixin
#define sipExportModule             sipAPI_progress_indicator->api_export_module
#define sipInitModule               sipAPI_progress_indicator->api_init_module
#define sipGetInterpreter           sipAPI_progress_indicator->api_get_interpreter
#define sipSetTypeUserData          sipAPI_progress_indicator->api_set_type_user_data
#define sipGetTypeUserData          sipAPI_progress_indicator->api_get_type_user_data
#define sipPyTypeDict               sipAPI_progress_indicator->api_py_type_dict
#define sipPyTypeName               sipAPI_progress_indicator->api_py_type_name
#define sipGetCFunction             sipAPI_progress_indicator->api_get_c_function
#define sipGetMethod                sipAPI_progress_indicator->api_get_method
#define sipFromMethod               sipAPI_progress_indicator->api_from_method
#define sipGetDate                  sipAPI_progress_indicator->api_get_date
#define sipFromDate                 sipAPI_progress_indicator->api_from_date
#define sipGetDateTime              sipAPI_progress_indicator->api_get_datetime
#define sipFromDateTime             sipAPI_progress_indicator->api_from_datetime
#define sipGetTime                  sipAPI_progress_indicator->api_get_time
#define sipFromTime                 sipAPI_progress_indicator->api_from_time
#define sipIsUserType               sipAPI_progress_indicator->api_is_user_type
#define sipCheckPluginForType       sipAPI_progress_indicator->api_check_plugin_for_type
#define sipUnicodeNew               sipAPI_progress_indicator->api_unicode_new
#define sipUnicodeWrite             sipAPI_progress_indicator->api_unicode_write
#define sipUnicodeData              sipAPI_progress_indicator->api_unicode_data
#define sipGetBufferInfo            sipAPI_progress_indicator->api_get_buffer_info
#define sipReleaseBufferInfo        sipAPI_progress_indicator->api_release_buffer_info
#define sipIsOwnedByPython          sipAPI_progress_indicator->api_is_owned_by_python
#define sipIsDerivedClass           sipAPI_progress_indicator->api_is_derived_class
#define sipGetUserObject            sipAPI_progress_indicator->api_get_user_object
#define sipSetUserObject            sipAPI_progress_indicator->api_set_user_object
#define sipRegisterEventHandler     sipAPI_progress_indicator->api_register_event_handler
#define sipConvertToArray           sipAPI_progress_indicator->api_convert_to_array
#define sipConvertToTypedArray      sipAPI_progress_indicator->api_convert_to_typed_array
#define sipEnableGC                 sipAPI_progress_indicator->api_enable_gc
#define sipPrintObject              sipAPI_progress_indicator->api_print_object
#define sipLong_AsChar              sipAPI_progress_indicator->api_long_as_char
#define sipLong_AsSignedChar        sipAPI_progress_indicator->api_long_as_signed_char
#define sipLong_AsUnsignedChar      sipAPI_progress_indicator->api_long_as_unsigned_char
#define sipLong_AsShort             sipAPI_progress_indicator->api_long_as_short
#define sipLong_AsUnsignedShort     sipAPI_progress_indicator->api_long_as_unsigned_short
#define sipLong_AsInt               sipAPI_progress_indicator->api_long_as_int
#define sipLong_AsUnsignedInt       sipAPI_progress_indicator->api_long_as_unsigned_int
#define sipLong_AsLong              sipAPI_progress_indicator->api_long_as_long
#define sipLong_AsUnsignedLong      sipAPI_progress_indicator->api_long_as_unsigned_long
#define sipLong_AsLongLong          sipAPI_progress_indicator->api_long_as_long_long
#define sipLong_AsUnsignedLongLong  sipAPI_progress_indicator->api_long_as_unsigned_long_long
#define sipLong_AsSizeT             sipAPI_progress_indicator->api_long_as_size_t
#define sipVisitWrappers            sipAPI_progress_indicator->api_visit_wrappers
#define sipRegisterExitNotifier     sipAPI_progress_indicator->api_register_exit_notifier
#define sipDeprecated               sipAPI_progress_indicator->api_deprecated
#define sipPyTypeDictRef            sipAPI_progress_indicator->api_py_type_dict_ref
#define sipNextExceptionHandler     sipAPI_progress_indicator->api_next_exception_handler
#define sipIsEnumFlag               sipAPI_progress_indicator->api_is_enum_flag
#define sipConvertToTypeUS          sipAPI_progress_indicator->api_convert_to_type_us
#define sipForceConvertToTypeUS     sipAPI_progress_indicator->api_force_convert_to_type_us
#define sipReleaseTypeUS            sipAPI_progress_indicator->api_release_type_us
#define sipIsPyMethod               sipAPI_progress_indicator->api_is_py_method_12_8

/* The strings used by this module. */
extern const char sipStrings_progress_indicator[];

#define sipType_CalibreStyle sipExportedTypes_progress_indicator[0]

extern sipClassTypeDef sipTypeDef_progress_indicator_CalibreStyle;

#define sipType_SpinAnimator sipExportedTypes_progress_indicator[2]

extern sipClassTypeDef sipTypeDef_progress_indicator_SpinAnimator;

#define sipType_QProgressIndicator sipExportedTypes_progress_indicator[1]

extern sipClassTypeDef sipTypeDef_progress_indicator_QProgressIndicator;

/* The SIP API, this module's API and the APIs of any imported modules. */
extern const sipAPIDef *sipAPI_progress_indicator;
extern sipExportedModuleDef sipModuleAPI_progress_indicator;
extern sipTypeDef *sipExportedTypes_progress_indicator[];

#define sipType_Qt_InputMethodQuery sipImportedTypes_progress_indicator_QtCore[11].it_td

#define sipType_Qt_Orientation sipImportedTypes_progress_indicator_QtCore[12].it_td

#define sipType_QObject sipImportedTypes_progress_indicator_QtCore[4].it_td

#define sipType_QByteArray sipImportedTypes_progress_indicator_QtCore[0].it_td

#define sipType_QEvent sipImportedTypes_progress_indicator_QtCore[2].it_td

#define sipType_QTimerEvent sipImportedTypes_progress_indicator_QtCore[9].it_td

#define sipType_QChildEvent sipImportedTypes_progress_indicator_QtCore[1].it_td

#define sipType_QMetaMethod sipImportedTypes_progress_indicator_QtCore[3].it_td

#define sipType_QPoint sipImportedTypes_progress_indicator_QtCore[5].it_td

#define sipType_QRect sipImportedTypes_progress_indicator_QtCore[6].it_td

#define sipType_QSize sipImportedTypes_progress_indicator_QtCore[7].it_td

#define sipType_QVariant sipImportedTypes_progress_indicator_QtCore[10].it_td

#define sipType_QString sipImportedTypes_progress_indicator_QtCore[8].it_td

#define sipType_qintptr sipImportedTypes_progress_indicator_QtCore[13].it_td
extern sipImportedTypeDef sipImportedTypes_progress_indicator_QtCore[];
extern sipImportedVirtErrorHandlerDef sipImportedVirtErrorHandlers_progress_indicator_QtCore[];

#define sipType_QAction sipImportedTypes_progress_indicator_QtGui[0].it_td

#define sipType_QPaintDevice sipImportedTypes_progress_indicator_QtGui[20].it_td

#define sipType_QPaintDevice_PaintDeviceMetric sipImportedTypes_progress_indicator_QtGui[21].it_td

#define sipType_QPixmap sipImportedTypes_progress_indicator_QtGui[27].it_td

#define sipType_QColor sipImportedTypes_progress_indicator_QtGui[3].it_td

#define sipType_QKeyEvent sipImportedTypes_progress_indicator_QtGui[17].it_td

#define sipType_QFocusEvent sipImportedTypes_progress_indicator_QtGui[10].it_td

#define sipType_QPaintEvent sipImportedTypes_progress_indicator_QtGui[23].it_td

#define sipType_QMoveEvent sipImportedTypes_progress_indicator_QtGui[19].it_td

#define sipType_QResizeEvent sipImportedTypes_progress_indicator_QtGui[28].it_td

#define sipType_QCloseEvent sipImportedTypes_progress_indicator_QtGui[2].it_td

#define sipType_QShowEvent sipImportedTypes_progress_indicator_QtGui[29].it_td

#define sipType_QHideEvent sipImportedTypes_progress_indicator_QtGui[12].it_td

#define sipType_QContextMenuEvent sipImportedTypes_progress_indicator_QtGui[4].it_td

#define sipType_QInputMethodEvent sipImportedTypes_progress_indicator_QtGui[16].it_td

#define sipType_QDropEvent sipImportedTypes_progress_indicator_QtGui[8].it_td

#define sipType_QDragMoveEvent sipImportedTypes_progress_indicator_QtGui[7].it_td

#define sipType_QDragEnterEvent sipImportedTypes_progress_indicator_QtGui[5].it_td

#define sipType_QDragLeaveEvent sipImportedTypes_progress_indicator_QtGui[6].it_td

#define sipType_QActionEvent sipImportedTypes_progress_indicator_QtGui[1].it_td

#define sipType_QEnterEvent sipImportedTypes_progress_indicator_QtGui[9].it_td

#define sipType_QMouseEvent sipImportedTypes_progress_indicator_QtGui[18].it_td

#define sipType_QWheelEvent sipImportedTypes_progress_indicator_QtGui[31].it_td

#define sipType_QTabletEvent sipImportedTypes_progress_indicator_QtGui[30].it_td

#define sipType_QFontMetrics sipImportedTypes_progress_indicator_QtGui[11].it_td

#define sipType_QIcon sipImportedTypes_progress_indicator_QtGui[13].it_td

#define sipType_QIcon_Mode sipImportedTypes_progress_indicator_QtGui[14].it_td

#define sipType_QImage sipImportedTypes_progress_indicator_QtGui[15].it_td

#define sipType_QPaintEngine sipImportedTypes_progress_indicator_QtGui[22].it_td

#define sipType_QPainter sipImportedTypes_progress_indicator_QtGui[24].it_td

#define sipType_QPalette sipImportedTypes_progress_indicator_QtGui[25].it_td

#define sipType_QPalette_ColorRole sipImportedTypes_progress_indicator_QtGui[26].it_td
extern sipImportedTypeDef sipImportedTypes_progress_indicator_QtGui[];

#define sipType_QWidget sipImportedTypes_progress_indicator_QtWidgets[18].it_td

#define sipType_QApplication sipImportedTypes_progress_indicator_QtWidgets[0].it_td

#define sipType_QStyle sipImportedTypes_progress_indicator_QtWidgets[5].it_td

#define sipType_QStyle_StandardPixmap sipImportedTypes_progress_indicator_QtWidgets[11].it_td

#define sipType_QStyle_StyleHint sipImportedTypes_progress_indicator_QtWidgets[12].it_td

#define sipType_QStyle_ContentsType sipImportedTypes_progress_indicator_QtWidgets[7].it_td

#define sipType_QStyle_PixelMetric sipImportedTypes_progress_indicator_QtWidgets[9].it_td

#define sipType_QStyle_SubControl sipImportedTypes_progress_indicator_QtWidgets[13].it_td

#define sipType_QStyle_ComplexControl sipImportedTypes_progress_indicator_QtWidgets[6].it_td

#define sipType_QStyle_SubElement sipImportedTypes_progress_indicator_QtWidgets[14].it_td

#define sipType_QStyle_ControlElement sipImportedTypes_progress_indicator_QtWidgets[8].it_td

#define sipType_QStyle_PrimitiveElement sipImportedTypes_progress_indicator_QtWidgets[10].it_td

#define sipType_QCommonStyle sipImportedTypes_progress_indicator_QtWidgets[1].it_td

#define sipType_QMenu sipImportedTypes_progress_indicator_QtWidgets[2].it_td

#define sipType_QProxyStyle sipImportedTypes_progress_indicator_QtWidgets[3].it_td

#define sipType_QSizePolicy_ControlType sipImportedTypes_progress_indicator_QtWidgets[4].it_td

#define sipType_QStyleOption sipImportedTypes_progress_indicator_QtWidgets[16].it_td

#define sipType_QStyleOptionComplex sipImportedTypes_progress_indicator_QtWidgets[17].it_td

#define sipType_QStyleHintReturn sipImportedTypes_progress_indicator_QtWidgets[15].it_td
extern sipImportedTypeDef sipImportedTypes_progress_indicator_QtWidgets[];

typedef const QMetaObject *(*sip_qt_metaobject_func)(sipSimpleWrapper *, sipTypeDef *);
extern sip_qt_metaobject_func sip_progress_indicator_qt_metaobject;

typedef int (*sip_qt_metacall_func)(sipSimpleWrapper *, sipTypeDef *, QMetaObject::Call, int, void **);
extern sip_qt_metacall_func sip_progress_indicator_qt_metacall;

typedef bool (*sip_qt_metacast_func)(sipSimpleWrapper *, const sipTypeDef *, const char *, void **);
extern sip_qt_metacast_func sip_progress_indicator_qt_metacast;
#line 9 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/QProgressIndicator.sip"
#include <QPainter>
void set_no_activate_on_click(QWidget *widget);
void draw_snake_spinner(QPainter &painter, QRect rect, int angle, const QColor & light, const QColor & dark);
#line 550 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipAPIprogress_indicator.h"

#endif
