/*
 * Interface wrapper code.
 *
 * Generated by SIP 6.16.1
 */

#include "sipAPIprogress_indicator.h"

#line 50 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/QProgressIndicator.sip"
#include <QProgressIndicator.h>
#line 12 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"

#line 28 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtWidgets/qwidget.sip"
#include <qwidget.h>
#line 16 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 254 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qevent.sip"
#include <qevent.h>
#line 19 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 30 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qsize.sip"
#include <qsize.h>
#line 22 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 24 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qstring.sip"
#include <qstring.h>
#line 25 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 26 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qscreen.sip"
#include <qscreen.h>
#line 28 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 26 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qnamespace.sip"
#include <qnamespace.h>
#line 154 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qtextstream.sip"
#include <qtextstream.h>
#line 33 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 26 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qicon.sip"
#include <qicon.h>
#line 36 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 26 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qpainter.sip"
#include <qpainter.h>
#line 39 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 30 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qbytearray.sip"
#include <qbytearray.h>
#line 42 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 116 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qglobal.sip"
#include <QtGlobal>
#line 45 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 26 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qwindow.sip"
#include <qwindow.h>
#line 48 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 25 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qflags.sip"
#include <qglobal.h>
#line 51 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 26 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qpixmap.sip"
#include <qpixmap.h>
#line 54 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 30 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qrect.sip"
#include <qrect.h>
#line 57 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 25 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qflags.sip"
#include <qglobal.h>
#line 60 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 30 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qmargins.sip"
#include <qmargins.h>
#line 63 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 25 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qflags.sip"
#include <qglobal.h>
#line 66 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 26 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtWidgets/qgraphicseffect.sip"
#include <qgraphicseffect.h>
#line 69 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 26 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtWidgets/qgraphicsproxywidget.sip"
#include <qgraphicsproxywidget.h>
#line 72 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 145 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qglobal.sip"
#include <QtGlobal>
#line 75 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 26 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qlocale.sip"
#include <qlocale.h>
#line 78 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 26 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qpaintdevice.sip"
#include <qpaintdevice.h>
#line 81 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 30 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qpoint.sip"
#include <qpoint.h>
#line 84 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 26 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qregion.sip"
#include <qregion.h>
#line 87 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 25 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qflags.sip"
#include <qglobal.h>
#line 90 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 26 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qfontinfo.sip"
#include <qfontinfo.h>
#line 93 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 26 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qfontmetrics.sip"
#include <qfontmetrics.h>
#line 96 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 26 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qfont.sip"
#include <qfont.h>
#line 99 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 26 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qvariant.sip"
#include <qvariant.h>
#line 102 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 380 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qevent.sip"
#include <qevent.h>
#line 105 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 26 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qcoreevent.sip"
#include <qcoreevent.h>
#line 108 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 339 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qevent.sip"
#include <qevent.h>
#line 111 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 325 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qevent.sip"
#include <qevent.h>
#line 114 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 444 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qevent.sip"
#include <qevent.h>
#line 117 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 497 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qevent.sip"
#include <qevent.h>
#line 120 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 466 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qevent.sip"
#include <qevent.h>
#line 123 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 483 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qevent.sip"
#include <qevent.h>
#line 126 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 555 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qevent.sip"
#include <qevent.h>
#line 129 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 807 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qevent.sip"
#include <qevent.h>
#line 132 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 353 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qevent.sip"
#include <qevent.h>
#line 135 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 297 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qevent.sip"
#include <qevent.h>
#line 138 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 283 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qevent.sip"
#include <qevent.h>
#line 141 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 269 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qevent.sip"
#include <qevent.h>
#line 144 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 742 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qevent.sip"
#include <qevent.h>
#line 147 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 239 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qevent.sip"
#include <qevent.h>
#line 150 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 213 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qevent.sip"
#include <qevent.h>
#line 153 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 788 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qevent.sip"
#include <qevent.h>
#line 156 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 754 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qevent.sip"
#include <qevent.h>
#line 159 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 50 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qpaintengine.sip"
#include <qpaintengine.h>
#line 162 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 110 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qpoint.sip"
#include <qpoint.h>
#line 165 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 135 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qpycore_qlist.sip"
#include <qlist.h>
#line 168 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 28 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qaction.sip"
#include <qaction.h>
#line 171 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 26 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qkeysequence.sip"
#include <qkeysequence.h>
#line 174 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 26 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtWidgets/qlayout.sip"
#include <qlayout.h>
#line 177 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 26 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtWidgets/qsizepolicy.sip"
#include <qsizepolicy.h>
#line 180 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 25 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qflags.sip"
#include <qglobal.h>
#line 183 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 26 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qcursor.sip"
#include <qcursor.h>
#line 186 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 26 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qbitmap.sip"
#include <qbitmap.h>
#line 189 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 26 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtGui/qpalette.sip"
#include <qpalette.h>
#line 192 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 26 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtWidgets/qstyle.sip"
#include <qstyle.h>
#line 195 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 26 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qobjectdefs.sip"
#include <qobjectdefs.h>
#line 154 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qobjectdefs.sip"
#include <qobjectdefs.h>
#line 200 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 26 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qmetaobject.sip"
#include <qmetaobject.h>
#line 203 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 277 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qcoreevent.sip"
#include <qcoreevent.h>
#line 206 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 254 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qcoreevent.sip"
#include <qcoreevent.h>
#line 209 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 28 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qobject.sip"
#include <qobject.h>
#line 212 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 28 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qpycore_qlist.sip"
#include <qlist.h>
#line 215 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 135 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qpycore_qlist.sip"
#include <qlist.h>
#line 218 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 26 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qthread.sip"
#include <qthread.h>
#line 221 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 26 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qanystringview.sip"
#include <qanystringview.h>
#line 224 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 25 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qflags.sip"
#include <qglobal.h>
#line 227 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 26 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qregularexpression.sip"
#include <qregularexpression.h>
#line 230 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"
#line 26 "/usr/lib/python3.14/site-packages/PyQt6/bindings/QtCore/qobjectdefs.sip"
#include <qobjectdefs.h>
#line 233 "/build/calibre/src/calibre-9.13.0/build/pyqt/progress_indicator/build/progress_indicator/sipprogress_indicatorQProgressIndicator.cpp"


class sipQProgressIndicator : public ::QProgressIndicator
{
public:
    sipQProgressIndicator(::QWidget*, int, int);
    virtual ~sipQProgressIndicator();

    int qt_metacall(QMetaObject::Call, int, void **) SIP_OVERRIDE;
    void *qt_metacast(const char *) SIP_OVERRIDE;
    const QMetaObject *metaObject() const SIP_OVERRIDE;

    /*
     * There is a protected method for every virtual method visible from
     * this class.
     */
protected:
    void disconnectNotify(const ::QMetaMethod&) SIP_OVERRIDE;
    void connectNotify(const ::QMetaMethod&) SIP_OVERRIDE;
    void customEvent(::QEvent*) SIP_OVERRIDE;
    void childEvent(::QChildEvent*) SIP_OVERRIDE;
    void timerEvent(::QTimerEvent*) SIP_OVERRIDE;
    bool eventFilter(::QObject*, ::QEvent*) SIP_OVERRIDE;
    bool event(::QEvent*) SIP_OVERRIDE;
    int metric(::QPaintDevice::PaintDeviceMetric) const SIP_OVERRIDE;
    ::QPaintEngine* paintEngine() const SIP_OVERRIDE;
    int devType() const SIP_OVERRIDE;
    void setVisible(bool) SIP_OVERRIDE;
    ::QSize sizeHint() const SIP_OVERRIDE;
    ::QSize minimumSizeHint() const SIP_OVERRIDE;
    int heightForWidth(int) const SIP_OVERRIDE;
    void mousePressEvent(::QMouseEvent*) SIP_OVERRIDE;
    void mouseReleaseEvent(::QMouseEvent*) SIP_OVERRIDE;
    void mouseDoubleClickEvent(::QMouseEvent*) SIP_OVERRIDE;
    void mouseMoveEvent(::QMouseEvent*) SIP_OVERRIDE;
    void wheelEvent(::QWheelEvent*) SIP_OVERRIDE;
    void keyPressEvent(::QKeyEvent*) SIP_OVERRIDE;
    void keyReleaseEvent(::QKeyEvent*) SIP_OVERRIDE;
    void focusInEvent(::QFocusEvent*) SIP_OVERRIDE;
    void focusOutEvent(::QFocusEvent*) SIP_OVERRIDE;
    void enterEvent(::QEnterEvent*) SIP_OVERRIDE;
    void leaveEvent(::QEvent*) SIP_OVERRIDE;
    void paintEvent(::QPaintEvent*) SIP_OVERRIDE;
    void moveEvent(::QMoveEvent*) SIP_OVERRIDE;
    void resizeEvent(::QResizeEvent*) SIP_OVERRIDE;
    void closeEvent(::QCloseEvent*) SIP_OVERRIDE;
    void contextMenuEvent(::QContextMenuEvent*) SIP_OVERRIDE;
    void tabletEvent(::QTabletEvent*) SIP_OVERRIDE;
    void actionEvent(::QActionEvent*) SIP_OVERRIDE;
    void dragEnterEvent(::QDragEnterEvent*) SIP_OVERRIDE;
    void dragMoveEvent(::QDragMoveEvent*) SIP_OVERRIDE;
    void dragLeaveEvent(::QDragLeaveEvent*) SIP_OVERRIDE;
    void dropEvent(::QDropEvent*) SIP_OVERRIDE;
    void showEvent(::QShowEvent*) SIP_OVERRIDE;
    void hideEvent(::QHideEvent*) SIP_OVERRIDE;
    void changeEvent(::QEvent*) SIP_OVERRIDE;
    void inputMethodEvent(::QInputMethodEvent*) SIP_OVERRIDE;
    ::QVariant inputMethodQuery(::Qt::InputMethodQuery) const SIP_OVERRIDE;
    bool focusNextPrevChild(bool) SIP_OVERRIDE;
    bool hasHeightForWidth() const SIP_OVERRIDE;
    bool nativeEvent(const ::QByteArray&, void*, ::qintptr*) SIP_OVERRIDE;
    ::QPainter* sharedPainter() const SIP_OVERRIDE;
    void initPainter(::QPainter*) const SIP_OVERRIDE;

public:
    sipSimpleWrapper *sipPySelf;

private:
    sipQProgressIndicator(const sipQProgressIndicator &);
    sipQProgressIndicator &operator = (const sipQProgressIndicator &);

    char sipPyMethods[46];
};

sipQProgressIndicator::sipQProgressIndicator(::QWidget*a0, int a1, int a2): ::QProgressIndicator(a0, a1, a2), sipPySelf(SIP_NULLPTR)
{
    memset(sipPyMethods, 0, sizeof (sipPyMethods));
}

sipQProgressIndicator::~sipQProgressIndicator()
{
    sipInstanceDestroyedEx(&sipPySelf);
}

const QMetaObject *sipQProgressIndicator::metaObject() const
{
    if (sipGetInterpreter())
        return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : sip_progress_indicator_qt_metaobject(sipPySelf, sipType_QProgressIndicator);

    return ::QProgressIndicator::metaObject();
}

int sipQProgressIndicator::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ::QProgressIndicator::qt_metacall(_c, _id, _a);

    if (_id >= 0)
    {
        SIP_BLOCK_THREADS
        _id = sip_progress_indicator_qt_metacall(sipPySelf, sipType_QProgressIndicator, _c, _id, _a);
        SIP_UNBLOCK_THREADS
    }

    return _id;
}

void *sipQProgressIndicator::qt_metacast(const char *_clname)
{
    void *sipCpp;

    return (sip_progress_indicator_qt_metacast(sipPySelf, sipType_QProgressIndicator, _clname, &sipCpp) ? sipCpp : ::QProgressIndicator::qt_metacast(_clname));
}

void sipQProgressIndicator::disconnectNotify(const ::QMetaMethod& a0)
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, &sipPyMethods[0], &sipPySelf, SIP_NULLPTR, sipName_disconnectNotify);

    if (!sipMeth)
    {
        ::QProgressIndicator::disconnectNotify(a0);
        return;
    }

    extern void sipVH_progress_indicator_26(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *, const ::QMetaMethod&);

    sipVH_progress_indicator_26(sipGILState, sipImportedVirtErrorHandlers_progress_indicator_QtCore[0].iveh_handler, sipPySelf, sipMeth, a0);
}

void sipQProgressIndicator::connectNotify(const ::QMetaMethod& a0)
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, &sipPyMethods[1], &sipPySelf, SIP_NULLPTR, sipName_connectNotify);

    if (!sipMeth)
    {
        ::QProgressIndicator::connectNotify(a0);
        return;
    }

    extern void sipVH_progress_indicator_26(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *, const ::QMetaMethod&);

    sipVH_progress_indicator_26(sipGILState, sipImportedVirtErrorHandlers_progress_indicator_QtCore[0].iveh_handler, sipPySelf, sipMeth, a0);
}

void sipQProgressIndicator::customEvent(::QEvent*a0)
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, &sipPyMethods[2], &sipPySelf, SIP_NULLPTR, sipName_customEvent);

    if (!sipMeth)
    {
        ::QProgressIndicator::customEvent(a0);
        return;
    }

    extern void sipVH_progress_indicator_25(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *, ::QEvent*);

    sipVH_progress_indicator_25(sipGILState, sipImportedVirtErrorHandlers_progress_indicator_QtCore[0].iveh_handler, sipPySelf, sipMeth, a0);
}

void sipQProgressIndicator::childEvent(::QChildEvent*a0)
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, &sipPyMethods[3], &sipPySelf, SIP_NULLPTR, sipName_childEvent);

    if (!sipMeth)
    {
        ::QProgressIndicator::childEvent(a0);
        return;
    }

    extern void sipVH_progress_indicator_24(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *, ::QChildEvent*);

    sipVH_progress_indicator_24(sipGILState, sipImportedVirtErrorHandlers_progress_indicator_QtCore[0].iveh_handler, sipPySelf, sipMeth, a0);
}

void sipQProgressIndicator::timerEvent(::QTimerEvent*a0)
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, &sipPyMethods[4], &sipPySelf, SIP_NULLPTR, sipName_timerEvent);

    if (!sipMeth)
    {
        ::QProgressIndicator::timerEvent(a0);
        return;
    }

    extern void sipVH_progress_indicator_23(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *, ::QTimerEvent*);

    sipVH_progress_indicator_23(sipGILState, sipImportedVirtErrorHandlers_progress_indicator_QtCore[0].iveh_handler, sipPySelf, sipMeth, a0);
}

bool sipQProgressIndicator::eventFilter(::QObject*a0, ::QEvent*a1)
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, &sipPyMethods[5], &sipPySelf, SIP_NULLPTR, sipName_eventFilter);

    if (!sipMeth)
        return ::QProgressIndicator::eventFilter(a0, a1);

    extern bool sipVH_progress_indicator_22(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *, ::QObject*, ::QEvent*);

    return sipVH_progress_indicator_22(sipGILState, sipImportedVirtErrorHandlers_progress_indicator_QtCore[0].iveh_handler, sipPySelf, sipMeth, a0, a1);
}

bool sipQProgressIndicator::event(::QEvent*a0)
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, &sipPyMethods[6], &sipPySelf, SIP_NULLPTR, sipName_event);

    if (!sipMeth)
        return ::QProgressIndicator::event(a0);

    extern bool sipVH_progress_indicator_21(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *, ::QEvent*);

    return sipVH_progress_indicator_21(sipGILState, sipImportedVirtErrorHandlers_progress_indicator_QtCore[0].iveh_handler, sipPySelf, sipMeth, a0);
}

int sipQProgressIndicator::metric(::QPaintDevice::PaintDeviceMetric a0) const
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, const_cast<char *>(&sipPyMethods[7]), const_cast<sipSimpleWrapper **>(&sipPySelf), SIP_NULLPTR, sipName_metric);

    if (!sipMeth)
        return ::QProgressIndicator::metric(a0);

    extern int sipVH_progress_indicator_57(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *, ::QPaintDevice::PaintDeviceMetric);

    return sipVH_progress_indicator_57(sipGILState, sipImportedVirtErrorHandlers_progress_indicator_QtCore[0].iveh_handler, sipPySelf, sipMeth, a0);
}

::QPaintEngine* sipQProgressIndicator::paintEngine() const
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, const_cast<char *>(&sipPyMethods[8]), const_cast<sipSimpleWrapper **>(&sipPySelf), SIP_NULLPTR, sipName_paintEngine);

    if (!sipMeth)
        return ::QProgressIndicator::paintEngine();

    extern ::QPaintEngine* sipVH_progress_indicator_56(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *);

    return sipVH_progress_indicator_56(sipGILState, sipImportedVirtErrorHandlers_progress_indicator_QtCore[0].iveh_handler, sipPySelf, sipMeth);
}

int sipQProgressIndicator::devType() const
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, const_cast<char *>(&sipPyMethods[9]), const_cast<sipSimpleWrapper **>(&sipPySelf), SIP_NULLPTR, sipName_devType);

    if (!sipMeth)
        return ::QProgressIndicator::devType();

    extern int sipVH_progress_indicator_55(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *);

    return sipVH_progress_indicator_55(sipGILState, sipImportedVirtErrorHandlers_progress_indicator_QtCore[0].iveh_handler, sipPySelf, sipMeth);
}

void sipQProgressIndicator::setVisible(bool a0)
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, &sipPyMethods[10], &sipPySelf, SIP_NULLPTR, sipName_setVisible);

    if (!sipMeth)
    {
        ::QProgressIndicator::setVisible(a0);
        return;
    }

    extern void sipVH_progress_indicator_54(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *, bool);

    sipVH_progress_indicator_54(sipGILState, sipImportedVirtErrorHandlers_progress_indicator_QtCore[0].iveh_handler, sipPySelf, sipMeth, a0);
}

::QSize sipQProgressIndicator::sizeHint() const
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, const_cast<char *>(&sipPyMethods[11]), const_cast<sipSimpleWrapper **>(&sipPySelf), SIP_NULLPTR, sipName_sizeHint);

    if (!sipMeth)
        return ::QProgressIndicator::sizeHint();

    extern ::QSize sipVH_progress_indicator_53(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *);

    return sipVH_progress_indicator_53(sipGILState, sipImportedVirtErrorHandlers_progress_indicator_QtCore[0].iveh_handler, sipPySelf, sipMeth);
}

::QSize sipQProgressIndicator::minimumSizeHint() const
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, const_cast<char *>(&sipPyMethods[12]), const_cast<sipSimpleWrapper **>(&sipPySelf), SIP_NULLPTR, sipName_minimumSizeHint);

    if (!sipMeth)
        return ::QProgressIndicator::minimumSizeHint();

    extern ::QSize sipVH_progress_indicator_53(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *);

    return sipVH_progress_indicator_53(sipGILState, sipImportedVirtErrorHandlers_progress_indicator_QtCore[0].iveh_handler, sipPySelf, sipMeth);
}

int sipQProgressIndicator::heightForWidth(int a0) const
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, const_cast<char *>(&sipPyMethods[13]), const_cast<sipSimpleWrapper **>(&sipPySelf), SIP_NULLPTR, sipName_heightForWidth);

    if (!sipMeth)
        return ::QProgressIndicator::heightForWidth(a0);

    extern int sipVH_progress_indicator_52(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *, int);

    return sipVH_progress_indicator_52(sipGILState, sipImportedVirtErrorHandlers_progress_indicator_QtCore[0].iveh_handler, sipPySelf, sipMeth, a0);
}

void sipQProgressIndicator::mousePressEvent(::QMouseEvent*a0)
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, &sipPyMethods[14], &sipPySelf, SIP_NULLPTR, sipName_mousePressEvent);

    if (!sipMeth)
    {
        ::QProgressIndicator::mousePressEvent(a0);
        return;
    }

    extern void sipVH_progress_indicator_51(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *, ::QMouseEvent*);

    sipVH_progress_indicator_51(sipGILState, sipImportedVirtErrorHandlers_progress_indicator_QtCore[0].iveh_handler, sipPySelf, sipMeth, a0);
}

void sipQProgressIndicator::mouseReleaseEvent(::QMouseEvent*a0)
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, &sipPyMethods[15], &sipPySelf, SIP_NULLPTR, sipName_mouseReleaseEvent);

    if (!sipMeth)
    {
        ::QProgressIndicator::mouseReleaseEvent(a0);
        return;
    }

    extern void sipVH_progress_indicator_51(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *, ::QMouseEvent*);

    sipVH_progress_indicator_51(sipGILState, sipImportedVirtErrorHandlers_progress_indicator_QtCore[0].iveh_handler, sipPySelf, sipMeth, a0);
}

void sipQProgressIndicator::mouseDoubleClickEvent(::QMouseEvent*a0)
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, &sipPyMethods[16], &sipPySelf, SIP_NULLPTR, sipName_mouseDoubleClickEvent);

    if (!sipMeth)
    {
        ::QProgressIndicator::mouseDoubleClickEvent(a0);
        return;
    }

    extern void sipVH_progress_indicator_51(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *, ::QMouseEvent*);

    sipVH_progress_indicator_51(sipGILState, sipImportedVirtErrorHandlers_progress_indicator_QtCore[0].iveh_handler, sipPySelf, sipMeth, a0);
}

void sipQProgressIndicator::mouseMoveEvent(::QMouseEvent*a0)
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, &sipPyMethods[17], &sipPySelf, SIP_NULLPTR, sipName_mouseMoveEvent);

    if (!sipMeth)
    {
        ::QProgressIndicator::mouseMoveEvent(a0);
        return;
    }

    extern void sipVH_progress_indicator_51(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *, ::QMouseEvent*);

    sipVH_progress_indicator_51(sipGILState, sipImportedVirtErrorHandlers_progress_indicator_QtCore[0].iveh_handler, sipPySelf, sipMeth, a0);
}

void sipQProgressIndicator::wheelEvent(::QWheelEvent*a0)
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, &sipPyMethods[18], &sipPySelf, SIP_NULLPTR, sipName_wheelEvent);

    if (!sipMeth)
    {
        ::QProgressIndicator::wheelEvent(a0);
        return;
    }

    extern void sipVH_progress_indicator_50(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *, ::QWheelEvent*);

    sipVH_progress_indicator_50(sipGILState, sipImportedVirtErrorHandlers_progress_indicator_QtCore[0].iveh_handler, sipPySelf, sipMeth, a0);
}

void sipQProgressIndicator::keyPressEvent(::QKeyEvent*a0)
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, &sipPyMethods[19], &sipPySelf, SIP_NULLPTR, sipName_keyPressEvent);

    if (!sipMeth)
    {
        ::QProgressIndicator::keyPressEvent(a0);
        return;
    }

    extern void sipVH_progress_indicator_49(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *, ::QKeyEvent*);

    sipVH_progress_indicator_49(sipGILState, sipImportedVirtErrorHandlers_progress_indicator_QtCore[0].iveh_handler, sipPySelf, sipMeth, a0);
}

void sipQProgressIndicator::keyReleaseEvent(::QKeyEvent*a0)
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, &sipPyMethods[20], &sipPySelf, SIP_NULLPTR, sipName_keyReleaseEvent);

    if (!sipMeth)
    {
        ::QProgressIndicator::keyReleaseEvent(a0);
        return;
    }

    extern void sipVH_progress_indicator_49(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *, ::QKeyEvent*);

    sipVH_progress_indicator_49(sipGILState, sipImportedVirtErrorHandlers_progress_indicator_QtCore[0].iveh_handler, sipPySelf, sipMeth, a0);
}

void sipQProgressIndicator::focusInEvent(::QFocusEvent*a0)
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, &sipPyMethods[21], &sipPySelf, SIP_NULLPTR, sipName_focusInEvent);

    if (!sipMeth)
    {
        ::QProgressIndicator::focusInEvent(a0);
        return;
    }

    extern void sipVH_progress_indicator_48(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *, ::QFocusEvent*);

    sipVH_progress_indicator_48(sipGILState, sipImportedVirtErrorHandlers_progress_indicator_QtCore[0].iveh_handler, sipPySelf, sipMeth, a0);
}

void sipQProgressIndicator::focusOutEvent(::QFocusEvent*a0)
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, &sipPyMethods[22], &sipPySelf, SIP_NULLPTR, sipName_focusOutEvent);

    if (!sipMeth)
    {
        ::QProgressIndicator::focusOutEvent(a0);
        return;
    }

    extern void sipVH_progress_indicator_48(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *, ::QFocusEvent*);

    sipVH_progress_indicator_48(sipGILState, sipImportedVirtErrorHandlers_progress_indicator_QtCore[0].iveh_handler, sipPySelf, sipMeth, a0);
}

void sipQProgressIndicator::enterEvent(::QEnterEvent*a0)
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, &sipPyMethods[23], &sipPySelf, SIP_NULLPTR, sipName_enterEvent);

    if (!sipMeth)
    {
        ::QProgressIndicator::enterEvent(a0);
        return;
    }

    extern void sipVH_progress_indicator_47(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *, ::QEnterEvent*);

    sipVH_progress_indicator_47(sipGILState, sipImportedVirtErrorHandlers_progress_indicator_QtCore[0].iveh_handler, sipPySelf, sipMeth, a0);
}

void sipQProgressIndicator::leaveEvent(::QEvent*a0)
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, &sipPyMethods[24], &sipPySelf, SIP_NULLPTR, sipName_leaveEvent);

    if (!sipMeth)
    {
        ::QProgressIndicator::leaveEvent(a0);
        return;
    }

    extern void sipVH_progress_indicator_25(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *, ::QEvent*);

    sipVH_progress_indicator_25(sipGILState, sipImportedVirtErrorHandlers_progress_indicator_QtCore[0].iveh_handler, sipPySelf, sipMeth, a0);
}

void sipQProgressIndicator::paintEvent(::QPaintEvent*a0)
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, &sipPyMethods[25], &sipPySelf, SIP_NULLPTR, sipName_paintEvent);

    if (!sipMeth)
    {
        ::QProgressIndicator::paintEvent(a0);
        return;
    }

    extern void sipVH_progress_indicator_46(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *, ::QPaintEvent*);

    sipVH_progress_indicator_46(sipGILState, sipImportedVirtErrorHandlers_progress_indicator_QtCore[0].iveh_handler, sipPySelf, sipMeth, a0);
}

void sipQProgressIndicator::moveEvent(::QMoveEvent*a0)
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, &sipPyMethods[26], &sipPySelf, SIP_NULLPTR, sipName_moveEvent);

    if (!sipMeth)
    {
        ::QProgressIndicator::moveEvent(a0);
        return;
    }

    extern void sipVH_progress_indicator_45(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *, ::QMoveEvent*);

    sipVH_progress_indicator_45(sipGILState, sipImportedVirtErrorHandlers_progress_indicator_QtCore[0].iveh_handler, sipPySelf, sipMeth, a0);
}

void sipQProgressIndicator::resizeEvent(::QResizeEvent*a0)
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, &sipPyMethods[27], &sipPySelf, SIP_NULLPTR, sipName_resizeEvent);

    if (!sipMeth)
    {
        ::QProgressIndicator::resizeEvent(a0);
        return;
    }

    extern void sipVH_progress_indicator_44(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *, ::QResizeEvent*);

    sipVH_progress_indicator_44(sipGILState, sipImportedVirtErrorHandlers_progress_indicator_QtCore[0].iveh_handler, sipPySelf, sipMeth, a0);
}

void sipQProgressIndicator::closeEvent(::QCloseEvent*a0)
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, &sipPyMethods[28], &sipPySelf, SIP_NULLPTR, sipName_closeEvent);

    if (!sipMeth)
    {
        ::QProgressIndicator::closeEvent(a0);
        return;
    }

    extern void sipVH_progress_indicator_43(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *, ::QCloseEvent*);

    sipVH_progress_indicator_43(sipGILState, sipImportedVirtErrorHandlers_progress_indicator_QtCore[0].iveh_handler, sipPySelf, sipMeth, a0);
}

void sipQProgressIndicator::contextMenuEvent(::QContextMenuEvent*a0)
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, &sipPyMethods[29], &sipPySelf, SIP_NULLPTR, sipName_contextMenuEvent);

    if (!sipMeth)
    {
        ::QProgressIndicator::contextMenuEvent(a0);
        return;
    }

    extern void sipVH_progress_indicator_42(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *, ::QContextMenuEvent*);

    sipVH_progress_indicator_42(sipGILState, sipImportedVirtErrorHandlers_progress_indicator_QtCore[0].iveh_handler, sipPySelf, sipMeth, a0);
}

void sipQProgressIndicator::tabletEvent(::QTabletEvent*a0)
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, &sipPyMethods[30], &sipPySelf, SIP_NULLPTR, sipName_tabletEvent);

    if (!sipMeth)
    {
        ::QProgressIndicator::tabletEvent(a0);
        return;
    }

    extern void sipVH_progress_indicator_41(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *, ::QTabletEvent*);

    sipVH_progress_indicator_41(sipGILState, sipImportedVirtErrorHandlers_progress_indicator_QtCore[0].iveh_handler, sipPySelf, sipMeth, a0);
}

void sipQProgressIndicator::actionEvent(::QActionEvent*a0)
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, &sipPyMethods[31], &sipPySelf, SIP_NULLPTR, sipName_actionEvent);

    if (!sipMeth)
    {
        ::QProgressIndicator::actionEvent(a0);
        return;
    }

    extern void sipVH_progress_indicator_40(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *, ::QActionEvent*);

    sipVH_progress_indicator_40(sipGILState, sipImportedVirtErrorHandlers_progress_indicator_QtCore[0].iveh_handler, sipPySelf, sipMeth, a0);
}

void sipQProgressIndicator::dragEnterEvent(::QDragEnterEvent*a0)
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, &sipPyMethods[32], &sipPySelf, SIP_NULLPTR, sipName_dragEnterEvent);

    if (!sipMeth)
    {
        ::QProgressIndicator::dragEnterEvent(a0);
        return;
    }

    extern void sipVH_progress_indicator_39(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *, ::QDragEnterEvent*);

    sipVH_progress_indicator_39(sipGILState, sipImportedVirtErrorHandlers_progress_indicator_QtCore[0].iveh_handler, sipPySelf, sipMeth, a0);
}

void sipQProgressIndicator::dragMoveEvent(::QDragMoveEvent*a0)
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, &sipPyMethods[33], &sipPySelf, SIP_NULLPTR, sipName_dragMoveEvent);

    if (!sipMeth)
    {
        ::QProgressIndicator::dragMoveEvent(a0);
        return;
    }

    extern void sipVH_progress_indicator_38(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *, ::QDragMoveEvent*);

    sipVH_progress_indicator_38(sipGILState, sipImportedVirtErrorHandlers_progress_indicator_QtCore[0].iveh_handler, sipPySelf, sipMeth, a0);
}

void sipQProgressIndicator::dragLeaveEvent(::QDragLeaveEvent*a0)
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, &sipPyMethods[34], &sipPySelf, SIP_NULLPTR, sipName_dragLeaveEvent);

    if (!sipMeth)
    {
        ::QProgressIndicator::dragLeaveEvent(a0);
        return;
    }

    extern void sipVH_progress_indicator_37(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *, ::QDragLeaveEvent*);

    sipVH_progress_indicator_37(sipGILState, sipImportedVirtErrorHandlers_progress_indicator_QtCore[0].iveh_handler, sipPySelf, sipMeth, a0);
}

void sipQProgressIndicator::dropEvent(::QDropEvent*a0)
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, &sipPyMethods[35], &sipPySelf, SIP_NULLPTR, sipName_dropEvent);

    if (!sipMeth)
    {
        ::QProgressIndicator::dropEvent(a0);
        return;
    }

    extern void sipVH_progress_indicator_36(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *, ::QDropEvent*);

    sipVH_progress_indicator_36(sipGILState, sipImportedVirtErrorHandlers_progress_indicator_QtCore[0].iveh_handler, sipPySelf, sipMeth, a0);
}

void sipQProgressIndicator::showEvent(::QShowEvent*a0)
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, &sipPyMethods[36], &sipPySelf, SIP_NULLPTR, sipName_showEvent);

    if (!sipMeth)
    {
        ::QProgressIndicator::showEvent(a0);
        return;
    }

    extern void sipVH_progress_indicator_35(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *, ::QShowEvent*);

    sipVH_progress_indicator_35(sipGILState, sipImportedVirtErrorHandlers_progress_indicator_QtCore[0].iveh_handler, sipPySelf, sipMeth, a0);
}

void sipQProgressIndicator::hideEvent(::QHideEvent*a0)
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, &sipPyMethods[37], &sipPySelf, SIP_NULLPTR, sipName_hideEvent);

    if (!sipMeth)
    {
        ::QProgressIndicator::hideEvent(a0);
        return;
    }

    extern void sipVH_progress_indicator_34(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *, ::QHideEvent*);

    sipVH_progress_indicator_34(sipGILState, sipImportedVirtErrorHandlers_progress_indicator_QtCore[0].iveh_handler, sipPySelf, sipMeth, a0);
}

void sipQProgressIndicator::changeEvent(::QEvent*a0)
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, &sipPyMethods[38], &sipPySelf, SIP_NULLPTR, sipName_changeEvent);

    if (!sipMeth)
    {
        ::QProgressIndicator::changeEvent(a0);
        return;
    }

    extern void sipVH_progress_indicator_25(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *, ::QEvent*);

    sipVH_progress_indicator_25(sipGILState, sipImportedVirtErrorHandlers_progress_indicator_QtCore[0].iveh_handler, sipPySelf, sipMeth, a0);
}

void sipQProgressIndicator::inputMethodEvent(::QInputMethodEvent*a0)
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, &sipPyMethods[39], &sipPySelf, SIP_NULLPTR, sipName_inputMethodEvent);

    if (!sipMeth)
    {
        ::QProgressIndicator::inputMethodEvent(a0);
        return;
    }

    extern void sipVH_progress_indicator_33(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *, ::QInputMethodEvent*);

    sipVH_progress_indicator_33(sipGILState, sipImportedVirtErrorHandlers_progress_indicator_QtCore[0].iveh_handler, sipPySelf, sipMeth, a0);
}

::QVariant sipQProgressIndicator::inputMethodQuery(::Qt::InputMethodQuery a0) const
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, const_cast<char *>(&sipPyMethods[40]), const_cast<sipSimpleWrapper **>(&sipPySelf), SIP_NULLPTR, sipName_inputMethodQuery);

    if (!sipMeth)
        return ::QProgressIndicator::inputMethodQuery(a0);

    extern ::QVariant sipVH_progress_indicator_32(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *, ::Qt::InputMethodQuery);

    return sipVH_progress_indicator_32(sipGILState, sipImportedVirtErrorHandlers_progress_indicator_QtCore[0].iveh_handler, sipPySelf, sipMeth, a0);
}

bool sipQProgressIndicator::focusNextPrevChild(bool a0)
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, &sipPyMethods[41], &sipPySelf, SIP_NULLPTR, sipName_focusNextPrevChild);

    if (!sipMeth)
        return ::QProgressIndicator::focusNextPrevChild(a0);

    extern bool sipVH_progress_indicator_31(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *, bool);

    return sipVH_progress_indicator_31(sipGILState, sipImportedVirtErrorHandlers_progress_indicator_QtCore[0].iveh_handler, sipPySelf, sipMeth, a0);
}

bool sipQProgressIndicator::hasHeightForWidth() const
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, const_cast<char *>(&sipPyMethods[42]), const_cast<sipSimpleWrapper **>(&sipPySelf), SIP_NULLPTR, sipName_hasHeightForWidth);

    if (!sipMeth)
        return ::QProgressIndicator::hasHeightForWidth();

    extern bool sipVH_progress_indicator_30(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *);

    return sipVH_progress_indicator_30(sipGILState, sipImportedVirtErrorHandlers_progress_indicator_QtCore[0].iveh_handler, sipPySelf, sipMeth);
}

bool sipQProgressIndicator::nativeEvent(const ::QByteArray& a0, void*a1, ::qintptr*a2)
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, &sipPyMethods[43], &sipPySelf, SIP_NULLPTR, sipName_nativeEvent);

    if (!sipMeth)
        return ::QProgressIndicator::nativeEvent(a0, a1, a2);

    extern bool sipVH_progress_indicator_29(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *, const ::QByteArray&, void*, ::qintptr*);

    return sipVH_progress_indicator_29(sipGILState, sipImportedVirtErrorHandlers_progress_indicator_QtCore[0].iveh_handler, sipPySelf, sipMeth, a0, a1, a2);
}

::QPainter* sipQProgressIndicator::sharedPainter() const
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, const_cast<char *>(&sipPyMethods[44]), const_cast<sipSimpleWrapper **>(&sipPySelf), SIP_NULLPTR, sipName_sharedPainter);

    if (!sipMeth)
        return ::QProgressIndicator::sharedPainter();

    extern ::QPainter* sipVH_progress_indicator_28(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *);

    return sipVH_progress_indicator_28(sipGILState, sipImportedVirtErrorHandlers_progress_indicator_QtCore[0].iveh_handler, sipPySelf, sipMeth);
}

void sipQProgressIndicator::initPainter(::QPainter*a0) const
{
    sip_gilstate_t sipGILState;
    PyObject *sipMeth;

    sipMeth = sipIsPyMethod(&sipGILState, const_cast<char *>(&sipPyMethods[45]), const_cast<sipSimpleWrapper **>(&sipPySelf), SIP_NULLPTR, sipName_initPainter);

    if (!sipMeth)
    {
        ::QProgressIndicator::initPainter(a0);
        return;
    }

    extern void sipVH_progress_indicator_27(sip_gilstate_t, sipVirtErrorHandlerFunc, sipSimpleWrapper *, PyObject *, ::QPainter*);

    sipVH_progress_indicator_27(sipGILState, sipImportedVirtErrorHandlers_progress_indicator_QtCore[0].iveh_handler, sipPySelf, sipMeth, a0);
}


PyDoc_STRVAR(doc_QProgressIndicator_isAnimated, "isAnimated(self) -> bool");

extern "C" {static PyObject *meth_QProgressIndicator_isAnimated(PyObject *, PyObject *);}
static PyObject *meth_QProgressIndicator_isAnimated(PyObject *sipSelf, PyObject *sipArgs)
{
    PyObject *sipParseErr = SIP_NULLPTR;

    {
        const ::QProgressIndicator *sipCpp;

        if (sipParseArgs(&sipParseErr, sipArgs, "B", &sipSelf, sipType_QProgressIndicator, &sipCpp))
        {
            bool sipRes;

            sipRes = sipCpp->isAnimated();

            return PyBool_FromLong(sipRes);
        }
    }

    sipNoMethod(sipParseErr, sipName_QProgressIndicator, sipName_isAnimated, doc_QProgressIndicator_isAnimated);

    return SIP_NULLPTR;
}


PyDoc_STRVAR(doc_QProgressIndicator_sizeHint, "sizeHint(self) -> QSize");

extern "C" {static PyObject *meth_QProgressIndicator_sizeHint(PyObject *, PyObject *);}
static PyObject *meth_QProgressIndicator_sizeHint(PyObject *sipSelf, PyObject *sipArgs)
{
    PyObject *sipParseErr = SIP_NULLPTR;
    bool sipSelfWasArg = (!PyObject_TypeCheck(sipSelf, sipTypeAsPyTypeObject(sipType_QProgressIndicator)) || sipIsDerivedClass((sipSimpleWrapper *)sipSelf));

    {
        const ::QProgressIndicator *sipCpp;

        if (sipParseArgs(&sipParseErr, sipArgs, "B", &sipSelf, sipType_QProgressIndicator, &sipCpp))
        {
            ::QSize*sipRes;

            sipRes = new ::QSize((sipSelfWasArg ? sipCpp->::QProgressIndicator::sizeHint() : sipCpp->sizeHint()));

            return sipConvertFromNewType(sipRes, sipType_QSize, SIP_NULLPTR);
        }
    }

    sipNoMethod(sipParseErr, sipName_QProgressIndicator, sipName_sizeHint, doc_QProgressIndicator_sizeHint);

    return SIP_NULLPTR;
}


PyDoc_STRVAR(doc_QProgressIndicator_heightForWidth, "heightForWidth(self, w: int) -> int");

extern "C" {static PyObject *meth_QProgressIndicator_heightForWidth(PyObject *, PyObject *);}
static PyObject *meth_QProgressIndicator_heightForWidth(PyObject *sipSelf, PyObject *sipArgs)
{
    PyObject *sipParseErr = SIP_NULLPTR;
    bool sipSelfWasArg = (!PyObject_TypeCheck(sipSelf, sipTypeAsPyTypeObject(sipType_QProgressIndicator)) || sipIsDerivedClass((sipSimpleWrapper *)sipSelf));

    {
        int a0;
        const ::QProgressIndicator *sipCpp;

        if (sipParseArgs(&sipParseErr, sipArgs, "Bi", &sipSelf, sipType_QProgressIndicator, &sipCpp, &a0))
        {
            int sipRes;

            sipRes = (sipSelfWasArg ? sipCpp->::QProgressIndicator::heightForWidth(a0) : sipCpp->heightForWidth(a0));

            return PyLong_FromLong(sipRes);
        }
    }

    sipNoMethod(sipParseErr, sipName_QProgressIndicator, sipName_heightForWidth, doc_QProgressIndicator_heightForWidth);

    return SIP_NULLPTR;
}


PyDoc_STRVAR(doc_QProgressIndicator_displaySize, "displaySize(self) -> QSize");

extern "C" {static PyObject *meth_QProgressIndicator_displaySize(PyObject *, PyObject *);}
static PyObject *meth_QProgressIndicator_displaySize(PyObject *sipSelf, PyObject *sipArgs)
{
    PyObject *sipParseErr = SIP_NULLPTR;

    {
        const ::QProgressIndicator *sipCpp;

        if (sipParseArgs(&sipParseErr, sipArgs, "B", &sipSelf, sipType_QProgressIndicator, &sipCpp))
        {
            ::QSize*sipRes;

            sipRes = new ::QSize(sipCpp->displaySize());

            return sipConvertFromNewType(sipRes, sipType_QSize, SIP_NULLPTR);
        }
    }

    sipNoMethod(sipParseErr, sipName_QProgressIndicator, sipName_displaySize, doc_QProgressIndicator_displaySize);

    return SIP_NULLPTR;
}


PyDoc_STRVAR(doc_QProgressIndicator_startAnimation, "startAnimation(self)");

extern "C" {static PyObject *meth_QProgressIndicator_startAnimation(PyObject *, PyObject *);}
static PyObject *meth_QProgressIndicator_startAnimation(PyObject *sipSelf, PyObject *sipArgs)
{
    PyObject *sipParseErr = SIP_NULLPTR;

    {
        ::QProgressIndicator *sipCpp;

        if (sipParseArgs(&sipParseErr, sipArgs, "B", &sipSelf, sipType_QProgressIndicator, &sipCpp))
        {
            sipCpp->startAnimation();

            Py_INCREF(Py_None);
            return Py_None;
        }
    }

    sipNoMethod(sipParseErr, sipName_QProgressIndicator, sipName_startAnimation, doc_QProgressIndicator_startAnimation);

    return SIP_NULLPTR;
}


PyDoc_STRVAR(doc_QProgressIndicator_start, "start(self)");

extern "C" {static PyObject *meth_QProgressIndicator_start(PyObject *, PyObject *);}
static PyObject *meth_QProgressIndicator_start(PyObject *sipSelf, PyObject *sipArgs)
{
    PyObject *sipParseErr = SIP_NULLPTR;

    {
        ::QProgressIndicator *sipCpp;

        if (sipParseArgs(&sipParseErr, sipArgs, "B", &sipSelf, sipType_QProgressIndicator, &sipCpp))
        {
            sipCpp->start();

            Py_INCREF(Py_None);
            return Py_None;
        }
    }

    sipNoMethod(sipParseErr, sipName_QProgressIndicator, sipName_start, doc_QProgressIndicator_start);

    return SIP_NULLPTR;
}


PyDoc_STRVAR(doc_QProgressIndicator_stopAnimation, "stopAnimation(self)");

extern "C" {static PyObject *meth_QProgressIndicator_stopAnimation(PyObject *, PyObject *);}
static PyObject *meth_QProgressIndicator_stopAnimation(PyObject *sipSelf, PyObject *sipArgs)
{
    PyObject *sipParseErr = SIP_NULLPTR;

    {
        ::QProgressIndicator *sipCpp;

        if (sipParseArgs(&sipParseErr, sipArgs, "B", &sipSelf, sipType_QProgressIndicator, &sipCpp))
        {
            sipCpp->stopAnimation();

            Py_INCREF(Py_None);
            return Py_None;
        }
    }

    sipNoMethod(sipParseErr, sipName_QProgressIndicator, sipName_stopAnimation, doc_QProgressIndicator_stopAnimation);

    return SIP_NULLPTR;
}


PyDoc_STRVAR(doc_QProgressIndicator_stop, "stop(self)");

extern "C" {static PyObject *meth_QProgressIndicator_stop(PyObject *, PyObject *);}
static PyObject *meth_QProgressIndicator_stop(PyObject *sipSelf, PyObject *sipArgs)
{
    PyObject *sipParseErr = SIP_NULLPTR;

    {
        ::QProgressIndicator *sipCpp;

        if (sipParseArgs(&sipParseErr, sipArgs, "B", &sipSelf, sipType_QProgressIndicator, &sipCpp))
        {
            sipCpp->stop();

            Py_INCREF(Py_None);
            return Py_None;
        }
    }

    sipNoMethod(sipParseErr, sipName_QProgressIndicator, sipName_stop, doc_QProgressIndicator_stop);

    return SIP_NULLPTR;
}


PyDoc_STRVAR(doc_QProgressIndicator_setDisplaySize, "setDisplaySize(self, size: QSize)\n"
"setDisplaySize(self, size: int)");

extern "C" {static PyObject *meth_QProgressIndicator_setDisplaySize(PyObject *, PyObject *);}
static PyObject *meth_QProgressIndicator_setDisplaySize(PyObject *sipSelf, PyObject *sipArgs)
{
    PyObject *sipParseErr = SIP_NULLPTR;

    {
        ::QSize* a0;
        ::QProgressIndicator *sipCpp;

        if (sipParseArgs(&sipParseErr, sipArgs, "BJ9", &sipSelf, sipType_QProgressIndicator, &sipCpp, sipType_QSize, &a0))
        {
            sipCpp->setDisplaySize(*a0);

            Py_INCREF(Py_None);
            return Py_None;
        }
    }

    {
        int a0;
        ::QProgressIndicator *sipCpp;

        if (sipParseArgs(&sipParseErr, sipArgs, "Bi", &sipSelf, sipType_QProgressIndicator, &sipCpp, &a0))
        {
            sipCpp->setDisplaySize(a0);

            Py_INCREF(Py_None);
            return Py_None;
        }
    }

    sipNoMethod(sipParseErr, sipName_QProgressIndicator, sipName_setDisplaySize, doc_QProgressIndicator_setDisplaySize);

    return SIP_NULLPTR;
}


PyDoc_STRVAR(doc_QProgressIndicator_setSizeHint, "setSizeHint(self, size: int)\n"
"setSizeHint(self, size: QSize)");

extern "C" {static PyObject *meth_QProgressIndicator_setSizeHint(PyObject *, PyObject *);}
static PyObject *meth_QProgressIndicator_setSizeHint(PyObject *sipSelf, PyObject *sipArgs)
{
    PyObject *sipParseErr = SIP_NULLPTR;

    {
        int a0;
        ::QProgressIndicator *sipCpp;

        if (sipParseArgs(&sipParseErr, sipArgs, "Bi", &sipSelf, sipType_QProgressIndicator, &sipCpp, &a0))
        {
            sipCpp->setSizeHint(a0);

            Py_INCREF(Py_None);
            return Py_None;
        }
    }

    {
        ::QSize* a0;
        ::QProgressIndicator *sipCpp;

        if (sipParseArgs(&sipParseErr, sipArgs, "BJ9", &sipSelf, sipType_QProgressIndicator, &sipCpp, sipType_QSize, &a0))
        {
            sipCpp->setSizeHint(*a0);

            Py_INCREF(Py_None);
            return Py_None;
        }
    }

    sipNoMethod(sipParseErr, sipName_QProgressIndicator, sipName_setSizeHint, doc_QProgressIndicator_setSizeHint);

    return SIP_NULLPTR;
}


PyDoc_STRVAR(doc_QProgressIndicator_paintEvent, "paintEvent(self, event: QPaintEvent|None)");

extern "C" {static PyObject *meth_QProgressIndicator_paintEvent(PyObject *, PyObject *);}
static PyObject *meth_QProgressIndicator_paintEvent(PyObject *sipSelf, PyObject *sipArgs)
{
    PyObject *sipParseErr = SIP_NULLPTR;
    bool sipSelfWasArg = (!PyObject_TypeCheck(sipSelf, sipTypeAsPyTypeObject(sipType_QProgressIndicator)) || sipIsDerivedClass((sipSimpleWrapper *)sipSelf));

    {
        ::QPaintEvent* a0;
        ::QProgressIndicator *sipCpp;

        if (sipParseArgs(&sipParseErr, sipArgs, "pJ8", &sipSelf, sipType_QProgressIndicator, &sipCpp, sipType_QPaintEvent, &a0))
        {
            (sipSelfWasArg ? sipCpp->::QProgressIndicator::paintEvent(a0) : sipCpp->paintEvent(a0));

            Py_INCREF(Py_None);
            return Py_None;
        }
    }

    sipNoMethod(sipParseErr, sipName_QProgressIndicator, sipName_paintEvent, doc_QProgressIndicator_paintEvent);

    return SIP_NULLPTR;
}


/* Cast a pointer to a type somewhere in its inheritance hierarchy. */
extern "C" {static void *cast_QProgressIndicator(void *, const sipTypeDef *);}
static void *cast_QProgressIndicator(void *sipCppV, const sipTypeDef *targetType)
{
    ::QProgressIndicator *sipCpp = reinterpret_cast< ::QProgressIndicator *>(sipCppV);

    if (targetType == sipType_QProgressIndicator)
        return sipCppV;

    sipCppV = ((const sipClassTypeDef *)sipType_QWidget)->ctd_cast(static_cast< ::QWidget *>(sipCpp), targetType);
    if (sipCppV)
        return sipCppV;

    return SIP_NULLPTR;
}


/* Call the instance's destructor. */
extern "C" {static void release_QProgressIndicator(void *, int);}
static void release_QProgressIndicator(void *sipCppV, int)
{
    ::QProgressIndicator *sipCpp = reinterpret_cast< ::QProgressIndicator *>(sipCppV);

    if (QThread::currentThread() == sipCpp->thread())
        delete sipCpp;
    else
        sipCpp->deleteLater();
}


extern "C" {static void *array_QProgressIndicator(Py_ssize_t);}
static void *array_QProgressIndicator(Py_ssize_t sipNrElem)
{
    return new ::QProgressIndicator[sipNrElem];
}


extern "C" {static void array_delete_QProgressIndicator(void *);}
static void array_delete_QProgressIndicator(void *sipCpp)
{
    delete[] reinterpret_cast< ::QProgressIndicator *>(sipCpp);
}


extern "C" {static void dealloc_QProgressIndicator(sipSimpleWrapper *);}
static void dealloc_QProgressIndicator(sipSimpleWrapper *sipSelf)
{
    if (sipIsDerivedClass(sipSelf))
        reinterpret_cast<sipQProgressIndicator *>(sipGetAddress(sipSelf))->sipPySelf = SIP_NULLPTR;

    if (sipIsOwnedByPython(sipSelf))
    {
        release_QProgressIndicator(sipGetAddress(sipSelf), sipIsDerivedClass(sipSelf));
    }
}


extern "C" {static void *init_type_QProgressIndicator(sipSimpleWrapper *, PyObject *, PyObject *, PyObject **, PyObject **, PyObject **);}
static void *init_type_QProgressIndicator(sipSimpleWrapper *sipSelf, PyObject *sipArgs, PyObject *sipKwds, PyObject **sipUnused, PyObject **sipOwner, PyObject **sipParseErr)
{
    sipQProgressIndicator *sipCpp = SIP_NULLPTR;

    {
        ::QWidget* a0 = 0;
        int a1 = 64;
        int a2 = 0;

        if (sipParseKwdArgs(sipParseErr, sipArgs, sipKwds, SIP_NULLPTR, sipUnused, "|JHii", sipType_QWidget, &a0, sipOwner, &a1, &a2))
        {
            sipCpp = new sipQProgressIndicator(a0, a1, a2);

            sipCpp->sipPySelf = sipSelf;

            return sipCpp;
        }
    }

    return SIP_NULLPTR;
}


/* Define this type's super-types. */
static sipEncodedTypeDef supers_QProgressIndicator[] = {{18, 2, 1}};


static PyMethodDef methods_QProgressIndicator[] = {
    {sipName_displaySize, meth_QProgressIndicator_displaySize, METH_VARARGS, doc_QProgressIndicator_displaySize},
    {sipName_heightForWidth, meth_QProgressIndicator_heightForWidth, METH_VARARGS, doc_QProgressIndicator_heightForWidth},
    {sipName_isAnimated, meth_QProgressIndicator_isAnimated, METH_VARARGS, doc_QProgressIndicator_isAnimated},
    {sipName_paintEvent, meth_QProgressIndicator_paintEvent, METH_VARARGS, doc_QProgressIndicator_paintEvent},
    {sipName_setDisplaySize, meth_QProgressIndicator_setDisplaySize, METH_VARARGS, doc_QProgressIndicator_setDisplaySize},
    {sipName_setSizeHint, meth_QProgressIndicator_setSizeHint, METH_VARARGS, doc_QProgressIndicator_setSizeHint},
    {sipName_sizeHint, meth_QProgressIndicator_sizeHint, METH_VARARGS, doc_QProgressIndicator_sizeHint},
    {sipName_start, meth_QProgressIndicator_start, METH_VARARGS, doc_QProgressIndicator_start},
    {sipName_startAnimation, meth_QProgressIndicator_startAnimation, METH_VARARGS, doc_QProgressIndicator_startAnimation},
    {sipName_stop, meth_QProgressIndicator_stop, METH_VARARGS, doc_QProgressIndicator_stop},
    {sipName_stopAnimation, meth_QProgressIndicator_stopAnimation, METH_VARARGS, doc_QProgressIndicator_stopAnimation}
};

PyDoc_STRVAR(doc_QProgressIndicator, "\1QProgressIndicator(parent: QWidget|None = None, size: int = 64, interval: int = 0)");


/* Define this type's signals. */
static const pyqt6QtSignal signals_QProgressIndicator[] = {
    {"running_state_changed(bool)", "\1running_state_changed(self, a0: bool)", SIP_NULLPTR, SIP_NULLPTR},
    {SIP_NULLPTR, SIP_NULLPTR, SIP_NULLPTR, SIP_NULLPTR}
};


static pyqt6ClassPluginDef plugin_QProgressIndicator = {
    &::QProgressIndicator::staticMetaObject,
    signals_QProgressIndicator,
    SIP_NULLPTR
};


sipClassTypeDef sipTypeDef_progress_indicator_QProgressIndicator = {
    {
        SIP_NULLPTR,
        SIP_TYPE_SCC|SIP_TYPE_SUPER_INIT|SIP_TYPE_CLASS,
        sipNameNr_QProgressIndicator,
        SIP_NULLPTR,
        &plugin_QProgressIndicator,
    },
    {
        sipNameNr_QProgressIndicator,
        {0, 0, 1},
        11, methods_QProgressIndicator,
        0, SIP_NULLPTR,
        {SIP_NULLPTR, SIP_NULLPTR, SIP_NULLPTR, SIP_NULLPTR, SIP_NULLPTR, SIP_NULLPTR, SIP_NULLPTR, SIP_NULLPTR, SIP_NULLPTR, SIP_NULLPTR},
    },
    doc_QProgressIndicator,
    -1,
    -1,
    supers_QProgressIndicator,
    SIP_NULLPTR,
    init_type_QProgressIndicator,
    SIP_NULLPTR,
    SIP_NULLPTR,
    SIP_NULLPTR,
    SIP_NULLPTR,
    dealloc_QProgressIndicator,
    SIP_NULLPTR,
    array_QProgressIndicator,
    SIP_NULLPTR,
    release_QProgressIndicator,
    cast_QProgressIndicator,
    SIP_NULLPTR,
    SIP_NULLPTR,
    SIP_NULLPTR,
    SIP_NULLPTR,
    SIP_NULLPTR,
    SIP_NULLPTR,
    array_delete_QProgressIndicator,
    sizeof (::QProgressIndicator),
};
