/*
 * Internal module API header file.
 *
 * Generated by SIP 6.16.1
 */

#ifndef _pictureflowAPI_H
#define _pictureflowAPI_H

#include "sip.h"

#include <QMetaType>
#include <QThread>

/* These are the qualifiers that are enabled. */
#define SIP_FEATURE_PyQt_Wayland
#define SIP_FEATURE_PyQt_XCB
#define SIP_FEATURE_PyQt_Permissions
#define SIP_FEATURE_PyQt_DTLS
#define SIP_FEATURE_PyQt_WebChannel
#define SIP_FEATURE_PyQt_Process
#define SIP_FEATURE_PyQt_OpenGL
#define SIP_FEATURE_PyQt_RawFont
#define SIP_FEATURE_PyQt_PrintPreviewDialog
#define SIP_FEATURE_PyQt_PrintPreviewWidget
#define SIP_FEATURE_PyQt_Printer
#define SIP_FEATURE_PyQt_PrintDialog
#define SIP_FEATURE_PyQt_qreal_double
#define SIP_FEATURE_PyQt_SSL
#define SIP_FEATURE_PyQt_SessionManager
#define SIP_FEATURE_PyQt_Accessibility
#define SIP_PLATFORM_Linux
#define SIP_TIMELINE_Qt_6_11_0


/*
 * Convenient names to refer to various strings defined in this module.
 * Only the class names are part of the public API.
 */
#define sipNameNr_setActivateOnDoubleClick 0
#define sipName_setActivateOnDoubleClick &sipStrings_pictureflow[0]
#define sipNameNr_setPreserveAspectRatio 25
#define sipName_setPreserveAspectRatio &sipStrings_pictureflow[25]
#define sipNameNr_activateOnDoubleClick 48
#define sipName_activateOnDoubleClick &sipStrings_pictureflow[48]
#define sipNameNr_mouseDoubleClickEvent 70
#define sipName_mouseDoubleClickEvent &sipStrings_pictureflow[70]
#define sipNameNr_preserveAspectRatio 92
#define sipName_preserveAspectRatio &sipStrings_pictureflow[92]
#define sipNameNr_focusNextPrevChild 112
#define sipName_focusNextPrevChild &sipStrings_pictureflow[112]
#define sipNameNr_setShowReflections 131
#define sipName_setShowReflections &sipStrings_pictureflow[131]
#define sipNameNr_hasHeightForWidth 150
#define sipName_hasHeightForWidth &sipStrings_pictureflow[150]
#define sipNameNr_mouseReleaseEvent 168
#define sipName_mouseReleaseEvent &sipStrings_pictureflow[168]
#define sipNameNr_contextMenuEvent 186
#define sipName_contextMenuEvent &sipStrings_pictureflow[186]
#define sipNameNr_disconnectNotify 203
#define sipName_disconnectNotify &sipStrings_pictureflow[203]
#define sipNameNr_inputMethodEvent 220
#define sipName_inputMethodEvent &sipStrings_pictureflow[220]
#define sipNameNr_inputMethodQuery 237
#define sipName_inputMethodQuery &sipStrings_pictureflow[237]
#define sipNameNr_keyReleaseEvent 254
#define sipName_keyReleaseEvent &sipStrings_pictureflow[254]
#define sipNameNr_minimumSizeHint 270
#define sipName_minimumSizeHint &sipStrings_pictureflow[270]
#define sipNameNr_mousePressEvent 286
#define sipName_mousePressEvent &sipStrings_pictureflow[286]
#define sipNameNr_setCurrentSlide 302
#define sipName_setCurrentSlide &sipStrings_pictureflow[302]
#define sipNameNr_setSubtitleFont 318
#define sipName_setSubtitleFont &sipStrings_pictureflow[318]
#define sipNameNr_showReflections 334
#define sipName_showReflections &sipStrings_pictureflow[334]
#define sipNameNr_currentChanged 350
#define sipName_currentChanged &sipStrings_pictureflow[350]
#define sipNameNr_dragEnterEvent 365
#define sipName_dragEnterEvent &sipStrings_pictureflow[365]
#define sipNameNr_dragLeaveEvent 380
#define sipName_dragLeaveEvent &sipStrings_pictureflow[380]
#define sipNameNr_heightForWidth 395
#define sipName_heightForWidth &sipStrings_pictureflow[395]
#define sipNameNr_mouseMoveEvent 410
#define sipName_mouseMoveEvent &sipStrings_pictureflow[410]
#define sipNameNr_setMaxFontSize 425
#define sipName_setMaxFontSize &sipStrings_pictureflow[425]
#define sipNameNr_connectNotify 206
#define sipName_connectNotify &sipStrings_pictureflow[206]
#define sipNameNr_dragMoveEvent 440
#define sipName_dragMoveEvent &sipStrings_pictureflow[440]
#define sipNameNr_focusOutEvent 454
#define sipName_focusOutEvent &sipStrings_pictureflow[454]
#define sipNameNr_itemActivated 468
#define sipName_itemActivated &sipStrings_pictureflow[468]
#define sipNameNr_keyPressEvent 482
#define sipName_keyPressEvent &sipStrings_pictureflow[482]
#define sipNameNr_sharedPainter 496
#define sipName_sharedPainter &sipStrings_pictureflow[496]
#define sipNameNr_currentSlide 510
#define sipName_currentSlide &sipStrings_pictureflow[510]
#define sipNameNr_focusInEvent 523
#define sipName_focusInEvent &sipStrings_pictureflow[523]
#define sipNameNr_setSlideSize 536
#define sipName_setSlideSize &sipStrings_pictureflow[536]
#define sipNameNr_showPrevious 549
#define sipName_showPrevious &sipStrings_pictureflow[549]
#define sipNameNr_subtitleFont 562
#define sipName_subtitleFont &sipStrings_pictureflow[562]
#define sipNameNr_PictureFlow 575
#define sipName_PictureFlow &sipStrings_pictureflow[575]
#define sipNameNr_actionEvent 587
#define sipName_actionEvent &sipStrings_pictureflow[587]
#define sipNameNr_changeEvent 599
#define sipName_changeEvent &sipStrings_pictureflow[599]
#define sipNameNr_clearCaches 611
#define sipName_clearCaches &sipStrings_pictureflow[611]
#define sipNameNr_customEvent 623
#define sipName_customEvent &sipStrings_pictureflow[623]
#define sipNameNr_dataChanged 635
#define sipName_dataChanged &sipStrings_pictureflow[635]
#define sipNameNr_eventFilter 647
#define sipName_eventFilter &sipStrings_pictureflow[647]
#define sipNameNr_initPainter 659
#define sipName_initPainter &sipStrings_pictureflow[659]
#define sipNameNr_maxFontSize 671
#define sipName_maxFontSize &sipStrings_pictureflow[671]
#define sipNameNr_nativeEvent 683
#define sipName_nativeEvent &sipStrings_pictureflow[683]
#define sipNameNr_paintEngine 695
#define sipName_paintEngine &sipStrings_pictureflow[695]
#define sipNameNr_pictureflow 707
#define sipName_pictureflow &sipStrings_pictureflow[707]
#define sipNameNr_resizeEvent 719
#define sipName_resizeEvent &sipStrings_pictureflow[719]
#define sipNameNr_tabletEvent 731
#define sipName_tabletEvent &sipStrings_pictureflow[731]
#define sipNameNr_FlowImages 743
#define sipName_FlowImages &sipStrings_pictureflow[743]
#define sipNameNr_childEvent 754
#define sipName_childEvent &sipStrings_pictureflow[754]
#define sipNameNr_closeEvent 765
#define sipName_closeEvent &sipStrings_pictureflow[765]
#define sipNameNr_enterEvent 776
#define sipName_enterEvent &sipStrings_pictureflow[776]
#define sipNameNr_leaveEvent 787
#define sipName_leaveEvent &sipStrings_pictureflow[787]
#define sipNameNr_paintEvent 798
#define sipName_paintEvent &sipStrings_pictureflow[798]
#define sipNameNr_setVisible 809
#define sipName_setVisible &sipStrings_pictureflow[809]
#define sipNameNr_timerEvent 820
#define sipName_timerEvent &sipStrings_pictureflow[820]
#define sipNameNr_wheelEvent 831
#define sipName_wheelEvent &sipStrings_pictureflow[831]
#define sipNameNr_dropEvent 842
#define sipName_dropEvent &sipStrings_pictureflow[842]
#define sipNameNr_hideEvent 852
#define sipName_hideEvent &sipStrings_pictureflow[852]
#define sipNameNr_moveEvent 862
#define sipName_moveEvent &sipStrings_pictureflow[862]
#define sipNameNr_setImages 872
#define sipName_setImages &sipStrings_pictureflow[872]
#define sipNameNr_showEvent 882
#define sipName_showEvent &sipStrings_pictureflow[882]
#define sipNameNr_showSlide 892
#define sipName_showSlide &sipStrings_pictureflow[892]
#define sipNameNr_slideSize 902
#define sipName_slideSize &sipStrings_pictureflow[902]
#define sipNameNr_showNext 912
#define sipName_showNext &sipStrings_pictureflow[912]
#define sipNameNr_sizeHint 921
#define sipName_sizeHint &sipStrings_pictureflow[921]
#define sipNameNr_subtitle 930
#define sipName_subtitle &sipStrings_pictureflow[930]
#define sipNameNr_caption 939
#define sipName_caption &sipStrings_pictureflow[939]
#define sipNameNr_devType 947
#define sipName_devType &sipStrings_pictureflow[947]
#define sipNameNr_metric 955
#define sipName_metric &sipStrings_pictureflow[955]
#define sipNameNr_render 962
#define sipName_render &sipStrings_pictureflow[962]
#define sipNameNr_count 969
#define sipName_count &sipStrings_pictureflow[969]
#define sipNameNr_event 975
#define sipName_event &sipStrings_pictureflow[975]
#define sipNameNr_image 981
#define sipName_image &sipStrings_pictureflow[981]
#define sipNameNr_slide 987
#define sipName_slide &sipStrings_pictureflow[987]
#define sipNameNr_stop 993
#define sipName_stop &sipStrings_pictureflow[993]

#define sipMalloc                   sipAPI_pictureflow->api_malloc
#define sipFree                     sipAPI_pictureflow->api_free
#define sipBuildResult              sipAPI_pictureflow->api_build_result
#define sipCallMethod               sipAPI_pictureflow->api_call_method
#define sipCallProcedureMethod      sipAPI_pictureflow->api_call_procedure_method
#define sipCallErrorHandler         sipAPI_pictureflow->api_call_error_handler
#define sipParseResultEx            sipAPI_pictureflow->api_parse_result_ex
#define sipParseResult              sipAPI_pictureflow->api_parse_result
#define sipParseArgs                sipAPI_pictureflow->api_parse_args
#define sipParseKwdArgs             sipAPI_pictureflow->api_parse_kwd_args
#define sipParsePair                sipAPI_pictureflow->api_parse_pair
#define sipInstanceDestroyed        sipAPI_pictureflow->api_instance_destroyed
#define sipInstanceDestroyedEx      sipAPI_pictureflow->api_instance_destroyed_ex
#define sipConvertFromSequenceIndex sipAPI_pictureflow->api_convert_from_sequence_index
#define sipConvertFromSliceObject   sipAPI_pictureflow->api_convert_from_slice_object
#define sipConvertFromVoidPtr       sipAPI_pictureflow->api_convert_from_void_ptr
#define sipConvertToVoidPtr         sipAPI_pictureflow->api_convert_to_void_ptr
#define sipAddException             sipAPI_pictureflow->api_add_exception
#define sipNoFunction               sipAPI_pictureflow->api_no_function
#define sipNoMethod                 sipAPI_pictureflow->api_no_method
#define sipAbstractMethod           sipAPI_pictureflow->api_abstract_method
#define sipBadClass                 sipAPI_pictureflow->api_bad_class
#define sipBadCatcherResult         sipAPI_pictureflow->api_bad_catcher_result
#define sipBadCallableArg           sipAPI_pictureflow->api_bad_callable_arg
#define sipBadOperatorArg           sipAPI_pictureflow->api_bad_operator_arg
#define sipTrace                    sipAPI_pictureflow->api_trace
#define sipTransferBack             sipAPI_pictureflow->api_transfer_back
#define sipTransferTo               sipAPI_pictureflow->api_transfer_to
#define sipSimpleWrapper_Type       sipAPI_pictureflow->api_simplewrapper_type
#define sipWrapper_Type             sipAPI_pictureflow->api_wrapper_type
#define sipWrapperType_Type         sipAPI_pictureflow->api_wrappertype_type
#define sipVoidPtr_Type             sipAPI_pictureflow->api_voidptr_type
#define sipGetPyObject              sipAPI_pictureflow->api_get_pyobject
#define sipGetAddress               sipAPI_pictureflow->api_get_address
#define sipGetMixinAddress          sipAPI_pictureflow->api_get_mixin_address
#define sipGetCppPtr                sipAPI_pictureflow->api_get_cpp_ptr
#define sipGetComplexCppPtr         sipAPI_pictureflow->api_get_complex_cpp_ptr
#define sipCallHook                 sipAPI_pictureflow->api_call_hook
#define sipEndThread                sipAPI_pictureflow->api_end_thread
#define sipRaiseUnknownException    sipAPI_pictureflow->api_raise_unknown_exception
#define sipRaiseTypeException       sipAPI_pictureflow->api_raise_type_exception
#define sipBadLengthForSlice        sipAPI_pictureflow->api_bad_length_for_slice
#define sipAddTypeInstance          sipAPI_pictureflow->api_add_type_instance
#define sipPySlotExtend             sipAPI_pictureflow->api_pyslot_extend
#define sipAddDelayedDtor           sipAPI_pictureflow->api_add_delayed_dtor
#define sipCanConvertToType         sipAPI_pictureflow->api_can_convert_to_type
#define sipConvertToType            sipAPI_pictureflow->api_convert_to_type
#define sipForceConvertToType       sipAPI_pictureflow->api_force_convert_to_type
#define sipConvertToEnum            sipAPI_pictureflow->api_convert_to_enum
#define sipConvertToBool            sipAPI_pictureflow->api_convert_to_bool
#define sipReleaseType              sipAPI_pictureflow->api_release_type
#define sipConvertFromType          sipAPI_pictureflow->api_convert_from_type
#define sipConvertFromNewType       sipAPI_pictureflow->api_convert_from_new_type
#define sipConvertFromNewPyType     sipAPI_pictureflow->api_convert_from_new_pytype
#define sipConvertFromEnum          sipAPI_pictureflow->api_convert_from_enum
#define sipGetState                 sipAPI_pictureflow->api_get_state
#define sipExportSymbol             sipAPI_pictureflow->api_export_symbol
#define sipImportSymbol             sipAPI_pictureflow->api_import_symbol
#define sipFindType                 sipAPI_pictureflow->api_find_type
#define sipBytes_AsChar             sipAPI_pictureflow->api_bytes_as_char
#define sipBytes_AsString           sipAPI_pictureflow->api_bytes_as_string
#define sipString_AsASCIIChar       sipAPI_pictureflow->api_string_as_ascii_char
#define sipString_AsASCIIString     sipAPI_pictureflow->api_string_as_ascii_string
#define sipString_AsLatin1Char      sipAPI_pictureflow->api_string_as_latin1_char
#define sipString_AsLatin1String    sipAPI_pictureflow->api_string_as_latin1_string
#define sipString_AsUTF8Char        sipAPI_pictureflow->api_string_as_utf8_char
#define sipString_AsUTF8String      sipAPI_pictureflow->api_string_as_utf8_string
#define sipUnicode_AsWChar          sipAPI_pictureflow->api_unicode_as_wchar
#define sipUnicode_AsWString        sipAPI_pictureflow->api_unicode_as_wstring
#define sipConvertFromConstVoidPtr  sipAPI_pictureflow->api_convert_from_const_void_ptr
#define sipConvertFromVoidPtrAndSize    sipAPI_pictureflow->api_convert_from_void_ptr_and_size
#define sipConvertFromConstVoidPtrAndSize   sipAPI_pictureflow->api_convert_from_const_void_ptr_and_size
#define sipWrappedTypeName(wt)      ((wt)->wt_td->td_cname)
#define sipGetReference             sipAPI_pictureflow->api_get_reference
#define sipKeepReference            sipAPI_pictureflow->api_keep_reference
#define sipRegisterProxyResolver    sipAPI_pictureflow->api_register_proxy_resolver
#define sipRegisterPyType           sipAPI_pictureflow->api_register_py_type
#define sipTypeFromPyTypeObject     sipAPI_pictureflow->api_type_from_py_type_object
#define sipTypeScope                sipAPI_pictureflow->api_type_scope
#define sipResolveTypedef           sipAPI_pictureflow->api_resolve_typedef
#define sipRegisterAttributeGetter  sipAPI_pictureflow->api_register_attribute_getter
#define sipEnableAutoconversion     sipAPI_pictureflow->api_enable_autoconversion
#define sipInitMixin                sipAPI_pictureflow->api_init_mixin
#define sipExportModule             sipAPI_pictureflow->api_export_module
#define sipInitModule               sipAPI_pictureflow->api_init_module
#define sipGetInterpreter           sipAPI_pictureflow->api_get_interpreter
#define sipSetTypeUserData          sipAPI_pictureflow->api_set_type_user_data
#define sipGetTypeUserData          sipAPI_pictureflow->api_get_type_user_data
#define sipPyTypeDict               sipAPI_pictureflow->api_py_type_dict
#define sipPyTypeName               sipAPI_pictureflow->api_py_type_name
#define sipGetCFunction             sipAPI_pictureflow->api_get_c_function
#define sipGetMethod                sipAPI_pictureflow->api_get_method
#define sipFromMethod               sipAPI_pictureflow->api_from_method
#define sipGetDate                  sipAPI_pictureflow->api_get_date
#define sipFromDate                 sipAPI_pictureflow->api_from_date
#define sipGetDateTime              sipAPI_pictureflow->api_get_datetime
#define sipFromDateTime             sipAPI_pictureflow->api_from_datetime
#define sipGetTime                  sipAPI_pictureflow->api_get_time
#define sipFromTime                 sipAPI_pictureflow->api_from_time
#define sipIsUserType               sipAPI_pictureflow->api_is_user_type
#define sipCheckPluginForType       sipAPI_pictureflow->api_check_plugin_for_type
#define sipUnicodeNew               sipAPI_pictureflow->api_unicode_new
#define sipUnicodeWrite             sipAPI_pictureflow->api_unicode_write
#define sipUnicodeData              sipAPI_pictureflow->api_unicode_data
#define sipGetBufferInfo            sipAPI_pictureflow->api_get_buffer_info
#define sipReleaseBufferInfo        sipAPI_pictureflow->api_release_buffer_info
#define sipIsOwnedByPython          sipAPI_pictureflow->api_is_owned_by_python
#define sipIsDerivedClass           sipAPI_pictureflow->api_is_derived_class
#define sipGetUserObject            sipAPI_pictureflow->api_get_user_object
#define sipSetUserObject            sipAPI_pictureflow->api_set_user_object
#define sipRegisterEventHandler     sipAPI_pictureflow->api_register_event_handler
#define sipConvertToArray           sipAPI_pictureflow->api_convert_to_array
#define sipConvertToTypedArray      sipAPI_pictureflow->api_convert_to_typed_array
#define sipEnableGC                 sipAPI_pictureflow->api_enable_gc
#define sipPrintObject              sipAPI_pictureflow->api_print_object
#define sipLong_AsChar              sipAPI_pictureflow->api_long_as_char
#define sipLong_AsSignedChar        sipAPI_pictureflow->api_long_as_signed_char
#define sipLong_AsUnsignedChar      sipAPI_pictureflow->api_long_as_unsigned_char
#define sipLong_AsShort             sipAPI_pictureflow->api_long_as_short
#define sipLong_AsUnsignedShort     sipAPI_pictureflow->api_long_as_unsigned_short
#define sipLong_AsInt               sipAPI_pictureflow->api_long_as_int
#define sipLong_AsUnsignedInt       sipAPI_pictureflow->api_long_as_unsigned_int
#define sipLong_AsLong              sipAPI_pictureflow->api_long_as_long
#define sipLong_AsUnsignedLong      sipAPI_pictureflow->api_long_as_unsigned_long
#define sipLong_AsLongLong          sipAPI_pictureflow->api_long_as_long_long
#define sipLong_AsUnsignedLongLong  sipAPI_pictureflow->api_long_as_unsigned_long_long
#define sipLong_AsSizeT             sipAPI_pictureflow->api_long_as_size_t
#define sipVisitWrappers            sipAPI_pictureflow->api_visit_wrappers
#define sipRegisterExitNotifier     sipAPI_pictureflow->api_register_exit_notifier
#define sipDeprecated               sipAPI_pictureflow->api_deprecated
#define sipPyTypeDictRef            sipAPI_pictureflow->api_py_type_dict_ref
#define sipNextExceptionHandler     sipAPI_pictureflow->api_next_exception_handler
#define sipIsEnumFlag               sipAPI_pictureflow->api_is_enum_flag
#define sipConvertToTypeUS          sipAPI_pictureflow->api_convert_to_type_us
#define sipForceConvertToTypeUS     sipAPI_pictureflow->api_force_convert_to_type_us
#define sipReleaseTypeUS            sipAPI_pictureflow->api_release_type_us
#define sipIsPyMethod               sipAPI_pictureflow->api_is_py_method_12_8

/* The strings used by this module. */
extern const char sipStrings_pictureflow[];

#define sipType_FlowImages sipExportedTypes_pictureflow[0]

extern sipClassTypeDef sipTypeDef_pictureflow_FlowImages;

#define sipType_PictureFlow sipExportedTypes_pictureflow[1]

extern sipClassTypeDef sipTypeDef_pictureflow_PictureFlow;

/* The SIP API, this module's API and the APIs of any imported modules. */
extern const sipAPIDef *sipAPI_pictureflow;
extern sipExportedModuleDef sipModuleAPI_pictureflow;
extern sipTypeDef *sipExportedTypes_pictureflow[];

#define sipType_Qt_InputMethodQuery sipImportedTypes_pictureflow_QtCore[9].it_td

#define sipType_QObject sipImportedTypes_pictureflow_QtCore[4].it_td

#define sipType_QByteArray sipImportedTypes_pictureflow_QtCore[0].it_td

#define sipType_QEvent sipImportedTypes_pictureflow_QtCore[2].it_td

#define sipType_QTimerEvent sipImportedTypes_pictureflow_QtCore[7].it_td

#define sipType_QChildEvent sipImportedTypes_pictureflow_QtCore[1].it_td

#define sipType_QMetaMethod sipImportedTypes_pictureflow_QtCore[3].it_td

#define sipType_QSize sipImportedTypes_pictureflow_QtCore[5].it_td

#define sipType_QVariant sipImportedTypes_pictureflow_QtCore[8].it_td

#define sipType_QString sipImportedTypes_pictureflow_QtCore[6].it_td

#define sipType_qintptr sipImportedTypes_pictureflow_QtCore[10].it_td
extern sipImportedTypeDef sipImportedTypes_pictureflow_QtCore[];
extern sipImportedVirtErrorHandlerDef sipImportedVirtErrorHandlers_pictureflow_QtCore[];

#define sipType_QPaintDevice sipImportedTypes_pictureflow_QtGui[16].it_td

#define sipType_QPaintDevice_PaintDeviceMetric sipImportedTypes_pictureflow_QtGui[17].it_td

#define sipType_QKeyEvent sipImportedTypes_pictureflow_QtGui[13].it_td

#define sipType_QFocusEvent sipImportedTypes_pictureflow_QtGui[8].it_td

#define sipType_QPaintEvent sipImportedTypes_pictureflow_QtGui[19].it_td

#define sipType_QMoveEvent sipImportedTypes_pictureflow_QtGui[15].it_td

#define sipType_QResizeEvent sipImportedTypes_pictureflow_QtGui[21].it_td

#define sipType_QCloseEvent sipImportedTypes_pictureflow_QtGui[1].it_td

#define sipType_QShowEvent sipImportedTypes_pictureflow_QtGui[22].it_td

#define sipType_QHideEvent sipImportedTypes_pictureflow_QtGui[10].it_td

#define sipType_QContextMenuEvent sipImportedTypes_pictureflow_QtGui[2].it_td

#define sipType_QInputMethodEvent sipImportedTypes_pictureflow_QtGui[12].it_td

#define sipType_QDropEvent sipImportedTypes_pictureflow_QtGui[6].it_td

#define sipType_QDragMoveEvent sipImportedTypes_pictureflow_QtGui[5].it_td

#define sipType_QDragEnterEvent sipImportedTypes_pictureflow_QtGui[3].it_td

#define sipType_QDragLeaveEvent sipImportedTypes_pictureflow_QtGui[4].it_td

#define sipType_QActionEvent sipImportedTypes_pictureflow_QtGui[0].it_td

#define sipType_QEnterEvent sipImportedTypes_pictureflow_QtGui[7].it_td

#define sipType_QMouseEvent sipImportedTypes_pictureflow_QtGui[14].it_td

#define sipType_QWheelEvent sipImportedTypes_pictureflow_QtGui[24].it_td

#define sipType_QTabletEvent sipImportedTypes_pictureflow_QtGui[23].it_td

#define sipType_QFont sipImportedTypes_pictureflow_QtGui[9].it_td

#define sipType_QImage sipImportedTypes_pictureflow_QtGui[11].it_td

#define sipType_QPaintEngine sipImportedTypes_pictureflow_QtGui[18].it_td

#define sipType_QPainter sipImportedTypes_pictureflow_QtGui[20].it_td
extern sipImportedTypeDef sipImportedTypes_pictureflow_QtGui[];

#define sipType_QWidget sipImportedTypes_pictureflow_QtWidgets[0].it_td
extern sipImportedTypeDef sipImportedTypes_pictureflow_QtWidgets[];

typedef const QMetaObject *(*sip_qt_metaobject_func)(sipSimpleWrapper *, sipTypeDef *);
extern sip_qt_metaobject_func sip_pictureflow_qt_metaobject;

typedef int (*sip_qt_metacall_func)(sipSimpleWrapper *, sipTypeDef *, QMetaObject::Call, int, void **);
extern sip_qt_metacall_func sip_pictureflow_qt_metacall;

typedef bool (*sip_qt_metacast_func)(sipSimpleWrapper *, const sipTypeDef *, const char *, void **);
extern sip_qt_metacast_func sip_pictureflow_qt_metacast;

#endif
